#include "ATLAS_EXOT_2018_06.h"
#include "include/isolation.h"

#include <algorithm>


/*

ATLAS Monojet search
https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/EXOT-2018-06

*/



bool comppT(const HEP::Jet* v1, const HEP::Jet* v2)
{
  return (v1->pT() > v2->pT());
}





void ATLAS_EXOT_2018_06::init() {

this->DetectorFunction="ATLAS";
cout << "---------------------------------------------------" << endl;
cout << "-- Analysis ATLAS_EXOT_2018_06 --" << endl;
cout << "---------------------------------------------------" << endl;



 const std::vector<std::string> allSRs={"EM0","EM1","EM2","EM3","EM4","EM5","EM6","EM7","EM8","EM9","EM10","EM11","EM12","IM0","IM1","IM2","IM3","IM4","IM5","IM6","IM7","IM8","IM9","IM10","IM11","IM12"};
 
  // =====Declare the signal regions in this analysis===== 
 
AddRegionSelection("EM0"); 
AddRegionSelection("EM1"); 
AddRegionSelection("EM2"); 
AddRegionSelection("EM3"); 
AddRegionSelection("EM4"); 
AddRegionSelection("EM5"); 
AddRegionSelection("EM6"); 
AddRegionSelection("EM7"); 
AddRegionSelection("EM8"); 
AddRegionSelection("EM9"); 
AddRegionSelection("EM10"); 
AddRegionSelection("EM11"); 
AddRegionSelection("EM12"); 
   
   
  //===================================// 
 
 AddRegionSelection("IM0"); 
AddRegionSelection("IM1"); 
AddRegionSelection("IM2"); 
AddRegionSelection("IM3"); 
AddRegionSelection("IM4"); 
AddRegionSelection("IM5"); 
AddRegionSelection("IM6"); 
AddRegionSelection("IM7"); 
AddRegionSelection("IM8"); 
AddRegionSelection("IM9"); 
AddRegionSelection("IM10"); 
AddRegionSelection("IM11"); 
AddRegionSelection("IM12"); 
 

  // Same as ATLAS table  
 
 AddCut("MET > 150 GeV",allSRs);  
AddCut("Lepton Veto",allSRs); 
AddCut("Njets <= 4",allSRs); 
AddCut("dphijmet > 0.4(0.6)",allSRs); 
AddCut("Leading jet pT > 150 GeV",allSRs);  
AddCut("MET > 200 GeV",allSRs); 
 
  //======Signal Region Cuts====RegionEMs========== 
 
AddCut("200 < MET < 250 GeV", "EM0"); 
AddCut("250 < MET < 300 GeV", "EM1"); 
AddCut("300 < MET < 350 GeV", "EM2"); 
AddCut("350 < MET < 400 GeV", "EM3"); 
AddCut("400 < MET < 500 GeV", "EM4"); 
AddCut("500 < MET < 600 GeV", "EM5"); 
AddCut("600 < MET < 700 GeV", "EM6"); 
AddCut("700 < MET < 800 GeV", "EM7"); 
AddCut("800 < MET < 900 GeV", "EM8"); 
AddCut("900 < MET < 1000 GeV","EM9"); 
AddCut("1000 < MET < 1100 GeV","EM10"); 
AddCut("1100 < MET < 1200 GeV","EM11"); 
AddCut("MET > 1200 GeV","EM12"); 
 
  //===Signal Region Cuts==RegionIMs=============== 
 
AddCut("MET > 200 GeV(IM0)", "IM0"); 
AddCut("MET > 250 GeV(IM1)", "IM1"); 
AddCut("MET > 300 GeV(IM2)", "IM2"); 
AddCut("MET > 350 GeV(IM3)", "IM3"); 
AddCut("MET > 400 GeV(IM4)", "IM4"); 
AddCut("MET > 500 GeV(IM5)", "IM5"); 
AddCut("MET > 600 GeV(IM6)", "IM6"); 
AddCut("MET > 700 GeV(IM7)", "IM7"); 
AddCut("MET > 800 GeV(IM8)", "IM8"); 
AddCut("MET > 900 GeV(IM9)", "IM9"); 
AddCut("MET > 1000 GeV(IM10)", "IM10");  
AddCut("MET > 1100 GeV(IM11)", "IM11");  
AddCut("MET > 1200 GeV(IM12)", "IM12");  
 
 
}

ATLAS_EXOT_2018_06::~ATLAS_EXOT_2018_06() {

};

void ATLAS_EXOT_2018_06::Execute(std::mt19937 &engine) {

static std::uniform_real_distribution<double> rd(0.0,1.0);

// cout << "New event" << endl;
vector<const HEP::Jet*> bJets, SignalJets; 
  vector<const HEP::Particle*>SignalMuons,SignalElectrons; 
  vector<const HEP::Particle*>SignalTaus; 
  vector<const HEP::Particle*>SignalPhotons; 
 
  HEP::P4 pTmiss = Event->missingmom();
  double MET=pTmiss.pT();


  for(auto CurrentJet : Event->jets())
    {
      if ( CurrentJet->pT() > 30.0 && abs(CurrentJet->eta())<2.8) 
      {
        if (CurrentJet->btag())
        {
          SignalJets.push_back(CurrentJet);

        }
        else
        {
          bJets.push_back(CurrentJet);
        }
      }
      
    }


  for(auto CurrentTrack : Event->electrons())
    {
      double pt = CurrentTrack->pT(); 
      double abseta = CurrentTrack->abseta(); 
      double sin=fabs(pt/CurrentTrack->p()); 
      if(pt > 7. && abseta < 2.47 && sin < 0.5)

	{
	  double ptratio = sumpTisolation(CurrentTrack->mom(),Event,0.4);
	  if (ptratio < 0.2*pt) SignalElectrons.push_back(CurrentTrack); 
      } 

    }

  for(auto  CurrentTrack : Event->muons())
    {
      double pt = CurrentTrack->pT(); 
      double abseta = CurrentTrack->abseta(); 
      double sin=fabs(pt/CurrentTrack->p()); 
      if(pt > 7. && abseta < 2.5 && sin < 0.5){
	double ptratio = sumpTisolation(CurrentTrack->mom(),Event,0.4);
	if (ptratio < 0.2*pt) SignalMuons.push_back(CurrentTrack); 
	
      } 

    }

   
for(auto CurrentTrack : Event->taus())
    {
      double pt = CurrentTrack->pT(); 
      double abseta = CurrentTrack->abseta(); 
 
      if(pt > 20. && abseta < 2.5)
	{
	  double ptratio = sumpTisolation(CurrentTrack->mom(),Event,0.2);
	if (ptratio < 0.2*pt) SignalTaus.push_back(CurrentTrack); 
	 
      } 

    }


for(auto CurrentTrack : Event->photons())
    {
      double pt = CurrentTrack->pT(); 
      double abseta = CurrentTrack->abseta(); 
 
      if(pt > 10. && abseta < 2.37) SignalPhotons.push_back(CurrentTrack); 
      

    }

SignalJets = Removal(SignalJets, SignalElectrons,0.2);
SignalElectrons = Removal(SignalElectrons,bJets,0.4);
SignalElectrons = Removal(SignalElectrons,SignalJets,0.4);
SignalJets = Removal(SignalJets, SignalMuons,0.4);   // The muon is discarded if it is matched to a
// jet with 𝑝 T > 30 GeV that has at least three tracks with pT > 500 MeV associated with it. If the jet has < 3 tracks, the muon is removed
SignalMuons = Removal(SignalMuons,bJets,0.4);

for(auto bjet : bJets)
{
  SignalJets.push_back(bjet);
}


SignalTaus = Removal(SignalTaus, SignalElectrons, 0.2); 
  SignalTaus = Removal(SignalTaus, SignalMuons, 0.2); 
  SignalJets = Removal(SignalJets, SignalTaus, 0.2); 
 
  SignalPhotons = Removal(SignalPhotons, SignalElectrons, 0.4); 
  SignalPhotons = Removal(SignalPhotons, SignalMuons, 0.4); 
  //SignalPhotons = Removal(SignalPhotons, SignalJets, 0.2); 
  SignalJets = Removal(SignalJets, SignalPhotons, 0.4); 


// SORTER->sort(SignalJets, PTordering);

  std::sort(SignalJets.begin(),SignalJets.end(),comppT);

int njets=0;

/*
 for(auto jet : SignalJets)
  {
    njets++;
    cout << "jet " << njets << " pT: " << jet->pT() << endl;
  }
*/
  int  NumSignalJets = SignalJets.size() ;  






//   double DeltaPhiJet1 = 0.0; 
//    double DeltaPhiJet2 = 0.0 ; 
//    double DeltaPhiJet3 =0.0; 
//    double DeltaPhiJet4 =0.0; 
   
//   if( NumSignalJets > 0) 
//     {DeltaPhiJet1= fabs(SignalJets[0]->mom()->DeltaPhi(pTmiss)); 
//   DeltaPhiJet2=9999999.9; 
//   DeltaPhiJet3=9999999.9; 
//   DeltaPhiJet4=9999999.9; 
//   } 
 
//   if( NumSignalJets > 1) 
//   { 
//     DeltaPhiJet1= fabs(SignalJets[0]->DeltaPhi(pTmiss)); 
//   DeltaPhiJet2= SignalJets[1]->dphi_0_pi(pTmiss); 
//   DeltaPhiJet3=999999.9; 
//   DeltaPhiJet4=9999999.9; 
//   } 
 
// if( NumSignalJets > 2) 
// { 
// DeltaPhiJet1= SignalJets[0]->dphi_0_pi(pTmiss); 
// DeltaPhiJet2= SignalJets[1]->dphi_0_pi(pTmiss); 
// DeltaPhiJet3= SignalJets[2]->dphi_0_pi(pTmiss); 
// DeltaPhiJet4=9999999.9; 
// } 
 
// if( NumSignalJets > 3) 
// { 
// DeltaPhiJet1= SignalJets[0]->dphi_0_pi(pTmiss); 
// DeltaPhiJet2= SignalJets[1]->dphi_0_pi(pTmiss); 
// DeltaPhiJet3= SignalJets[2]->dphi_0_pi(pTmiss); 
// DeltaPhiJet4 = SignalJets[3]->dphi_0_pi(pTmiss); 
// } 



ApplyCut((MET > 150.),"MET > 150 GeV"); 
 
ApplyCut(((SignalElectrons.size())==0 && (SignalMuons.size())==0 && (SignalTaus.size())==0 && (SignalPhotons.size())==0) ,"Lepton Veto"); 
 
  //=====Three Jets at Most====//   
ApplyCut((NumSignalJets < 5),"Njets <= 4"); 

 if(NumSignalJets >4 || NumSignalJets ==0) return;

double DeltaPhiJet[4];

for(njets=0; njets<4; njets++)
  {
    DeltaPhiJet[njets] = 999999.9;
    
  }

njets=0;
for(auto jet : SignalJets)
  {
    DeltaPhiJet[njets] = fabs(jet->mom().deltaPhi(pTmiss));
    njets++;
     
  }

 
  //========DeltaPhi Cuts===========================// 
 
  // For the (0.6) in the case 200 < MET < 250 Gev 
  if (200 < MET  && MET < 250) 
  { 
ApplyCut((DeltaPhiJet[0] > 0.6) &&(DeltaPhiJet[1] > 0.6) && (DeltaPhiJet[2] > 0.6) && (DeltaPhiJet[3] > 0.6),"dphijmet > 0.4(0.6)"); 
  } 
  else  
  { 
ApplyCut((DeltaPhiJet[0] > 0.4) &&(DeltaPhiJet[1] > 0.4) && (DeltaPhiJet[2] > 0.4) && (DeltaPhiJet[3] > 0.4),"dphijmet > 0.4(0.6)"); 
  } 


  
  //=============Leading jet pT cut=================// 
  ApplyCut(( SignalJets[0]->pT() > 150 && abs(SignalJets[0]->mom().eta()) < 2.4 ),"Leading jet pT > 150 GeV"); 
 
ApplyCut((MET > 200.),"MET > 200 GeV"); 
 
 
 
  //=======Signal Region Specific Cuts==============// 
 
  //===Exclusive signal regions EM0-12, validated on ATLAS cutflows ====//// 
 
  //=====SREM0===// 
ApplyCut((MET > 200 && MET < 250),"200 < MET < 250 GeV"); 
  //=====SREM1===// 
ApplyCut((MET > 250 && MET < 300),"250 < MET < 300 GeV"); 
  //=====SREM2===// 
ApplyCut((MET > 300 && MET < 350),"300 < MET < 350 GeV"); 
  //====SREM3===// 
ApplyCut((MET > 350 && MET < 400),"350 < MET < 400 GeV"); 
  //====SREM4===// 
ApplyCut((MET > 400 && MET < 500),"400 < MET < 500 GeV"); 
  //=====SREM5====// 
ApplyCut((MET > 500 && MET < 600),"500 < MET < 600 GeV"); 
  //====SREM6====// 
ApplyCut((MET > 600 && MET < 700),"600 < MET < 700 GeV"); 
  //===SREM7=====// 
ApplyCut((MET > 700 && MET < 800 ),"700 < MET < 800 GeV"); 
  //===SREM8=====// 
ApplyCut((MET > 800 && MET < 900 ),"800 < MET < 900 GeV"); 
  //===SREM9=====// 
ApplyCut((MET > 900 && MET < 1000 ),"900 < MET < 1000 GeV"); 
  //===SREM10=====// 
ApplyCut((MET > 1000 && MET < 1100 ),"1000 < MET < 1100 GeV"); 
  //===SREM11=====// 
ApplyCut((MET > 1100 && MET < 1200 ),"1100 < MET < 1200 GeV"); 
  //===SREM12=====// 
ApplyCut((MET > 1200 ),"MET > 1200 GeV"); 
  
   
   
  //===============================================================================// 
 
  //=====SRIM0===// 
ApplyCut((MET > 200 ),"MET > 200 GeV(IM0)"); 
  //=====SRIM1===// 
ApplyCut((MET > 250 ),"MET > 250 GeV(IM1)"); 
  //=====SRIM2===// 
ApplyCut((MET > 300 ),"MET > 300 GeV(IM2)"); 
  //=====SRIM3===// 
ApplyCut((MET > 350),"MET > 350 GeV(IM3)"); 
  //=====SRIM4===// 
ApplyCut((MET > 400 ),"MET > 400 GeV(IM4)"); 
  //=====SRIM5===// 
ApplyCut((MET > 500 ),"MET > 500 GeV(IM5)"); 
  //=====SRIM6===// 
ApplyCut((MET > 600 ),"MET > 600 GeV(IM6)"); 
  //=====SRIM7===// 
ApplyCut((MET > 700 ),"MET > 700 GeV(IM7)"); 
  //=====SRIM8===// 
ApplyCut((MET > 800 ),"MET > 800 GeV(IM8)"); 
  //=====SRIM9===// 
ApplyCut((MET > 900 ),"MET > 900 GeV(IM9)"); 
  //=====SRIM10===// 
ApplyCut((MET > 1000 ),"MET > 1000 GeV(IM10)"); 
  //=====SRIM11===// 
ApplyCut((MET > 1100 ),"MET > 1100 GeV(IM11)"); 
  //=====SRIM12===// 
ApplyCut((MET > 1200 ),"MET > 1200 GeV(IM12)"); 
 

// cout << "end event" << endl;



}

