#include "ATLAS_SUSY_2017_04_3body.h"

//#include <chrono>


//static std::uniform_real_distribution<double> rd(0.0,1.0);

// Define a global variable to hold the number of different types of vertices
// these will be triggered in the fit. 



void ATLAS_SUSY_2017_04_3body::update_event_weight(vector<dv_candidate> & dv_candidates)
{
	 // This will reweight 
	 if(USE_REWEIGHT)
	{
	  double initialweight=Event->weight();

	  double tweight=1.0;

	  for(auto dv : dv_candidates)
	    {
	      tweight*=(1.0 - dv.weight_multiplier);
	    }

	  this->set_weight(initialweight*(1.0-tweight));
	}

}



void ATLAS_SUSY_2017_04_3body::ApplyCuts(bool condition,const std::string& cutname)
{
	this->ApplyCut(condition,cutname);

	if(!USE_REWEIGHT)
	{		
			this->ApplyCut(condition&&found_ee,cutname+"_ee");
			this->ApplyCut(condition&&found_emu,cutname+"_emu");
			this->ApplyCut(condition&&found_mumu,cutname+"_mumu");
	}

}


void ATLAS_SUSY_2017_04_3body::AddCuts(const std::string& cutname)
{
	this->AddCut(cutname,"SR");

	if(!USE_REWEIGHT)
	{		
			this->AddCut(cutname+"_ee","SR_ee");
			this->AddCut(cutname+"_emu","SR_emu");
			this->AddCut(cutname+"_mumu","SR_mumu");
	}

}



void ATLAS_SUSY_2017_04_3body::do_DV_fit (vector<dv_candidate> &DV_candidates, std::mt19937 &engine) {
	
    vector<dv_candidate> new_candidates;


	for (auto dv : DV_candidates) {
		bool match=false;
		for(auto part: dv.particles)
		{
			for(int j=0; j< new_candidates.size(); j++)
			{
				if(fullMatch(part,new_candidates[j].particles[0]))
				{
					new_candidates[j].add_particle(part);
					match=true;
				}

			}

		}
		if (!match)
		{
	    	new_candidates.push_back(dv);
		}
    }

	 DV_candidates = new_candidates;
	
		double rdv1=0.0;
  double rdv2=0.0;
  for(auto dv: DV_candidates)
  {
	  double r = dv.get_position().pT();
	  if(r > rdv1) 
	  {
		  rdv2=rdv1;
		  rdv1 = r;

	  }
	  else
	  {
		  if(r > rdv2) rdv2 = r;

	  }

  }
  /*
	if((DV_candidates.size() > 0 )&& (rdv1 < 300.0))
	{
		
		this->FillYodaHisto1D("R_DV_good",rdv1);
		
	}
	if((DV_candidates.size() > 1) && (rdv2 < 300.0))
	{
		this->FillYodaHisto1D("R_DV_good",rdv2);
		// Fill the second histo with the larger radius point: this is only filled if we have at least one "good" vertex
		this->FillYodaHisto1D("R_DV2",rdv1);
	}
  */
	/*
    for (int i=0; i<DV_candidates.size(); i++) {
	// check if the velecity vector indicates that the radius becomes smaller,
	// which would cause hits in pixel layers at smaller radius or on the opposite side of the barrel.
	// we use the 3-dim. scalar product between production vertex and momentum to identify these cases.
	// MAdouble64 sProd = DV_candidates[i].particles[0]->momentum().X()*DV_candidates[i].particles[0]->momentum();
	
	// 
	bool match = false;
	for (int j=0; j<new_candidates.size(); j++) {
	    if (vertexMatch(DV_candidates[i].get_position(),new_candidates[j].get_position())){
		//new_candidates[j].add_particle(DV_candidates[i].particles[0]);
		for(auto part : DV_candidates[i].particles)
		{
			new_candidates[j].add_particle(part);
		}
		match = true;
		break;
	    }
	}
	if (!match)
	    new_candidates.push_back(DV_candidates[i]);
    }
	*/

    
    
    new_candidates.clear();
	found_ee=false;
	found_emu=false;
	found_mumu=false;
	//cout << DV_candidates.size() << " dvs found " <<endl;
	for(auto dv : DV_candidates)
	{
		// keep only DV candidates which have at least two particles
		if (dv.particles.size() < 2) continue;

		//if (dv.particles.size() > 2) dv.sort_reverse_pt();
		double r_xy = dv.d_xy();
		HEP::P4 vert_mom=dv.get_momentum();
		double pt=vert_mom.pT();
		double m_inv = vert_mom.m();
		//double pt = dv.get_pt();
		//double m_inv = dv.get_momentum().m();

		int nelec=0;
		int nmuon=0;
		for(int i=0; i < dv.particles.size(); i++) 
		{

			int apid = abs(dv.particles[i]->pid());
			if(apid == 11) nelec++;
			if(apid == 13) nmuon++;

		}
		//cout << nelec <<", " <<nmuon << endl;
		double prob=1.0;
		if(nelec ==2) 
		{
		  //if(m_inv < 500.0) prob = DVRecoEfficiencyZPrime100GeVToEE (r_xy,pt);
		  //else prob = DVRecoEfficiencyZPrime1000GeVToEE (r_xy,pt);
		  prob = DVRecoEfficiencyRPVEE (r_xy,m_inv,pt);
		  found_ee=true;
		  //else prob = DVRecoEfficiencyZPrime1000GeVToEE (r_xy,pt);
		  
		}
		else if(nmuon ==2)
		{
		  //if(m_inv < 500.0) prob = DVRecoEfficiencyZPrime100GeVToMUMU (r_xy,pt);
		  //else prob = DVRecoEfficiencyZPrime1000GeVToMUMU (r_xy,pt);
			prob = DVRecoEfficiencyRPVMUMU (r_xy,m_inv,pt);
			found_mumu=true;
		}
		else if((nelec ==1) && (nmuon ==1))
		{
		  //if(m_inv < 500.0) prob = DVRecoEfficiencyZPrime100GeVToEMU (r_xy,pt);
		  //else prob = DVRecoEfficiencyZPrime1000GeVToEMU (r_xy,pt);
			prob = DVRecoEfficiencyRPVEMU (r_xy,m_inv,pt);
			found_emu=true;
		}
		else 
		{
			// essentially a pileup vertex
			double dz = fabs(dv.get_position().pz());
			if((r_xy > 300.0) || (dz > 300.0)) prob = 0.9; 
			else prob = 1.0;
		}
		//if (rd(engine) < prob)  new_candidates.push_back(dv);
		if(prob > 0.0)
		{
			if(USE_REWEIGHT)
			{
				// For reweighting instead -- which doesn't work at present for different SRs
				dv.weight_multiplier=prob;
				new_candidates.push_back(dv);
			}
			else
			{
				if (rd(engine) < prob) new_candidates.push_back(dv);
			}
			
		}

	}

	
    //for (int i=0; i<DV_candidates.size(); i++) {
	//if (DV_candidates[i].particles.size() >=2) {

	// try the zprime vertex reco efficiencies
	//  double rnd = rd(engine);
	//  double prob = 1.0;
	//double r_xy = sqrt(
	//    pow(DV_candidates[i].get_position().px(),2)+
	//    pow(DV_candidates[i].get_position().py(),2));
	//double pt = DV_candidates[i].get_pt();



	
	/*
	switch (zprime_case)
	{
	case 1:
	    prob = DVRecoEfficiencyZPrime100GeVToEE (r_xy,pt);
	    break;
	case 2:
	    prob = DVRecoEfficiencyZPrime100GeVToEMU (r_xy,pt); 
	    break;
	case 3:
	    prob = DVRecoEfficiencyZPrime100GeVToMUMU (r_xy,pt); 
	    break;
	case 4:
	    prob = DVRecoEfficiencyZPrime1000GeVToEE (r_xy,pt);
	    break;
	case 5:
	    prob = DVRecoEfficiencyZPrime1000GeVToEMU (r_xy,pt); 
	    break;
	case 6:
	    prob = DVRecoEfficiencyZPrime1000GeVToMUMU (r_xy,pt); 
	    break;
	default:
	    prob = 1;
	}
	*/
	//if (rnd < prob)
	//    new_candidates.push_back(DV_candidates[i]);
	//}
	
    
    DV_candidates=new_candidates;
}



// // loose photon isolation requirement: 1810.05087, page 10
// bool is_loose_photon (const HEP::Particle *photon, const HEP::Event* event) {
// 	//return true;
//     double et = 0;
//     double pt = 0;
//     bool isolated;

//     et=sumETIsolation(photon->mom(),event,0.2);
//     pt=sumpTisolation(photon->mom(),event,0.2);

//     isolated = (et < 0.065 * photon->pT()) && (pt < 0.05 * photon->pT());
    
  
//     //isolated = (et < 0.065 * myPhoton->momentum().Et()) &&
//     //	(pt < 0.05 * myPhoton->momentum().Et());
//     return isolated;
// }




void ATLAS_SUSY_2017_04_3body::init() {

cout << "---------------------------------------------------" << endl;
cout << "-- Analysis ATLAS_SUSY_2017_04_3body --" << endl;
cout << "---------------------------------------------------" << endl;

  const std::vector<std::string> allSRs={"SR","SR_ee","SR_emu","SR_mumu"};

  const std::vector<std::string> append_strs={"","_ee","_emu","_mumu"};	

  const static vector<double> goodedges={0,2,22,38,73,111,145,300};
  const static vector<double> binedges={0,2,22,38,73,111,145,300,500,750,1000,1200,1500,2000,3000,4000,5000,7500,10000,25000};	

  //AddYodaHisto1D("R_DV_good",goodedges);
  //AddYodaHisto1D("R_DV2",binedges);

  // Only one signal region
  if(!USE_REWEIGHT)
  {
	for(std::string region : allSRs)
      {
	AddRegionSelection(region);
      }

	  AddCut("ee","SR_ee");
  	AddCut("emu","SR_emu");
 	 AddCut("mumu","SR_mumu");

  }
  else
  {
	  AddRegionSelection("SR");
  }
  //AddRegionSelection("SR");

// Event level requirements
  AddCuts("Triggers");
  AddCuts("Cosmic-ray veto");
  AddCuts("Primary vertex");
  AddCuts("N(DV)>=1");
  
  // Vertex level requirements
  AddCuts("Vertex fit");
  AddCuts("d_xy");
  AddCuts("Fiducial volume");
  AddCuts("Dis. pixel mod. veto");
  AddCuts("Material veto");
  AddCuts("N(l)>=1");
  AddCuts("N(l)>=2");
  AddCuts("Lepton kinematics");
  AddCuts("Lepton identification");
  AddCuts("Overlap removal");
  AddCuts("Trigger matching");
  AddCuts("Presel. matching");  
  AddCuts("m_DV");
  AddCuts("Opposite charge");
  

}




ATLAS_SUSY_2017_04_3body::~ATLAS_SUSY_2017_04_3body() {

};

void ATLAS_SUSY_2017_04_3body::Execute(std::mt19937 &engine) {

	found_ee=false;
	found_emu=false;
	found_mumu=false;

    double myWeight=Event->weight();

	
	//cout << "weight: " << myWeight << std::endl;
	if(!(myWeight >0.)) return;

	double MET=Event->met();

	// In RPV case large MET carried away by neutrinos
	/*
	if(MET > 100.0)
	{
		cout << "MET! Look at invisible stuff: ";
		for(auto nu : Event->invisible_particles())
		{
			cout << nu->mom() << " [ " << nu->pid() << " ] ";
		}
		cout << endl;

	}
	*/
	/*
	if(MET > 10.0)
	{
		cout << "MET! " << MET << " ";
		for (auto myElec: Event->electrons())
	 	 {
		  // First check to see whether we reconstruct an electron
		  
		  if(myElec->prod_vertex().pT() > 100.0)
		  {
			  cout << "e: (" << myElec->prod_vertex().pT() << ", " << myElec->prod_vertex().pz()<<", " << myElec->prod_vertex().phi()<< ") ";
		  }

		}

		for (auto myElec: Event->muons())
	 	 {
		  // First check to see whether we reconstruct an electron
		  
		  if(myElec->prod_vertex().pT() > 100.0)
		  {
			  cout << "mu: (" << myElec->prod_vertex().pT() << ", " << myElec->prod_vertex().pz()<<", " << myElec->prod_vertex().phi()<< ") ";
		  }
		  //}

		}
		cout << endl;

	}
	else
	{
		cout << "No MET!" << endl;
	}
    */
	// DEBUGGING (determine Zprime decay mode to apply model-dependent DV reco efficiencies)
	bool EE100 = false;
	bool EMU100 = false;
	bool MUMU100 = false;
	bool EE1000 = false;
	bool EMU1000 = false;
	bool MUMU1000 = false;

	
	/*
	for (int i=0; i< event.mc()->particles().size(); i++)
	    if (event.mc()->particles()[i].pdgid() == 32 && event.mc()->particles()[i].daughters().size() == 2) {
		EE100 = (fabs(event.mc()->particles()[i].daughters()[0]->pdgid()) == 11) &&
		    (fabs(event.mc()->particles()[i].daughters()[1]->pdgid()) == 11) &&
		    (event.mc()->particles()[i].m() == 100);
		EMU100 = (((fabs(event.mc()->particles()[i].daughters()[0]->pdgid()) == 11) &&
			   (fabs(event.mc()->particles()[i].daughters()[1]->pdgid()) == 13)) ||
			  ((fabs(event.mc()->particles()[i].daughters()[0]->pdgid()) == 13) &&
			   (fabs(event.mc()->particles()[i].daughters()[1]->pdgid()) == 11))) &&
		    (event.mc()->particles()[i].m() == 100);
		MUMU100 = (fabs(event.mc()->particles()[i].daughters()[0]->pdgid()) == 13) &&
		    (fabs(event.mc()->particles()[i].daughters()[1]->pdgid()) == 13) &&
		    (event.mc()->particles()[i].m() == 100);
		EE1000 = (fabs(event.mc()->particles()[i].daughters()[0]->pdgid()) == 11) &&
		    (fabs(event.mc()->particles()[i].daughters()[1]->pdgid()) == 11) &&
		    (event.mc()->particles()[i].m() == 1000);
		EMU100 = (((fabs(event.mc()->particles()[i].daughters()[0]->pdgid()) == 11) &&
			   (fabs(event.mc()->particles()[i].daughters()[1]->pdgid()) == 13)) ||
			  ((fabs(event.mc()->particles()[i].daughters()[0]->pdgid()) == 13) &&
			   (fabs(event.mc()->particles()[i].daughters()[1]->pdgid()) == 11))) &&
		    (event.mc()->particles()[i].m() == 1000);
		MUMU1000 = (fabs(event.mc()->particles()[i].daughters()[0]->pdgid()) == 13) &&
		    (fabs(event.mc()->particles()[i].daughters()[1]->pdgid()) == 13) &&
		    (event.mc()->particles()[i].m() == 1000);
		break;
	    }
	*/
	    
	vector<dv_candidate> DV_candidates;
	DV_candidates.clear();
	//vector<HEP::Particle*> disp_leptons;


	
	// counters for trigger requirements
	int NElectrons150GeV = 0;     // [ pt>150GeV         and |eta|<2.5 and |d0|>2.0  ]
	int NElectrons55GeVDispl = 0; // [ 150GeV>=pt>55GeV  and |eta|<2.5 and |d0|>2.0  ]
	int NElectrons55GeV = 0;      // [ 150GeV>=pt>55GeV  and |eta|<2.5 and |d0|<=2.0 ]
	int NElectrons10GeV = 0;      // [ 55GeV>=pt>10GeV   and |eta|<2.5 and |d0|>2.0  ]


	int n_single_photon = 0;
	int n_di_photon = 0;

	int dvelectrons=0;
	int dvmuons=0;
	
	// Fill the electron container
	//for (MAuint32 i=0;i<event.rec()->electrons().size();i++)
	//{
	//	const RecLeptonFormat *myElec = &(event.rec()->electrons()[i]);

	//	auto start_fill = std::chrono::high_resolution_clock::now();
  
    /// Stuff for signal region selection: define vectors of production vertices for
	// electrons and muons

	vector<HEP::P4> electron_positions;
	vector<HEP::P4> muon_positions;

	/////// 

	for(auto myElec : Event->electrons())
	  {
		  // First check to see whether we reconstruct an electron
		  
		  if(myElec->prod_vertex().pT() > 0.1)
		  {
		  	electron_positions.push_back(myElec->prod_vertex());
		  }

		  if((myElec->prod_vertex().pT() > 1150.0) || (fabs(myElec->prod_vertex().pz())> 3511.0)) continue; 
		  //if((myElec->prod_vertex().pT() > 300.0) || (fabs(myElec->prod_vertex().pz())>300.0)) continue; 
	    double pt = myElec->pT();
	    double abseta = myElec->abseta();
	    double absd0 = d0Calc(myElec->prod_vertex(),myElec->mom());
	    double absdz = dzCalc(myElec->prod_vertex(),myElec->mom());
	    /*
	    MAdouble64 pt = myElec->pt();
	    MAdouble64 abseta =fabs(myElec->eta());
	    MAdouble64 absd0 = fabs(myElec->d0());
	    MAdouble64 absdz = fabs(myElec->dz());
	    Manager()->FillHisto("pt_electron",pt);
	    */
	    // Trigger counters
	    if (abseta < 2.47) {
		if (pt > 140.) n_single_photon++;
		if (pt > 50.) n_di_photon++;
	    }

	    // counters for trigger requirements
	    if (abseta < 2.5) {
		if (absd0 > 2.0) {
		    if (pt > 150.) NElectrons150GeV++;
		    else if (pt > 55.) NElectrons55GeVDispl++;
		    else if (pt > 10.) NElectrons10GeV++;
		}
		else if (pt > 55.) NElectrons55GeV++;
	    }

	    if (absd0 > 2. && absd0 < 300. && pt > 1. && absdz < 1500. &&
		abseta < 5.) {
	      dv_candidate new_vertex(myElec);
		dvelectrons++;
		DV_candidates.push_back(new_vertex);
		//disp_leptons.push_back(myElec);
		
	    }
	}

	

	// counters for trigger requirements
	int NMuons62GeV = 0; // [ pt>62GeV and |eta|<1.07 and |d0| > 1.5 mm ]
	int NMuons10GeV = 0; // [ pt>10GeV and |eta|<2.5  and |d0| > 1.5 mm ]
	// check the d0 criterion

	bool muon_trigger = false;
	// Fill the muon container
	/*
	for (MAuint32 i=0;i<event.rec()->muons().size();i++)
	{
	    const RecLeptonFormat *myMuon = &(event.rec()->muons()[i]);
	    MAdouble64 pt = myMuon->pt();
	    MAdouble64 abseta =fabs(myMuon->eta());
	    MAdouble64 absd0 = fabs(myMuon->d0());
	    MAdouble64 absdz = fabs(myMuon->dz());
	    Manager()->FillHisto("pt_muon",pt);
	*/
	for(auto myMuon : Event->muons())
	  {
		  
		  if(myMuon->prod_vertex().pT() > 0.1)
		  {
		  	muon_positions.push_back(myMuon->prod_vertex());
		  }
		  if((myMuon->prod_vertex().pT() > 1150.0) || (fabs(myMuon->prod_vertex().pz())> 3511.0)) continue; 
		  if((myMuon->prod_vertex().pT() > 300.0) || (fabs(myMuon->prod_vertex().pz())> 300.0)) continue; 
	    double pt = myMuon->pT();
	    double abseta = myMuon->abseta();
	    double absd0 = d0Calc(myMuon->prod_vertex(),myMuon->mom());
	    double absdz = dzCalc(myMuon->prod_vertex(),myMuon->mom());
	
	
	    // trigger requirements
	    if (absd0 > 1.5 && pt > 60 && abseta < 1.05)
	    	muon_trigger = true;

	    // counters for preselection
	    if (absd0 > 1.5) {
		if (pt > 62. && abseta < 1.07) NMuons62GeV++;
		if (pt > 10. && abseta < 2.5) NMuons10GeV++;
	    }
	    
	    // DV reconstruction criteria
	    if (absd0 > 2. && absd0 < 300. && pt > 1. && absdz < 1500. &&
		abseta < 5. ) {
	      dv_candidate new_vertex(myMuon);
		dvmuons++;
		DV_candidates.push_back(new_vertex);
		//disp_leptons.push_back(myMuon);
		
	    }	    
	}
	/*
	for(auto tVertex : Event->get_hadron_vertices())
	{
		dv_candidate new_vertex;
		for(auto myTrack : tVertex->particles)
		{
			double pt = myTrack->pT();
	    	double abseta = myTrack->abseta();
	    	double absd0 = d0Calc(myTrack->prod_vertex(),myTrack->mom());
	    	double absdz = dzCalc(myTrack->prod_vertex(),myTrack->mom());
			if (absd0 > 2. && absd0 < 300. && pt > 1. && absdz < 1500. &&
				abseta < 5. ) {
	      		new_vertex.particles.push_back(myTrack);
				}
		

		}
		if(new_vertex.particles.size() > 0)
		{
			DV_candidates.push_back(new_vertex);
		}


	}
	*/
	//	auto end_fill = std::chrono::high_resolution_clock::now();

	//  std::chrono::duration<double> fill_diff = std::chrono::duration_cast<std::chrono::duration<double>>(end_fill -start_fill);
  //double timediff = difftime(endtime, starttime) * 1000.0;

	// cout.precision(10);
  // cout << "fill took " << fill_diff.count() << std::endl;

  //	cout << "Starting with candidate tracks: " << DV_candidates.size() << std::endl;

  /*     UNCOMMENT THIS TO ADD CHARGED HADRONS
  
	for(auto myTrack : Event->get_charged_hadrons())
	{
		double pt = myTrack->pT();
	    double abseta = myTrack->abseta();
	    double absd0 = d0Calc(myTrack->prod_vertex(),myTrack->mom());
	    double absdz = dzCalc(myTrack->prod_vertex(),myTrack->mom());
		if (absd0 > 2. && absd0 < 300. && pt > 1. && absdz < 1500. &&
		abseta < 5. ) {
	      dv_candidate new_vertex(myTrack);
		DV_candidates.push_back(new_vertex);
		//disp_leptons.push_back(myMuon);
		
	    }	    

	}
	
  */
	// counters for preselection requirements
	int NPhotons150GeV = 0; // [ pt>150GeV        and |eta|<2.5 ]
	int NPhotons55GeV = 0;  // [ 150GeV>=pt>55GeV and |eta|<2.5 ]
	int NPhotons10GeV = 0;  // [ 55GeV>=pt>10GeV  and |eta|<2.5 ]

	/*
	// Debugging: check population of photons in event
	for (MAuint32 i=0;i<event.rec()->photons().size();i++)
	{
	    const RecPhotonFormat *myPhoton = &(event.rec()->photons()[i]);
	    MAdouble64 pt = myPhoton->pt();
	    MAdouble64 abseta = fabs(myPhoton->eta());
	    MAdouble64 absd0 = fabs(myPhoton->d0());
	    MAdouble64 absdz = fabs(myPhoton->dz());
	    Manager()->FillHisto("pt_gamma",pt);
	*/
	for(auto myPhoton : Event->photons())
	  {
	    double pt = myPhoton->pT();
		if(pt < 10.0) continue;

		// photon produced at least before end of ECAL
		 if((myPhoton->prod_vertex().pT() > 4000.0) || (fabs(myPhoton->prod_vertex().pz())> 3200.0)) continue; 
		
	    double abseta = myPhoton->abseta();
	    //double absd0 = d0Calc(my->prod_vertex(),myMuon->mom());
	    //double absdz = dzCalc(myMuon->prod_vertex(),myMuon->mom());
	
	    if (!is_loose_photon(myPhoton,Event)) continue;
	    // trigger requirements
	    if (abseta < 2.47) {
		if (pt > 140.) n_single_photon++;
		if (pt > 50.) n_di_photon++;
	    }

	    // counters for preselection
	    if (abseta < 2.5) {
		if (pt > 150.) NPhotons150GeV++;
		else if (pt > 55.) NPhotons55GeV++;
		else if (pt > 10.) NPhotons10GeV++;
	    }
	}


	bool single_photon_trigger = (n_single_photon >= 1);
	bool di_photon_trigger = (n_di_photon >= 2);
	bool trigger = muon_trigger || single_photon_trigger || di_photon_trigger;

	
	

	
	
	// DEBUGGING: used to apply the correct efficiencies corresponding
	// to the Z' samples (depends on mass and decay mode)
	// MAint32 zprime_case;
	// if(EE100) zprime_case = 1;
	// else if (EMU100) zprime_case = 2;
	// else if (MUMU100) zprime_case = 3;
	// else if (EE1000) zprime_case = 4;
	// else if (EMU1000) zprime_case = 5;
	// else if (MUMU1000) zprime_case = 6;

	// check preselection cuts
	// MAbool preselected = false;
	// if (muon_trigger) {
	//     if (NMuons62GeV >= 1) preselected = true;
	// }
	// if (single_photon_trigger) {
	//     // single photon lines 2-4
	//     if (NPhotons150GeV >= 1 &&
	// 	((NPhotons150GeV - 1 // subtract candidate 1
	// 	  + NPhotons55GeV + NPhotons10GeV
	// 	  + NElectrons150GeV + NElectrons55GeVDispl + NElectrons10GeV
	// 	  + NMuons10GeV) >= 1)) {
	// 	preselected = true;
	//     }
	//     // single photon line 5
	//     else if (NElectrons150GeV >= 1)
	// 	preselected = true;
	// }
	// if (di_photon_trigger) {
	//     // diphoton line 6
	//     if ((NPhotons55GeV + NPhotons150GeV) >= 2)
	// 	preselected = true;
	//     // diphoton line 7
	//     else if ((NElectrons150GeV + NElectrons55GeVDispl) >= 2)
	// 	preselected = true;
	//     // diphoton lines 8+9
	//     else if ((NElectrons150GeV + NElectrons55GeVDispl + NElectrons55GeV) >=1 &&
	// 	     (NPhotons150GeV + NPhotons55GeV) >=1)
	// 	preselected = true;
	// }
    
	//if (!Manager()->ApplyCut(trigger,"Triggers"))
	//    return true;



	
	/*
	if(found_ee) cout << "found ee ";
	if(found_emu) cout << "found emu ";
	if(found_mumu) cout << "found mumu";
	cout << endl;*/

	// Now we match electons & muons to vertices to see what types we have

	if(!USE_REWEIGHT)
	{
	for(int ii=0; ii<electron_positions.size(); ii++)
	{
		for(int jj=0; jj < ii; jj++)
		{
			if(vertexMatch(electron_positions[ii],electron_positions[jj]))
			{
				found_ee=true;
				break;
			}
		}
		for(int jj=0; jj < muon_positions.size(); jj++)
		{
			if(vertexMatch(electron_positions[ii],muon_positions[jj]))
			{
				found_emu=true;
				break;
			}
		}

	}

	for(int ii=0; ii<muon_positions.size(); ii++)
	{
		for(int jj=0; jj < ii; jj++)
		{
			if(vertexMatch(muon_positions[ii],muon_positions[jj]))
			{
				found_mumu=true;
				break;
			}
		}
	}

	ApplyCut(found_ee,"ee");
	ApplyCut(found_emu,"emu");
	ApplyCut(found_mumu,"mumu");
	}


	ApplyCuts(trigger,"Triggers");
	/*
	ApplyCut(trigger&&found_ee,"Triggers_ee");
	ApplyCut(trigger&&found_emu,"Triggers_emu");
	ApplyCut(trigger&&found_mumu,"Triggers_mumu");
	*/
	// reset these variables since we check again during the DV matching. 
	
	//	auto end_trigger = std::chrono::high_resolution_clock::now();
	//	 std::chrono::duration<double> trigger_diff = std::chrono::duration_cast<std::chrono::duration<double>>(end_trigger-end_fill);
  //double timediff = difftime(endtime, starttime) * 1000.0;

  //cout.precision(10);
  //  cout << "trigger took " << trigger_diff.count() << std::endl;
	//cout << "triggers" << std::endl;
	if(!trigger) return;
	bool cr_veto = cosmicRayVeto (Event->electrons(),Event->muons());
	//bool cr_veto=true;
	ApplyCuts(!cr_veto,"Cosmic-ray veto");
	/*
	ApplyCut((!cr_veto)&&found_ee,"Cosmic-ray veto_ee");
	ApplyCut((!cr_veto)&&found_emu,"Cosmic-ray veto_emu");
	ApplyCut((!cr_veto)&&found_mumu,"Cosmic-ray veto_mumu");
	*/
	if (cr_veto) return;
	// Not implemented
	//ApplyCut(DV_candidates.size()>0,"Primary vertex");
	ApplyCuts(true,"Primary vertex");
	/*
	ApplyCut(true,"Primary vertex_ee");
	ApplyCut(true,"Primary vertex_emu");
	ApplyCut(true,"Primary vertex_mumu");
	*/
	//cout << "Primary " << std::endl;
	if(DV_candidates.size()==0) return;


 
	//auto start_fit = std::chrono::high_resolution_clock::now();
	// Move this line up to before the trigger cut so that we can choose the signal region
  //do_DV_fit (DV_candidates,engine);
  // but only update the event weight now ...
 
  this->do_DV_fit (DV_candidates,engine);
	update_event_weight(DV_candidates);
	update_vertex_types(DV_candidates,found_ee,found_emu,found_mumu);

	/*
  double rdv1=0.0;
  double rdv2=0.0;
  for(auto dv: DV_candidates)
  {
	  double r = dv.get_position().pT();
	  if(r > rdv1) 
	  {
		  rdv2=rdv1;
		  rdv1 = r;

	  }
	  else
	  {
		  if(r > rdv2) rdv2 = r;

	  }

  }	
	if(DV_candidates.size() > 0)
	{
		FillYodaHisto1D("R_DV1",rdv1);
	}
	if((DV_candidates.size() > 1) && (rdv2 > 0.0))
	{
		FillYodaHisto1D("R_DV2",rdv2);
	}
	*/
	// auto end_fit = std::chrono::high_resolution_clock::now();

	// std::chrono::duration<double> fit_diff = std::chrono::duration_cast<std::chrono::duration<double>>(end_fit -end_trigger);
  //double timediff = difftime(endtime, starttime) * 1000.0;

	// cout.precision(10);
  //  cout << "fit took " << fit_diff.count() << std::endl;



	
	ApplyCuts(DV_candidates.size()>0,"N(DV)>=1");
	/*
	ApplyCut((DV_candidates.size()>0)&&found_ee,"N(DV)>=1_ee");
	ApplyCut((DV_candidates.size()>0)&&found_emu,"N(DV)>=1_emu");
	ApplyCut((DV_candidates.size()>0)&&found_mumu,"N(DV)>=1_mumu");
*/

	if(DV_candidates.size()==0) return;
	// Not implemented
	ApplyCuts(true,"Vertex fit");
	/*
	ApplyCut(true,"Vertex fit_ee");
	ApplyCut(true,"Vertex fit_emu");
	ApplyCut(true,"Vertex fit_mumu");
*/

	apply_d_xy_criterion(DV_candidates,2.);
	update_event_weight(DV_candidates);
	update_vertex_types(DV_candidates,found_ee,found_emu,found_mumu);
	ApplyCuts(DV_candidates.size()>0,"d_xy");
	/*
	ApplyCut((DV_candidates.size()>0)&&found_ee,"d_xy_ee");
	ApplyCut((DV_candidates.size()>0)&&found_emu,"d_xy_emu");
	ApplyCut((DV_candidates.size()>0)&&found_mumu,"d_xy_mumu");
	*/
	if(DV_candidates.size()==0) return;

	apply_fiducial_volume_criterion(DV_candidates,300.,300.);
	update_event_weight(DV_candidates);
	update_vertex_types(DV_candidates,found_ee,found_emu,found_mumu);
	// apply a cut, if no vertex passes the vertex selection criteria
	ApplyCuts(DV_candidates.size()>0,"Fiducial volume");
	/*
	ApplyCut((DV_candidates.size()>0)&&found_ee,"Fiducial volume_ee");
	ApplyCut((DV_candidates.size()>0)&&found_emu,"Fiducial volume_emu");
	ApplyCut((DV_candidates.size()>0)&&found_mumu,"Fiducial volume_mumu");
	*/
	if(DV_candidates.size()==0) return;
	// Update weights according to the probability for the DVs to be
	// vetoed, remove event if all DVs are vetoed with 100% probability

	
	double pixel_mult=apply_dis_pixel_mod_veto (DV_candidates,engine);
	update_event_weight(DV_candidates);
	update_vertex_types(DV_candidates,found_ee,found_emu,found_mumu);
	ApplyCuts(pixel_mult>0.,"Dis. pixel mod. veto");
	/*
	ApplyCut((pixel_mult>0.)&&found_ee,"Dis. pixel mod. veto_ee");
ApplyCut((pixel_mult>0.)&&found_emu,"Dis. pixel mod. veto_emu");
ApplyCut((pixel_mult>0.)&&found_mumu,"Dis. pixel mod. veto_mumu");*/

	//myWeight *= pixel_mult;
	
	
	  //Manager()->SetCurrentEventWeight(myWeight);
	//Event->set_weight(myWeight);
	
	//Reweight(pixel_mult);
	//ApplyCut(myWeight>0.,"Dis. pixel mod. veto");

	//if(myWeight ==0.) return;
	


	double material_mult=apply_material_veto (DV_candidates,engine);
	update_event_weight(DV_candidates);
	update_vertex_types(DV_candidates,found_ee,found_emu,found_mumu);
	ApplyCuts(material_mult>0.,"Material veto");
	/*
	ApplyCut((material_mult>0.)&&found_ee,"Material veto_ee");
	ApplyCut((material_mult>0.)&&found_emu,"Material veto_emu");
	ApplyCut((material_mult>0.)&&found_mumu,"Material veto_mumu");*/
	//cout << "material veto reweight:" << material_mult << " : " << Event->weight() << " -> ";
	//myWeight *= apply_material_veto (DV_candidates);
	
	//myWeight *=material_mult;
	
	//Event->set_weight(myWeight);
	//Reweight(material_mult);
	//Manager()->SetCurrentEventWeight(myWeight);
	//ApplyCut(myWeight>0.,"Material veto");
	//if(myWeight ==0.) return;
	
	//cout << Event->weight() << std::endl;
	//cout << "New event weight: " << Event->weight() << std::endl;

	
	require_leptons (DV_candidates,1);
	update_event_weight(DV_candidates);
	update_vertex_types(DV_candidates,found_ee,found_emu,found_mumu);
	ApplyCuts(DV_candidates.size()>0,"N(l)>=1");

	/*
	ApplyCut((DV_candidates.size()>0)&&found_ee,"N(l)>=1_ee");
	ApplyCut((DV_candidates.size()>0)&&found_emu,"N(l)>=1_emu");
	ApplyCut((DV_candidates.size()>0)&&found_mumu,"N(l)>=1_mumu");
	*/
	 if(DV_candidates.size()==0) return;  

	
	require_leptons (DV_candidates,2);
	update_vertex_types(DV_candidates,found_ee,found_emu,found_mumu);
	ApplyCuts(DV_candidates.size()>0,"N(l)>=2");

	/*
	ApplyCut((DV_candidates.size()>0)&&found_ee,"N(l)>=2_ee");
	ApplyCut((DV_candidates.size()>0)&&found_emu,"N(l)>=2_emu");
	ApplyCut((DV_candidates.size()>0)&&found_mumu,"N(l)>=2_mumu");
	*/
	    if(DV_candidates.size()==0) return;
	
	// // TODO: not implemented
	ApplyCuts(true,"Lepton kinematics");
	//     return true;
	// // TODO: not implemented
	ApplyCuts(true,"Lepton identification");
	//     return true;
	// // TODO: not implemented
	ApplyCuts(true,"Overlap removal");
	//     return true;


	apply_trigger_matching (DV_candidates);
	update_event_weight(DV_candidates);
    ApplyCuts(DV_candidates.size()>0,"Trigger matching");
if(DV_candidates.size()==0) return;

	apply_preselection_matching (DV_candidates);
	update_event_weight(DV_candidates);
	ApplyCuts(DV_candidates.size()>0,"Presel. matching");
if(DV_candidates.size()==0) return;

	
	require_invariant_mass(DV_candidates,12.);
	update_event_weight(DV_candidates);
	ApplyCuts(DV_candidates.size()>0,"m_DV");
if(DV_candidates.size()==0) return;

	
	require_opposite_charge(DV_candidates);
	update_event_weight(DV_candidates);
	update_vertex_types(DV_candidates,found_ee,found_emu,found_mumu);
	ApplyCuts(DV_candidates.size()>0,"Opposite charge");
	/*
	ApplyCut((DV_candidates.size()>0)&&found_ee,"Opposite charge_ee");
	ApplyCut((DV_candidates.size()>0)&&found_emu,"Opposite charge_emu");
	ApplyCut((DV_candidates.size()>0)&&found_mumu,"Opposite charge_mumu");*/
//if(DV_candidates.size()==0) return;
	
//	if(DV_candidates.size() > 0) 
//	{
//		cout << "Passed!" << std::endl;
//	}
//auto end_analysis = std::chrono::high_resolution_clock::now();

	// std::chrono::duration<double> end_diff = std::chrono::duration_cast<std::chrono::duration<double>>(end_analysis - end_fit);
  //double timediff = difftime(endtime, starttime) * 1000.0;

  // cout.precision(10);
  //  cout << "remaining bits took " << end_diff.count() << std::endl;
	////////////////////////////////////////////// GOT TO HERE



  
}

