//#ifdef USE_RESTFRAMES
#include "ATLAS_SUSY_2018_16.h"


/**
 * @file
 * ATLAS SUSY-2018-16 
 * arXiv:1911.12606
 * 
 * 139 fb^{-1} @ 13 TeV
 * Search for 2 soft OS leptons
 * 
 * Recast by Mark Goodsell
 * arXiv:2404.XXXXX 
 * 
 **/


   






namespace HEP
{
    namespace ATLAS_SUSY_2018_16 
    {
/* Place here your user-defined functions */
    
    double calcMTauTau(const HEP::P4 &v1, const HEP::P4 &v2,HEP::P4 &v3) 
{  
    
    HEP::P4 pv1(v1.px(),v1.py(),0.0,0.0); 
    HEP::P4 pv2(v2.px(),v2.py(),0.0,0.0); 
    HEP::P4 pv3(v3.px(),v3.py(),0.0,0.0); 
    double xi1,xi2; 
    double A11=pv1.dot3(pv1);  
    double A12=pv1.dot3(pv2); 
    double A21=pv2.dot3(pv1); 
    double A22=pv2.dot3(pv2); 
    double a=pv3.dot3(pv1); 
    double b=pv3.dot3(pv2); 
    double det = A11*A22-A21*A12;  
    if(det == 0) return 0.; 
    xi1=(A22*a-A12*b)/det; 
    xi2=(-A21*a+A11*b)/det; 
 
    double m_tautau_square = 2*v1.dot(v2)*(1+xi1)*(1+xi2); 
    double m_tautau = 0.; 
    if (m_tautau_square > 0) m_tautau = sqrt(m_tautau_square); 
    else m_tautau = -sqrt(-m_tautau_square); 
    return (m_tautau);  
   
} 

    double muonRecoEff(const HEP::Particle* lepton)
    {
      double x,x2,x3,x4,res;
      x=lepton->pT();
      x2=x*x;
      x3=x*x2;
      x4=x*x3;
      res=0.98 - 55.2473/x4 - 19.5118/x3 + 18.6462/x2 - 3.8107/x;

       /* This was a perhaps misguided attempt to improve agreement */
      //if ((x < 10.0) &&(x > 5.0)) res=0.9*res;
      return res;
     // return 0.98 - 55.2473/x4 - 19.5118/x3 + 18.6462/x2 - 3.8107/x;

    }

    double electronRecoEff(const HEP::Particle* lepton)
    {
      double x,x2,x3,x4;
      x=lepton->pT();
      x2=x*x;
      x3=x*x2;
      x4=x*x3;
      return 0.98 - 228.108/x3 + 72.6552/x2 - 8.86632/x;


    }

    double trackRecoEff(const HEP::Particle* lepton)
    {
       double x,x1,x12,x13,x14,x15;
      x=lepton->pT();
      if(x > 6.0) return 0.0;
      x1=x-1;
      x12=x1*x1;
      x13=x1*x12;
      x14=x1*x13;
      x15=x1*x14;
      //return (0.46051 - 0.547044*x + 0.164621*x2)/((-2.39615 + x)*(-2.39615 + x) + 0.0512832*x);
      //return 0.055 + (-0.172022*x + 0.0916718*x2)/(5.83032 - 4.75591*x + x2);

      return (0.131677*x1 - 0.330357 *x12 + 0.253828 *x13 -  0.0508656 *x14 +  0.00341932 *x15)/(0.0658491 + (-2.31126 + x)*(-2.31126 + x));

    }

    double electronCandidateEff(const HEP::Particle* lepton)
    {
      // interpolated from Fig 5 of 1908.00005
       double x,x2,x3,x4,x5;
      x=lepton->pT();
      if(x > 6.0) return 0.0;
      x2=x*x;
      x3=x*x2;
      x4=x*x3;
      x5=x*x4;
      
      return  (0.261548*x - 0.027799*x2 + 0.001416256*x3 - 0.000033979*x4 + 3.02444449e-7*x5)/(1 + exp(7.33147172 - 2.0243*x));

    }


    }
}

using namespace HEP::ATLAS_SUSY_2018_16;

void ATLAS_SUSY_2018_16::init() {
this->DetectorFunction="ATLAS";
cout << "---------------------------------------------------" << endl;
cout << "-- Analysis ATLAS_SUSY_2018_16 --" << endl;
cout << "---------------------------------------------------" << endl;
// MuonEfficiency.readcsv("ATLAS_SUSY_2018_16/Muoneff.csv");

AddYodaHisto1D("mll",60,0,30);

vector<double> mlledges={0,2,3,5,10,20,30,40,60};
AddYodaHisto1D("mllbins",mlledges);

  /*
LabRecoFrame*       LAB = new LabRecoFrame("LAB","LAB");
        DecayRecoFrame* CM = new DecayRecoFrame("CM","CM");
        DecayRecoFrame* S = new DecayRecoFrame("S","S");
        VisibleRecoFrame* ISR = new VisibleRecoFrame("ISR","ISR");
        VisibleRecoFrame* V= new VisibleRecoFrame("V","V");
        VisibleRecoFrame* L= new VisibleRecoFrame("L","L");
        VisibleRecoFrame* I= new VisibleRecoFrame("I","I");
        InvisibleGroup* INV= new InvisibleGroup("INV","INV");

        // InvisibleJigsaw(const std::string& sname, 		    const std::string& stitle,   int Ninvisible, int Nvisible);
        //kSetMass does InvisibleJigsaw = new SetMassInvJigsaw("InvMass","InvMass");
        //InvisibleJigsaw* InvMass = m_RF_helper.addInvisibleJigsaw("InvMass", kSetMass);
        InvisibleJigsaw* InvMass= new InvisibleJigsaw("InvMass","InvMass");
        CombinatoricGroup* VIS= new CombinatoricGroup("VIS","VIS");
        //kMinMasses does MinMassesCombJigsaw = new MinMassesCombJigsaw(SplitVis,SplitVis);

        MinMassesCombJigsaw* SplitVis= new MinMassesCombJigsaw("SplitVis","SplitVis");
      */

      LabRecoFrame* LAB        = m_RF_helper.addLabFrame("LAB");
  DecayRecoFrame* CM       = m_RF_helper.addDecayFrame("CM");
  DecayRecoFrame* S        = m_RF_helper.addDecayFrame("S");
  VisibleRecoFrame* ISR    = m_RF_helper.addVisibleFrame("ISR");
  VisibleRecoFrame* V      = m_RF_helper.addVisibleFrame("V");
  VisibleRecoFrame* L      = m_RF_helper.addVisibleFrame("L");
  InvisibleRecoFrame* I    = m_RF_helper.addInvisibleFrame("I");

  InvisibleGroup* INV      = m_RF_helper.addInvisibleGroup("INV");

  // set the invisible system mass to zero
  InvisibleJigsaw* InvMass = m_RF_helper.addInvisibleJigsaw("InvMass", kSetMass);
  //kSetMass does InvisibleJigsaw = new SetMassInvJigsaw("InvMass","InvMass");

  CombinatoricGroup* VIS   = m_RF_helper.addCombinatoricGroup("VIS");

  // define the rule for partitioning objects between "ISR" and "V"
  MinMassesCombJigsaw* SplitVis = m_RF_helper.addCombinatoricJigsaw("SplitVis", kMinMasses);
  //kMinMasses does MinMassesCombJigsaw = new MinMassesCombJigsaw(SplitVis,SplitVis);


LAB->SetChildFrame(*CM);
  CM->AddChildFrame(*ISR);
  CM->AddChildFrame(*S);
  S->AddChildFrame(*V);
  S->AddChildFrame(*I);
  S->AddChildFrame(*L);

  LAB->InitializeTree();
//cout << "Initialised Tree" << endl;
  INV->AddFrame(*I);

  VIS->AddFrame(*ISR);
  VIS->SetNElementsForFrame(*ISR,1,false);
  VIS->AddFrame(*V);
  VIS->SetNElementsForFrame(*V,0,false);

  INV->AddJigsaw(*InvMass);

  VIS->AddJigsaw(*SplitVis);
  // "0" group (ISR)
  SplitVis->AddFrame(*ISR, 0);
  // "1" group (V + I + L)
  SplitVis->AddFrame(*V,1);
  SplitVis->AddFrame(*I,1);
  SplitVis->AddFrame(*L,1);

  LAB->InitializeAnalysis();
//cout << "Initialised restframes " << endl;
//const std::vector<std::string> allSRs={"SR_E_lT","SR_E_high","SR_E_med","SR_E_low"};


// NB only the exclusive regions are present in the pyhf json model, so I will only keep those.

const std::vector<std::string> SR_E_lT={"SR_E_lT_eMLLa","SR_E_lT_eMLLb","SR_E_lT_eMLLc","SR_E_lT_eMLLd","SR_E_lT_eMLLe","SR_E_lT_eMLLf"};
const std::vector<std::string> SR_E_high={"SRee_E_high_eMLLa","SRee_E_high_eMLLb","SRee_E_high_eMLLc","SRee_E_high_eMLLd","SRee_E_high_eMLLe","SRee_E_high_eMLLf","SRee_E_high_eMLLg","SRee_E_high_eMLLh","SRmm_E_high_eMLLa","SRmm_E_high_eMLLb","SRmm_E_high_eMLLc","SRmm_E_high_eMLLd","SRmm_E_high_eMLLe","SRmm_E_high_eMLLf","SRmm_E_high_eMLLg","SRmm_E_high_eMLLh"};
const std::vector<std::string> SR_E_med={"SRee_E_med_eMLLa","SRee_E_med_eMLLb","SRee_E_med_eMLLc","SRee_E_med_eMLLd","SRee_E_med_eMLLe","SRee_E_med_eMLLf","SRmm_E_med_eMLLa","SRmm_E_med_eMLLb","SRmm_E_med_eMLLc","SRmm_E_med_eMLLd","SRmm_E_med_eMLLe","SRmm_E_med_eMLLf"};
const std::vector<std::string> SR_E_low={"SRee_E_low_eMLLa","SRee_E_low_eMLLb","SRee_E_low_eMLLc","SRee_E_low_eMLLd","SRee_E_low_eMLLe","SRee_E_low_eMLLf","SRee_E_low_eMLLg","SRee_E_low_eMLLh","SRmm_E_low_eMLLa","SRmm_E_low_eMLLb","SRmm_E_low_eMLLc","SRmm_E_low_eMLLd","SRmm_E_low_eMLLe","SRmm_E_low_eMLLf","SRmm_E_low_eMLLg","SRmm_E_low_eMLLh"};

const std::vector<std::string> allSRs={"SR_E_lT_eMLLa","SR_E_lT_eMLLb","SR_E_lT_eMLLc","SR_E_lT_eMLLd","SR_E_lT_eMLLe","SR_E_lT_eMLLf","SRee_E_high_eMLLa","SRee_E_high_eMLLb","SRee_E_high_eMLLc","SRee_E_high_eMLLd","SRee_E_high_eMLLe","SRee_E_high_eMLLf","SRee_E_high_eMLLg","SRee_E_high_eMLLh","SRee_E_med_eMLLa","SRee_E_med_eMLLb","SRee_E_med_eMLLc","SRee_E_med_eMLLd","SRee_E_med_eMLLe","SRee_E_med_eMLLf","SRee_E_low_eMLLa","SRee_E_low_eMLLb","SRee_E_low_eMLLc","SRee_E_low_eMLLd","SRee_E_low_eMLLe","SRee_E_low_eMLLf","SRee_E_low_eMLLg","SRee_E_low_eMLLh","SRmm_E_high_eMLLa","SRmm_E_high_eMLLb","SRmm_E_high_eMLLc","SRmm_E_high_eMLLd","SRmm_E_high_eMLLe","SRmm_E_high_eMLLf","SRmm_E_high_eMLLg","SRmm_E_high_eMLLh","SRmm_E_med_eMLLa","SRmm_E_med_eMLLb","SRmm_E_med_eMLLc","SRmm_E_med_eMLLd","SRmm_E_med_eMLLe","SRmm_E_med_eMLLf","SRmm_E_low_eMLLa","SRmm_E_low_eMLLb","SRmm_E_low_eMLLc","SRmm_E_low_eMLLd","SRmm_E_low_eMLLe","SRmm_E_low_eMLLf","SRmm_E_low_eMLLg","SRmm_E_low_eMLLh"};


const std::vector<std::string> SR_E_2L={"SRee_E_high_eMLLa","SRee_E_high_eMLLb","SRee_E_high_eMLLc","SRee_E_high_eMLLd","SRee_E_high_eMLLe","SRee_E_high_eMLLf","SRee_E_high_eMLLg","SRee_E_high_eMLLh","SRee_E_med_eMLLa","SRee_E_med_eMLLb","SRee_E_med_eMLLc","SRee_E_med_eMLLd","SRee_E_med_eMLLe","SRee_E_med_eMLLf","SRee_E_low_eMLLa","SRee_E_low_eMLLb","SRee_E_low_eMLLc","SRee_E_low_eMLLd","SRee_E_low_eMLLe","SRee_E_low_eMLLf","SRee_E_low_eMLLg","SRee_E_low_eMLLh","SRmm_E_high_eMLLa","SRmm_E_high_eMLLb","SRmm_E_high_eMLLc","SRmm_E_high_eMLLd","SRmm_E_high_eMLLe","SRmm_E_high_eMLLf","SRmm_E_high_eMLLg","SRmm_E_high_eMLLh","SRmm_E_med_eMLLa","SRmm_E_med_eMLLb","SRmm_E_med_eMLLc","SRmm_E_med_eMLLd","SRmm_E_med_eMLLe","SRmm_E_med_eMLLf","SRmm_E_low_eMLLa","SRmm_E_low_eMLLb","SRmm_E_low_eMLLc","SRmm_E_low_eMLLd","SRmm_E_low_eMLLe","SRmm_E_low_eMLLf","SRmm_E_low_eMLLg","SRmm_E_low_eMLLh"};

const std::vector<std::string> SRee={"SRee_E_high_eMLLa","SRee_E_high_eMLLb","SRee_E_high_eMLLc","SRee_E_high_eMLLd","SRee_E_high_eMLLe","SRee_E_high_eMLLf","SRee_E_high_eMLLg","SRee_E_high_eMLLh","SRee_E_med_eMLLa","SRee_E_med_eMLLb","SRee_E_med_eMLLc","SRee_E_med_eMLLd","SRee_E_med_eMLLe","SRee_E_med_eMLLf","SRee_E_low_eMLLa","SRee_E_low_eMLLb","SRee_E_low_eMLLc","SRee_E_low_eMLLd","SRee_E_low_eMLLe","SRee_E_low_eMLLf","SRee_E_low_eMLLg","SRee_E_low_eMLLh"};


const std::vector<std::string> SRmm={"SRmm_E_high_eMLLa","SRmm_E_high_eMLLb","SRmm_E_high_eMLLc","SRmm_E_high_eMLLd","SRmm_E_high_eMLLe","SRmm_E_high_eMLLf","SRmm_E_high_eMLLg","SRmm_E_high_eMLLh","SRmm_E_med_eMLLa","SRmm_E_med_eMLLb","SRmm_E_med_eMLLc","SRmm_E_med_eMLLd","SRmm_E_med_eMLLe","SRmm_E_med_eMLLf","SRmm_E_low_eMLLa","SRmm_E_low_eMLLb","SRmm_E_low_eMLLc","SRmm_E_low_eMLLd","SRmm_E_low_eMLLe","SRmm_E_low_eMLLf","SRmm_E_low_eMLLg","SRmm_E_low_eMLLh"};

for(auto region : allSRs)
{
 // cout << region << ", ";
  AddRegionSelection(region);
}
//cout << endl << "Added " << allSRs.size() << " regions" <<endl;

/* DEBUG CODE ...
for(auto region : SR_E_high)
{
  if(std::find(allSRs.begin(), allSRs.end(), region) ==allSRs.end())
  {
    cout << "Couldn't find high region: " << region << endl;
  }

}
*/


/*
AddRegionSelection("SR_E_lT"); 
AddRegionSelection("SR_E_high"); 
AddRegionSelection("SR_E_med"); 
AddRegionSelection("SR_E_low"); 
*/   
  //const std::vector<std::string> SR_E_2L={"SR_E_high","SR_E_med","SR_E_low"}; 
   
// Cutflows preselect for one jet ...
AddCut("Filter 0 Jets",allSRs);

  /*
  // Have to do another code for VBF channels 
  Manager()->AddRegionSelection(SR-VBF-high);
  Manager()->AddRegionSelection(SR-VBF-low);
  */
 
  //=========General Cuts==1l1T===============// 
   

AddCut("ET Trigger",allSRs); 


AddCut("1 lepton and >= 1 track",SR_E_lT);  

 //=========General Cuts==2l===============// 
 
AddCut("2 leptons",SR_E_2L);  
//AddCut("2 electrons",SRee); 
//AddCut("2 muons",SRmm); 


//AddCut("veto 3 GeV < mll < 3.2 GeV",SR_E_lT); 
AddCut("veto 3 GeV < mll < 3.2 GeV",allSRs); 


AddCut("ET_miss > 200 GeV",SR_E_lT); 

AddCut("min(delta phi(j1,pT_miss)) > 0.4",allSRs); 
AddCut("delta phi(j1,pT_miss) > 2.0",allSRs);  


AddCut("m_{ell track} < 5 GeV",SR_E_lT); 
AddCut("Delta R(ell track) > 0.05",SR_E_lT); 

AddCut("1 < mll < 60 GeV",SR_E_2L); 
AddCut("deltaR_ee > 0.30",SR_E_2L); 
AddCut("Leading lepton pT > 5 GeV",SR_E_2L); 

//AddCut("delta phi(j1,pT_miss) > 2.0",SR_E_lT);  // common
AddCut("Number of jets >= 1",allSRs);  // common
AddCut("Leading jet pT > 100 GeV",allSRs); 
 

//AddCut("Number of jets >= 1",SR_E_lT);  // common
//AddCut("Leading jet pT > 100 GeV",SR_E_lT); 

AddCut("ET_miss/HT_lep > 30",SR_E_lT); 
AddCut("Delta R(ell track) < 1.5",SR_E_lT); 


// end preselection


// 1llT

AddCut("Lepton pT < 10 GeV",SR_E_lT); 
AddCut("Track pT < 5 GeV",SR_E_lT); 
AddCut("deltaphi(l,met) < 1.0",SR_E_lT); 
   
 
 

//cout << "Added two lepton cuts" <<endl;
//AddCut("veto 3 GeV < mll < 3.2 GeV",SR_E_2L); 
//AddCut("min(delta phi(j1,pT_miss)) > 0.4",SR_E_2L); 
//AddCut("delta phi(j1,pT_miss) > 2.0",SR_E_2L);  


//AddCut("1 < mll < 60 GeV",SR_E_2L); 
//AddCut("deltaR_ee > 0.30",SR_E_2L); 
//AddCut("Leading lepton pT > 5 GeV",SR_E_2L); 
//AddCut("Number of jets >= 1",SR_E_2L); 
//AddCut("Leading jet pT > 100 GeV",SR_E_2L); 
AddCut("Number of b-tagged jets = 0",SR_E_2L); 
AddCut("m_tautau < 0 or > 160 GeV",SR_E_2L); 
 
 //cout << "added SR_E_2L cuts" << endl;
  //======Signal Region Cuts=================// 
  //(high) to differentiate this cut from other SRs cut 

// New cut, just before the mll ones for 1l1T  
//AddCut("OSSF",allSRs);
AddCut("OSSF",SR_E_lT);
AddCut("2 electrons",SRee); 
AddCut("2 muons",SRmm); 

//cout << "added ETmiss high cut " << endl;
AddCut("m_T^l1 < 60 GeV",SR_E_high); 
AddCut("ET_miss > 200 GeV (high)",SR_E_high); 

AddCut("R_ISR",SR_E_high); 
AddCut("sub-leading lepton pT (high)",SR_E_high); 
 //cout << "added high cuts" << endl;
 
AddCut("120 < ET_miss < 200 GeV",SR_E_med); 
AddCut("ET_miss/HT_lep > 10.0",SR_E_med); 
AddCut("M_T^S < 50 GeV",SR_E_med); 
 //cout << "added med cuts" << endl;
AddCut("120 < ET_miss < 200 GeV (low)",SR_E_low); 
AddCut("ET_miss/HT_lep < 10.0 (low)",SR_E_low); 
AddCut("0.8 < R_ISR < 1.0",SR_E_low); 
AddCut("sub-leading lepton pT (low)",SR_E_low); 
AddCut("10 < m_T^l1 < 60 GeV",SR_E_low); 
   
 
//cout << "added common cuts" << endl;
// SR_E_high mass cuts  
AddCut("m_ll < 60 GeV",SR_E_high); 

AddCut("m_ll in [40,60] GeV","SRee_E_high_eMLLh");
AddCut("m_ll in [40,60] GeV","SRmm_E_high_eMLLh");

AddCut("m_ll in [30,40] GeV","SRee_E_high_eMLLg");
AddCut("m_ll in [30,40] GeV","SRmm_E_high_eMLLg");

AddCut("m_ll in [20,30] GeV","SRee_E_high_eMLLf");
AddCut("m_ll in [20,30] GeV","SRmm_E_high_eMLLf");

AddCut("m_ll in [10,20] GeV","SRee_E_high_eMLLe");
AddCut("m_ll in [10,20] GeV","SRmm_E_high_eMLLe");

AddCut("m_ll in [5,10] GeV","SRee_E_high_eMLLd");
AddCut("m_ll in [5,10] GeV","SRmm_E_high_eMLLd");

AddCut("m_ll in [3,5] GeV","SRee_E_high_eMLLc");
AddCut("m_ll in [3,5] GeV","SRmm_E_high_eMLLc");

AddCut("m_ll in [2,3] GeV","SRee_E_high_eMLLb");
AddCut("m_ll in [2,3] GeV","SRmm_E_high_eMLLb");

AddCut("m_ll < 2 GeV","SRee_E_high_eMLLa");
AddCut("m_ll < 2 GeV","SRmm_E_high_eMLLa");

//cout << "Added high cuts" <<endl;
/*
AddCut("m_ll < 40 GeV","SR_E_high"); 
AddCut("m_ll < 30 GeV","SR_E_high"); 
AddCut("m_ll < 20 GeV","SR_E_high"); 
AddCut("m_ll < 10 GeV","SR_E_high"); 
AddCut("m_ll < 5 GeV","SR_E_high"); 
AddCut("m_ll < 3 GeV","SR_E_high"); 
AddCut("m_ll < 2 GeV","SR_E_high"); 
 */
// SR_E_low mass cuts  
 
   
AddCut("m_ll < 60 GeV",SR_E_low);
AddCut("m_ll in [40,60] GeV","SRee_E_low_eMLLh");
AddCut("m_ll in [40,60] GeV","SRmm_E_low_eMLLh");

AddCut("m_ll in [30,40] GeV","SRee_E_low_eMLLg");
AddCut("m_ll in [30,40] GeV","SRmm_E_low_eMLLg");

AddCut("m_ll in [20,30] GeV","SRee_E_low_eMLLf");
AddCut("m_ll in [20,30] GeV","SRmm_E_low_eMLLf");

AddCut("m_ll in [10,20] GeV","SRee_E_low_eMLLe");
AddCut("m_ll in [10,20] GeV","SRmm_E_low_eMLLe");

AddCut("m_ll in [5,10] GeV","SRee_E_low_eMLLd");
AddCut("m_ll in [5,10] GeV","SRmm_E_low_eMLLd");

AddCut("m_ll in [3,5] GeV","SRee_E_low_eMLLc");
AddCut("m_ll in [3,5] GeV","SRmm_E_low_eMLLc");

AddCut("m_ll in [2,3] GeV","SRee_E_low_eMLLb");
AddCut("m_ll in [2,3] GeV","SRmm_E_low_eMLLb");

AddCut("m_ll < 2 GeV","SRee_E_low_eMLLa");
AddCut("m_ll < 2 GeV","SRmm_E_low_eMLLa");

//cout << "Added low cuts" <<endl;

/* 
AddCut("m_ll < 40 GeV","SR_E_low"); 
AddCut("m_ll < 30 GeV","SR_E_low"); 
AddCut("m_ll < 20 GeV","SR_E_low"); 
AddCut("m_ll < 10 GeV","SR_E_low"); 
AddCut("m_ll < 5 GeV","SR_E_low"); 
AddCut("m_ll < 3 GeV","SR_E_low"); 
AddCut("m_ll < 2 GeV","SR_E_low"); 
 */
 
//SR_E_med mass cuts 
 
AddCut("m_ll < 30 GeV",SR_E_med); 

AddCut("m_ll in [20,30] GeV","SRee_E_med_eMLLf");
AddCut("m_ll in [20,30] GeV","SRmm_E_med_eMLLf");

AddCut("m_ll in [10,20] GeV","SRee_E_med_eMLLe");
AddCut("m_ll in [10,20] GeV","SRmm_E_med_eMLLe");

AddCut("m_ll in [5,10] GeV","SRee_E_med_eMLLd");
AddCut("m_ll in [5,10] GeV","SRmm_E_med_eMLLd");

AddCut("m_ll in [3,5] GeV","SRee_E_med_eMLLc");
AddCut("m_ll in [3,5] GeV","SRmm_E_med_eMLLc");

AddCut("m_ll in [2,3] GeV","SRee_E_med_eMLLb");
AddCut("m_ll in [2,3] GeV","SRmm_E_med_eMLLb");

AddCut("m_ll < 2 GeV","SRee_E_med_eMLLa");
AddCut("m_ll < 2 GeV","SRmm_E_med_eMLLa");
//cout << "Added med cuts" <<endl;
/*
AddCut("m_ll < 20 GeV","SR_E_med"); 
AddCut("m_ll < 10 GeV","SR_E_med"); 
AddCut("m_ll < 5 GeV","SR_E_med"); 
AddCut("m_ll < 3 GeV","SR_E_med"); 
AddCut("m_ll < 2 GeV","SR_E_med"); 
 */
// SR_E_lT mass cuts 
AddCut("ml_track < 5 GeV",SR_E_lT); 
AddCut("ml_track in [4,5] GeV","SR_E_lT_eMLLf");
AddCut("ml_track in [3,4] GeV","SR_E_lT_eMLLe");
AddCut("ml_track in [2,3] GeV","SR_E_lT_eMLLd");
AddCut("ml_track in [1.5,2] GeV","SR_E_lT_eMLLc");
AddCut("ml_track in [1,1.5] GeV","SR_E_lT_eMLLb");
AddCut("ml_track in < 1 GeV","SR_E_lT_eMLLa");
/*
AddCut("m_ll in [4,5] GeV","SR_E_lT_eMLLf");
AddCut("m_ll in [3,4] GeV","SR_E_lT_eMLLe");
AddCut("m_ll in [2,3] GeV","SR_E_lT_eMLLd");
AddCut("m_ll in [1.5,2] GeV","SR_E_lT_eMLLc");
AddCut("m_ll in [1,1.5] GeV","SR_E_lT_eMLLb");
AddCut("ml_track < 1","SR_E_lT_eMLLa"); 
*/
/*
AddCut("ml_track < 4","SR_E_lT"); 
AddCut("ml_track < 3","SR_E_lT"); 
AddCut("ml_track < 2","SR_E_lT"); 
AddCut("ml_track < 1.5","SR_E_lT"); 
AddCut("ml_track < 1","SR_E_lT"); 
*/

cout << "============ Initialised ATLAS_SUSY_2018_16 ========= " << endl;
}

ATLAS_SUSY_2018_16::~ATLAS_SUSY_2018_16() {

};

void ATLAS_SUSY_2018_16::Execute(std::mt19937 &engine) {


static std::uniform_real_distribution<double> rd(0.0,1.0);
//Defining the containers 
vector<const HEP::Jet*> baseJets;
 vector<const HEP::Particle*>baseMuons,baseElectrons,baseLeptons;

  vector<const HEP::Jet*> SignalJets, signalBJets; 
  vector<const HEP::Particle*> signalMuons,signalElectrons; 
   
   
  //to get tracks within R < 0.01 of reconstructed electron or muon candidate 
  vector<const HEP::Particle*> preElTracks,preMuTracks; 
    vector<const HEP::Particle*>baseElTracks,baseMuTracks; 
  //the MET 
  HEP::P4 metVec = Event->missingmom(); 
  double met = metVec.pT(); 
  //cout << met << endl; 


  /* 
  Used to eliminate signals with 3 leptons: paper claims that more than three signal leptons
  are used, but applying that after the efficiencies is too aggressive compared to the
  exclusion plots. 

  In the ATLAS fill functions we only count leptons that don't come from hadron decays, so should be safe


  */
  int CountBaseLeptons = 0;  

  double basethreshold=2.5;
  for(auto CurrentJet : Event->jets())
    {
      if ( CurrentJet->pT() > 20.0 && fabs(CurrentJet->eta())<4.5) 
      {
        //MV2c10 algorithm 85% efficiency working point
        // background-jet rejection factors, defined as 1/𝜀 for background jets -> 25 for light jets
        
        SignalJets.push_back(CurrentJet); 
        if(fabs(CurrentJet->eta())<2.5)
        {
          double rdbtag=rd(engine);
          if(CurrentJet->btag())
          {
            if(rdbtag < 0.85) signalBJets.push_back(CurrentJet);
          }  // mistag
          else if(rdbtag < 0.04) signalBJets.push_back(CurrentJet);

        }
        
        
        
      }
    }

    for(auto CurrentTrack : Event->electrons())
    {
      
      double abseta = CurrentTrack->abseta(); 

      if (abseta > 2.5) continue;
      double pt = CurrentTrack->pT(); 
       if (pt > basethreshold) CountBaseLeptons++;  // efficiency for tracks is very small below 2 GeV
      ///double sin=fabs(pt/CurrentTrack->p()); 
      if(pt > 4.5 && abseta < 2.47 && ED0Sigma5_ATLAS_Run2(CurrentTrack, Event,engine) && (rd(engine) < electronRecoEff(CurrentTrack)))
	{
	  //double ptratio = sumpTisolation(CurrentTrack->mom(),Event,0.4);
	  //if (ptratio < 0.2*pt) 

     //if(rd(engine) <  ElectronCombinedEfficiency.get_at(pt))  
     baseElectrons.push_back(CurrentTrack); 
      } 
      else if(pt > 0.5 && abseta < 2.5 && pt < 4.5&& (rd(engine) < trackRecoEff(CurrentTrack)*electronCandidateEff(CurrentTrack))) preElTracks.push_back(CurrentTrack); 
      

    }
 
  for(auto  CurrentTrack : Event->muons())
    {
      
      double abseta = CurrentTrack->abseta(); 
       if (abseta > 2.5) continue;
       double pt = CurrentTrack->pT(); 

       if (pt > basethreshold) CountBaseLeptons++; // efficiency for tracks is very small below 2 GeV
      ///double sin=fabs(pt/CurrentTrack->p()); 
      if(pt > 3. && abseta < 2.5 && MuD0Sigma3_ATLAS_Run2(CurrentTrack, Event,engine) && (rd(engine) < muonRecoEff(CurrentTrack))){
        //if(pt > 3. && abseta < 2.5 && MuD0Sigma3_ATLAS_Run2(CurrentTrack, Event,engine) && (rd(engine) < MuonEfficiency.get_at(pt))){
//	double ptratio = sumpTisolation(CurrentTrack->mom(),Event,0.4);
	//if (ptratio < 0.2*pt) 
  //if(rd(engine) <  MuonCombinedEfficiency.get_at(pt))  baseMuons.push_back(CurrentTrack); 
  baseMuons.push_back(CurrentTrack); 
	
      } 
      else if(pt > 0.5 && abseta < 2.5 && pt < 3.0  && (rd(engine) < trackRecoEff(CurrentTrack))) 
      {
         
        preMuTracks.push_back(CurrentTrack); 
      }
    }


     baseElectrons  	= Removal(baseElectrons, baseMuons, 0.01);
  SignalJets = Removal(SignalJets,baseElectrons,0.2);

  baseElectrons= Removal(baseElectrons,SignalJets,0.4);
    baseMuons= Removal(baseMuons,SignalJets,0.4);

baseElectrons= Removal(baseElectrons,signalBJets,0.4);
    baseMuons= Removal(baseMuons,signalBJets,0.4);

ApplyCut(SignalJets.size()>0,"Filter 0 Jets");
if(SignalJets.size() ==0) return;

// now do ET trigger

bool ETtrigger=true;
// et trigger eff= 0.98/(1.0 + exp(6.6 - 0.059*met))

//if(rd(engine) < 0.98/(1.0 + exp(6.6 - 0.059*met))) ETtrigger = true;

//if(met < 120.0) ETtrigger = false; // addition offline cut on all signal regions 
double ET_eff= 0.98/(1.0 + exp(6.6 - 0.059*met));
this->Reweight(ET_eff); // reweight instead of randomly throwing away
ApplyCut(ETtrigger,"ET Trigger");
//if(!ETtrigger) return;


if(CountBaseLeptons > 2) return; // remove events with more base leptons



std::sort(baseElectrons.begin(),baseElectrons.end(),HEP::compare_pT);
std::sort(baseMuons.begin(),baseMuons.end(),HEP::compare_pT);


std::sort(SignalJets.begin(),SignalJets.end(),HEP::compare_pTJ);

signalElectrons=baseElectrons; // apply efficiencies for these later
signalMuons=baseMuons; 

//filterEfficiency(signalElectrons,&HEP::ATLAS_SUSY_2018_16::electronRecoEff,engine);
//filterEfficiency(signalMuons,&HEP::ATLAS_SUSY_2018_16::muonRecoEff,engine);

 vector<const HEP::Particle*> signalLeptons = signalElectrons; 
  signalLeptons.insert(signalLeptons.end(),signalMuons.begin(),signalMuons.end());
for (auto& preElTrack : preElTracks) { 
    bool match = false; 
    for (auto& signalElectron : signalElectrons) { 
      if (preElTrack == signalElectron) { 
        match = true; 
        break; 
      } 
    } 
    if (!match) baseElTracks.push_back(preElTrack); 
  } 
 
  //cout << baseElTracks.size() << endl; 
 
   
  for (auto& preMuTrack : preMuTracks) { 
    bool match = false; 
    for (auto& signalMuon : signalMuons) { 
      if (preMuTrack == signalMuon) { 
        match = true; 
        break; 
      } 
    } 
    if (!match) baseMuTracks.push_back(preMuTrack); 
  } 

 //baseline tracks do not have signal leptons 
 // vector<const HEP::Particle*> baseTracks   = baseElTracks; 
 // baseTracks.insert(baseTracks.end(),baseMuTracks.begin(),baseMuTracks.end()); 
 // Tracks are required to be separated from preselected jets by at least dR > 0.5 
  baseElTracks = Removal(baseElTracks, SignalJets, 0.5); 
  baseMuTracks = Removal(baseMuTracks, SignalJets, 0.5); 

// sum pT of preselected tracks within dR < 0.3 of signal tracks is required to be smaller than 0.5 GeV (basically means nothing within 0.3 of the track) 
vector<const HEP::Particle*>signalElTracks,signalMuTracks; 
signalElTracks = SelfRemoval(baseElTracks, 0.3);  
  //signalMuTracks = OverlapRemoval(baseMuTracks, baseTracks, 0.3); 
  signalMuTracks = SelfRemoval(baseMuTracks, 0.3);  
 
  filterPhaseSpace(signalElTracks,1.0,2.5);
  filterPhaseSpace(signalMuTracks,2.0,2.5);


  //cout << signalMuTracks.size() << endl; 
 
  vector<const HEP::Particle*> signalTracks = signalElTracks; 
  signalTracks.insert(signalTracks.end(),signalMuTracks.begin(),signalMuTracks.end()); 

  //filterEfficiency(signalTracks,&HEP::ATLAS_SUSY_2018_16::trackRecoEff,engine);
  std::sort(signalTracks.begin(),signalTracks.end(),HEP::compare_pT);
 
 
 
   
 
 //================Calculate variables=============== 
 

  // Events must have ==2 signal leptons or ==1 signal lepton plus >=1 track 
  bool is2L  = (signalLeptons.size() == 2 && signalTracks.size() ==0); 
  bool is1LT = (signalLeptons.size() == 1 && signalTracks.size()>=1);  
  if ( !(is2L || is1LT) ) return; 
 
  const HEP::Particle* lep1 = signalLeptons[0]; 
  const HEP::Particle* lep2 = is2L ? signalLeptons[1] : signalTracks[0]; 
  HEP::P4 lep = lep1->mom()+lep2->mom(); 
   
  //double jet1Pt    = SignalJets.size() >= 1 ? SignalJets[0]->mom().pT() : -1.; 
  //double dPhiJ1Met = SignalJets.size() >= 1 ? fabs( SignalJets[0]->mom().deltaPhi( metVec ) ) : -1.;  
 double jet1Pt    =SignalJets[0]->mom().pT();
 double dPhiJ1Met =fabs( SignalJets[0]->mom().deltaPhi( metVec ) );
   
  // Minimum of the delta phi between all jets with MET 
 
  double minDPhiAllJetsMet = 10.; 
  for (auto Jets : SignalJets)  
  { 
    double dPhi = fabs(metVec.deltaPhi(Jets->mom())); 
    if (minDPhiAllJetsMet > dPhi)  minDPhiAllJetsMet=dPhi; 
  } 
   
  //cout << minDPhiAllJetsMet << endl; 
 
  double lep1Pt = lep1->mom().pT(); 
  double lep2Pt = lep2->mom().pT(); 
 
  double dPhiLepMet = fabs( lep.deltaPhi( metVec ) ); 
 
 
  // SFOS decision 
 
  //bool isOS = ( lep1->charge() != lep2->charge() ); // opposite charge  
  //isOS and is2L at never happen the same time  
  //opposite charge incorporated into isee ans ismm 
  bool isSF = ( abs(lep1->pid()) == abs(lep2->pid()) ); // same flavour 
  bool isee = ( lep1->pid()*lep2->pid() == -121 ); // OSS di-lepton 
  bool ismm = ( lep1->pid()*lep2->pid() == -169 ); //di-muon 
 
  
  //bool isee = ( abs(lep1->mc()->pdgid()) == 11 && abs( lep2->mc()->pdgid()) == 11 ); //di-electron 
  //bool isee = ( lep1->mc()->pdgid() == 11 && lep2->mc()->pdgid() == 11 ); //di-electron 
   
  // Higher level leptonic variables 
  double mll          = lep.m(); 
  double drll         = lep1->mom().deltaR_eta(lep2->mom()); 
  double metOverHtLep = met / (lep1Pt + lep2Pt);  
  double MTauTau      = calcMTauTau((lep1->mom()), (lep2->mom()), metVec); 
  double mt = mTcalc_simple(lep1->mom(),metVec); 
  
  FillYodaHisto1D("mll",mll);
   FillYodaHisto1D("mllbins",mll);


  //cout << "is1LT :"<< is1LT << endl; 
  //cout << "is2L :"<< is2L << endl; 
  //cout << "isOS :"<< isOS << endl; 
  //cout << "isSF :"<< isSF << endl; 
  
 
  //cout << "mll :"<< mll << endl; 
  //cout << ismm << endl; 
 
  //if (isee) cout << "drll if isee :" << drll << endl; 
  //if (ismm) cout << "drll if ismm :" << drll << endl; 
  
 
  //-------------------------Cuts-------------------------------- 

/*


AddCut("1 lepton and >= 1 track","SR_E_lT");  
AddCut("veto 3 GeV< mll < 3.2 GeV","SR_E_lT"); 
AddCut("ET_miss > 200 GeV","SR_E_lT"); 
AddCut("delta phi(j1,pT_miss) > 2.0","SR_E_lT");  // common
AddCut("Number of jets >= 1","SR_E_lT");  // common
AddCut("Leading jet pT > 100 GeV","SR_E_lT"); 
AddCut("ET_miss/HT_lep > 30","SR_E_lT"); 
AddCut("Lepton pT < 10 GeV","SR_E_lT"); 
AddCut("Track pT < 5 GeV","SR_E_lT"); 
AddCut("deltaphi(l,met) < 1.0","SR_E_lT"); 
   
 
  //=========General Cuts==2l===============// 
 
AddCut("2 leptons",SR_E_2L);  
AddCut("veto 3 GeV < mll < 3.2 GeV",SR_E_2L); 
AddCut("min(delta phi(j1,pT_miss)) > 0.4",SR_E_2L); 
AddCut("deltaphi(j1,met) > 2.0",SR_E_2L);  
AddCut("1 < mll < 60 GeV",SR_E_2L); 
AddCut("deltaR_ee > 0.30",SR_E_2L); 
AddCut("Leading lepton pT > 5 GeV",SR_E_2L); 
AddCut("Number of jets >= 1",SR_E_2L); 
AddCut("Leading jet pT > 100 GeV",SR_E_2L); 
AddCut("Number of b-tagged jets = 0",SR_E_2L); 
AddCut("m_tautau < 0 or > 160 GeV",SR_E_2L); 

*/


 
   

// Mostly common cuts   
ApplyCut(is1LT,"1 lepton and >= 1 track"); 
//ApplyCut((is2L && isSF),"2 leptons"); 

ApplyCut(is2L,"2 leptons"); 

/*
if(is2L)
{
ApplyCut(isee,"2 electrons");
ApplyCut(ismm,"2 muons");
};
*/


ApplyCut((mll < 3. || mll > 3.2 ),"veto 3 GeV < mll < 3.2 GeV"); 
ApplyCut((met > 200.),"ET_miss > 200 GeV"); 

ApplyCut((minDPhiAllJetsMet > 0.4),"min(delta phi(j1,pT_miss)) > 0.4"); 
ApplyCut((dPhiJ1Met >  2.0),"delta phi(j1,pT_miss) > 2.0"); 

ApplyCut((mll < 5. && mll > 0.5 ), "m_{ell track} < 5 GeV");
ApplyCut(drll>0.05,"Delta R(ell track) > 0.05");


ApplyCut(((mll > 1. && mll < 60. && ismm) || (mll > 3. && mll < 60. && isee)),"1 < mll < 60 GeV"); 
ApplyCut(((drll > 0.05 && ismm) || (drll > 0.3 && isee)),"deltaR_ee > 0.30"); 
ApplyCut((lep1Pt > 5.),"Leading lepton pT > 5 GeV"); 
ApplyCut((SignalJets.size() >= 1),"Number of jets >= 1"); 
ApplyCut((jet1Pt > 100.),"Leading jet pT > 100 GeV"); 

 // 1L1T Cuts  
ApplyCut((metOverHtLep >  30.),"ET_miss/HT_lep > 30");  

ApplyCut(drll<1.5,"Delta R(ell track) < 1.5");

ApplyCut((lep1Pt < 10.),"Lepton pT < 10 GeV"); 
ApplyCut((lep2Pt < 5.),"Track pT < 5 GeV"); 
ApplyCut((dPhiLepMet < 1.),"deltaphi(l,met) < 1.0"); 


  // Common 2L cuts 
ApplyCut((signalBJets.size() == 0),"Number of b-tagged jets = 0"); 
ApplyCut((MTauTau < 0. || MTauTau > 160.),"m_tautau < 0 or > 160 GeV"); 


//ApplyCut(isSF && (isee || ismm),"OSSF");

if(is1LT)
{
  ApplyCut(isee || ismm,"OSSF");
}

if(is2L)
{
ApplyCut(isee,"2 electrons");
ApplyCut(ismm,"2 muons");
};  
 
 

  //if(!Manager()->ApplyCut((is2L && isOS && isSF),"2 leptons")) return true; 

//ApplyCut((mll < 3. || mll > 3.2 ),"veto 3 GeV < mll < 3.2 GeV"); 

//ApplyCut((dPhiJ1Met > 2.0),"deltaphi(j1,met) > 2.0"); 


//ApplyCut((jet1Pt > 100.),"Leading jet pT > 100 GeV"); 


 
 
  //RJR variables 

  /*
  The momentum of the 'S' system should be leptons + neutralinos + decay products from the W. 
  We require zero b-jets for the 2 lepton regions for which the M_S cut is relevant. 
  But the W could produce jets or leptons. 

  The W-> leptons is only a small part of the signal since it focusses on the Z -> 2l region, so 
  we really need to include jets from the W in our M_S calculation. 

  I can't include the restframes package without root so we need some other proxy; perhaps a way of choosing
  one jet or jets to include in the 'S' system.
  */ 
 //double M_S =0.0;
  /*
  if(signalBJets.size() > 0)
  {
   HEP::P4 Smom=lep1->mom()+lep2->mom()+metVec+signalBJets[0]->mom();
    M_S = Smom.mT(); 
    cout << "M_S: " << M_S << endl;
  }
  */
  //Smom.setPE(Smom.px(),Smom.py(),0.0,Smom.E());

  

  /*
  HEP::P4 p_T_jet; 
  for (auto Jet : SignalJets) 
  { 
    p_T_jet = p_T_jet+Jet->mom(); 
  } 
  double R_ISR = -(metVec.px()*p_T_jet.px()+metVec.py()*p_T_jet.py())/(p_T_jet.px()*p_T_jet.px()+p_T_jet.py()*p_T_jet.py()); 
  */
  //cout << "R_ISR: " << R_ISR << endl;

//---------------------------------------------------------
  //
  //                Calculate RJR variables
  //
  //---------------------------------------------------------


 // analyze event in RestFrames tree
  // Combinatoric setup for jets
  LabRecoFrame* LAB = m_RF_helper.getLabFrame("LAB");
  CombinatoricGroup* VIS = m_RF_helper.getCombinatoricGroup("VIS");
  InvisibleGroup* INV = m_RF_helper.getInvisibleGroup("INV");
  DecayRecoFrame* CM = m_RF_helper.getDecayFrame("CM");
  DecayRecoFrame* S = m_RF_helper.getDecayFrame("S");
  VisibleRecoFrame* ISR = m_RF_helper.getVisibleFrame("ISR");
  VisibleRecoFrame* V = m_RF_helper.getVisibleFrame("V");      
  VisibleRecoFrame* L = m_RF_helper.getVisibleFrame("L");      // leptons
  InvisibleRecoFrame* I = m_RF_helper.getInvisibleFrame("I");

  LAB->ClearEvent();

  std::vector<RFKey> jetID;
  for (const auto jet : SignalJets){
    // transFourVect???
    //jetID.push_back(VIS->AddLabFrameFourVector(jet.transFourVect()));
  /*
    virtual AnalysisObject transFourVect() const {
    TLorentzVector tlv;
    tlv.SetPtEtaPhiM(Pt(), 0.0, Phi(), M());
    return AnalysisObject(tlv, charge(), id(), type(), motherID(), _orgIndex);
  };

  i.e. we set the z momentum to zero but preserve the mass of the jet
  */
    HEP::P4 jetmom=jet->mom();
    jetmom.setEtaPhiMPt(0.0,jetmom.phi(),jetmom.m(),jetmom.pT());
 //   cout << "jetmomz: " << jetmom.pz() << endl;
    jetID.push_back(VIS->AddLabFrameFourVector(jetmom));
  } 
//cout << "added jets to VIS" << endl;
  HEP::P4 lepSys(0.,0.,0.,0.);
  for(const auto lep1 : signalLeptons){
    //lepSys = lepSys + lep1.transFourVect();
    HEP::P4 tmom=lep1->mom();
    tmom.setEtaPhiMPt(0.0,tmom.phi(),tmom.m(),tmom.pT());
    
    lepSys = lepSys + tmom;
  }
//  cout << "lepmomz: " << lepSys.pz() << endl;
  L->SetLabFrameFourVector(lepSys);
 //   cout << "metvecz: " << metVec.pz() << endl;
  INV->SetLabFrameThreeVector(metVec.Vect().setPz(0.0));
  if (!LAB->AnalyzeEvent()) std::cout << "Something went wrong with RJR ..." << std::endl; //else std::cout << " RJR ok ..." << std::endl;

  // Compressed variables from tree
  int Njet_V = 0;
  int Njet_ISR = 0;  
  for (unsigned int i = 0; i < SignalJets.size(); ++i){
    if (VIS->GetFrame(jetID[i]) == *V) {
      Njet_V++;
    } else {
      Njet_ISR++;
    }
  }

  //RJR variables
  float Pt_ISR = 0.0; 
  float R_ISR = 0.0;
  float M_S = 0.0;
  float M_ISR = 0.0;
  float DPhi_I_ISR = 0.0;

 HEP::P3 v_P_ISR = ISR->GetFourVector(*CM).Vect();
  HEP::P3 v_P_I   = I->GetFourVector(*CM).Vect();
  Pt_ISR = v_P_ISR.Mag();
  R_ISR = fabs(v_P_I.Dot(v_P_ISR.Unit())) / Pt_ISR;
  M_S = S->GetMass();
  M_ISR = ISR->GetMass();
  DPhi_I_ISR = fabs(v_P_ISR.deltaPhi(v_P_I));
/*
  HEP::P4 SFV = S->GetFourVector(); // or S->GetFourVector(*LAB); ???


  if(is2L && met < 200)
  {
  cout << "M_S: " << M_S << ", M_T^S: " << SFV.mT() << " Zval: " << SFV.pz() << endl;
  }
*/



  //===============Specific Cuts for each SRs=============== 
 
  //SR_E_high 
ApplyCut(( mt < 60.),"m_T^l1 < 60 GeV"); 
ApplyCut((met > 200.),"ET_miss > 200 GeV (high)"); 

ApplyCut((( R_ISR < 1. ) && ( R_ISR > std::max(0.85, 0.98 - 0.02*mll))),"R_ISR"); 
ApplyCut(( lep2Pt > std::min(10., 2 + mll/3.)),"sub-leading lepton pT (high)"); 
 
  // SR_E_med 
ApplyCut((( met > 120. ) && ( met < 200. )),"120 < ET_miss < 200 GeV"); 
ApplyCut(( metOverHtLep > 10.),"ET_miss/HT_lep > 10.0"); 
ApplyCut((M_S < 50),"M_T^S < 50 GeV"); 
 
  // SR_E_low   
ApplyCut((( met > 120. ) && ( met < 200. )),"120 < ET_miss < 200 GeV (low)"); 
ApplyCut(( metOverHtLep < 10.),"ET_miss/HT_lep < 10.0 (low)"); 
ApplyCut((R_ISR < 1. && R_ISR > 0.8),"0.8 < R_ISR < 1.0"); 
ApplyCut((lep2Pt > (5. + mll/4.)),"sub-leading lepton pT (low)"); 
ApplyCut((mt > 10. && mt < 60. ),"10 < m_T^l1 < 60 GeV"); 
 
 
  //================Electroweakino inclusive SRs mll cuts=============== 
  //high 
ApplyCut((mll < 60.),"m_ll < 60 GeV"); 
if(mll >= 60.0) return;

ApplyCut((mll < 30.),"m_ll < 30 GeV"); // only applies for med

if(mll >= 40.) ApplyCut(true,"m_ll in [40,60] GeV"); 
else if (mll >= 30.0) ApplyCut(true,"m_ll in [30,40] GeV");
else if (mll >= 20.0) ApplyCut(true,"m_ll in [20,30] GeV");
else if (mll >= 10.0) ApplyCut(true,"m_ll in [10,20] GeV");
else if (mll >= 5.) ApplyCut(true,"m_ll in [5,10] GeV");
else if (mll >= 3.) ApplyCut(true,"m_ll in [3,5] GeV");
else if (mll >= 2.) ApplyCut(true,"m_ll in [2,3] GeV");
else ApplyCut(true,"m_ll < 2 GeV");

if(mll >= 5.) return;
// now for the tracks

ApplyCut((mll < 5.),"ml_track < 5 GeV"); // only applies for tracks

if(mll < 1.) ApplyCut(true,"ml_track < 1 GeV");
else if(mll < 1.5) ApplyCut(true,"ml_track in [1,1.5] GeV");
else if(mll < 2) ApplyCut(true,"ml_track in [1.5,2] GeV");
else if(mll < 3) ApplyCut(true,"ml_track in [2,3] GeV");
else if(mll < 4) ApplyCut(true,"ml_track in [3,4] GeV");
else ApplyCut(mll < 5,"ml_track in [4,5] GeV");



/*
ApplyCut((mll < 40.),"m_ll < 40 GeV"); 
ApplyCut((mll < 30.),"m_ll < 30 GeV"); 
ApplyCut((mll < 20.),"m_ll < 20 GeV"); 
ApplyCut((mll < 10.),"m_ll < 10 GeV"); 
ApplyCut((mll < 5.),"m_ll < 5 GeV"); 
ApplyCut((mll < 3.),"m_ll < 3 GeV"); 
ApplyCut((mll < 2.),"m_ll < 2 GeV"); 
 */ 

   
  /*
  //1LT 
ApplyCut((mll < 5.),"ml_track < 5"); 
ApplyCut((mll < 4.),"ml_track < 4"); 
ApplyCut((mll < 3.),"ml_track < 3"); 
ApplyCut((mll < 2.),"ml_track < 2"); 
ApplyCut((mll < 1.5),"ml_track < 1.5"); 
ApplyCut((mll < 1.),"ml_track < 1"); 
*/
   
}

//#endif