#include "ATLAS_SUSY_2019_08.h"
#include "include/kinematics.h"
/**
 * @file
 * ATLAS SUSY 2019 08
 * 1909.09226
 * 
 * EWinos in the WH channel, W -> lnu, h-> bbar
 * 139 fb^{-1} @ 13 TeV
 * 
 * Recast by Mark Goodsell
 * Originally performed in parallel to implementation in MadAnalysis,
 * now updated for latest version of HackAnalysis
 * 
**/




namespace HEP
{
    namespace ATLAS_SUSY_2019_08 
    {
/* Place here your user-defined functions */

    }
}

using namespace HEP::ATLAS_SUSY_2019_08;

void ATLAS_SUSY_2019_08::init() {

cout << "---------------------------------------------------" << endl;
cout << "-- Analysis ATLAS_SUSY_2019_08 --" << endl;
cout << "---------------------------------------------------" << endl;


AddRegionSelection("LMdisc"); 
AddRegionSelection("MMdisc"); 
AddRegionSelection("HMdisc"); 
 
AddRegionSelection("LMlowCT"); 
AddRegionSelection("LMmedCT"); 
AddRegionSelection("LMhighCT"); 
 
AddRegionSelection("MMlowCT"); 
AddRegionSelection("MMmedCT"); 
AddRegionSelection("MMhighCT"); 
 
AddRegionSelection("HMlowCT"); 
AddRegionSelection("HMmedCT"); 
AddRegionSelection("HMhighCT"); 

  const std::vector<std::string> alldiscSRs={"LMdisc","MMdisc","HMdisc"}; 
  const std::vector<std::string> allSRs={"LMdisc","MMdisc","HMdisc","LMlowCT","LMmedCT","LMhighCT","MMlowCT","MMmedCT","MMhighCT","HMlowCT","HMmedCT","HMhighCT"}; 
   
   
  const std::vector<std::string> LMexcl={"LMlowCT","LMmedCT","LMhighCT"}; 
const std::vector<std::string> allLM={"LMdisc","LMlowCT","LMmedCT","LMhighCT"}; 
 
  const std::vector<std::string> MMexcl={"MMlowCT","MMmedCT","MMhighCT"}; 
  const std::vector<std::string> allMM={"MMdisc","MMlowCT","MMmedCT","MMhighCT"}; 
 
  const std::vector<std::string> HMexcl={"HMlowCT","HMmedCT","HMhighCT"}; 
  const std::vector<std::string> allHM={"HMdisc","HMlowCT","HMmedCT","HMhighCT"}; 
   
     
  const std::vector<std::string> lowCT={"LMlowCT","MMlowCT","HMlowCT"}; 
  const std::vector<std::string> medCT={"LMmedCT","MMmedCT","HMmedCT"}; 
  const std::vector<std::string> highCT={"LMhighCT","MMhighCT","HMhighCT"}; 
 
 
 
 
    
 
   
  // for cutflows, preselection cuts are split out: 
   
  //Manager()->AddCut("Njets25 >=2",allSRs2); 
   
   
  //Manager()->AddCut("1 signal lepton",allSRs); 
 
AddCut("Njets25 >=2",allSRs); 
AddCut("1 signal lepton",allSRs); 
AddCut("Second baseline lepton veto",allSRs); 
AddCut("mT>50",allSRs); 
AddCut("ET>180",allSRs); 
AddCut("Njets<=3",allSRs); 
AddCut("2 bjets",allSRs); 
AddCut("mbb>50",allSRs); 
AddCut("ET>240",allSRs); 
AddCut("mbb",allSRs); 
 
AddCut("mlb > 120",allHM); 
 
AddCut("mT in [100,160]",LMexcl); 
AddCut("mT in [160,240]",MMexcl); 
AddCut("mT > 100","LMdisc"); 
AddCut("mT > 160","MMdisc"); 
AddCut("mT > 240",allHM); 
 
   
 
AddCut("minCT",alldiscSRs); 
 
AddCut("mCT in [180,230]",lowCT); 
AddCut("mCT in [230,280]",medCT); 
AddCut("mCT > 280",highCT); 
this->DetectorFunction="ATLAS";
}

ATLAS_SUSY_2019_08::~ATLAS_SUSY_2019_08() {

};

void ATLAS_SUSY_2019_08::Execute(std::mt19937 &engine) {

std::vector<const HEP::Particle*> InitialElectrons, InitialMuons, Electrons, Muons, BaselineLeptons, BaselineElectrons, BaselineMuons, baseElectrons, baseMuons; 
    std::vector<const HEP::Jet*> SoftJets, PSoftJets,SignalJets, bJets, Jets25; 
 
    std::vector<const HEP::Particle*> InitialPhotons; 

 //SoftElectrons.clear(); 
    Electrons.clear(); 
    //SoftMuons.clear(); 
    Muons.clear(); 
    SoftJets.clear(); 
    SignalJets.clear(); 
    InitialPhotons.clear(); 
 
    HEP::P4 pTmiss = Event->missingmom(); 

HEP::P4 precleanpt; 



  for(auto CurrentJet : Event->jets())
    {
      if ( CurrentJet->pT() > 20.0 && fabs(CurrentJet->eta())<4.5) 
      {

        SoftJets.push_back(CurrentJet); 
      //  if(CurrentJet->btag()&& fabs(CurrentJet->eta())<2.5)
      //  {
      //    signalBJets.push_back(CurrentJet);
      //  }
        //else if (fabs(CurrentJet->eta())<2.5)
       // {
        //    pTmiss+=CurrentJet->mom();
     //   }
        
      }
    }

    for(auto CurrentTrack : Event->electrons())
    {
      double pt = CurrentTrack->pT(); 
      double abseta = CurrentTrack->abseta(); 
      ///double sin=fabs(pt/CurrentTrack->p()); 
      if(pt > 7.0 && abseta < 2.47 && ED0Sigma5_ATLAS_Run2(CurrentTrack, Event,engine))
	{
	  //double ptratio = sumpTisolation(CurrentTrack->mom(),Event,0.4);
	  //if (ptratio < 0.2*pt) 

     //if(rd(engine) <  ElectronCombinedEfficiency.get_at(pt))  
     baseElectrons.push_back(CurrentTrack); 
      } 
      
    }
 
  for(auto  CurrentTrack : Event->muons())
    {
      double pt = CurrentTrack->pT(); 
      double abseta = CurrentTrack->abseta(); 
      ///double sin=fabs(pt/CurrentTrack->p()); 
      if(pt > 6. && abseta < 2.5 && MuD0Sigma3_ATLAS_Run2(CurrentTrack, Event,engine)){
//	double ptratio = sumpTisolation(CurrentTrack->mom(),Event,0.4);
	//if (ptratio < 0.2*pt) 
  //if(rd(engine) <  MuonCombinedEfficiency.get_at(pt))  baseMuons.push_back(CurrentTrack); 
  baseMuons.push_back(CurrentTrack); 
	
      } 
      
    }

    baseElectrons=SelfRemoval(baseElectrons,0.01); 
     baseElectrons  	= Removal(baseElectrons, baseMuons, 0.01);
     baseElectrons  	= RemovalJE(baseElectrons,SoftJets);

       SoftJets=Removal(SoftJets,baseMuons,0.2); 
  //SoftJets = Removal(SoftJets,baseElectrons,0.2);

 // baseElectrons= Removal(baseElectrons,SoftJets,0.4);
    baseMuons= RemovalJE(baseMuons,SoftJets);


filterEfficiency(SoftJets,&ATLASMediumJetEff,engine);


  
   


//baseElectrons= Removal(baseElectrons,signalBJets,0.4);
 //   baseMuons= Removal(baseMuons,signalBJets,0.4);

//signalElectrons=filterObjects(baseElectrons,7.0,2.47,Event,engine,&IsTightElectron_ATLAS_Run2);

/*
for(auto tElec : baseElectrons)
{
    if((tElec->pT()<200.0) || IsHighPTCaloOnly(tElec, event,0.2)) 
	      { 
		Electrons.push_back(tElec); 
	      } 

}
Muons=filterObjects(baseMuons,6.0, 2.5,Event,engine,&IsLooseTightMuon_ATLAS_Run2);
*/

Electrons=baseElectrons;
Muons=baseMuons;



// Here for GAMBIT efficiencies. Makes more sense if we do not have the pileup etc

std::string electronpoint="Loose";
Gambit::ColliderBit::ATLAS::applyElectronIDEfficiency2019(baseElectrons,electronpoint,engine);
//Gambit::ColliderBit::ATLAS::applyElectronReconstructionEfficiency2020(baseElectrons,electronpoint,engine);
std::string muonpoint="Medium";
Gambit::ColliderBit::ATLAS::applyMuonIDEfficiency2020(baseMuons,muonpoint,engine);
//Gambit::ColliderBit::ATLAS::applyMuonIDEfficiency2020(baseMuons,muonpoint,engine);

electronpoint="Tight";
Gambit::ColliderBit::ATLAS::applyElectronIDEfficiency2019(Electrons,electronpoint,engine);
//Gambit::ColliderBit::ATLAS::applyElectronIsolationEfficiency2020(Electrons,electronpoint,engine);
//Gambit::ColliderBit::ATLAS::applyElectronReconstructionEfficiency2020(baseElectrons,electronpoint,engine);
muonpoint="Loose";
Gambit::ColliderBit::ATLAS::applyMuonIDEfficiency2020(Muons,muonpoint,engine);
//Gambit::ColliderBit::ATLAS::applyMuonIDEfficiency2020(baseMuons,muonpoint,engine);
//Gambit::ColliderBit::ATLAS::applyMuonIsolationEfficiency2020(signalMuons,muonpoint,engine);
//Electrons=baseElectrons;
//signalElectrons=filterObjects(signalElectrons,Event,engine,&ED0Sigma5_ATLAS_Run2);

//signalMuons=filterObjects(signalMuons,Event,engine,&MuD0Sigma3_ATLAS_Run2);

 HEP::P4 bJetMomentum; 
int nj25=0; 
     
    for (auto tJet : SoftJets) 
      { 
        precleanpt-=tJet->mom(); 
	if(tJet->pT() > 25.0) 
	  { 
	     
	    nj25++; 
	  } 
	 
 
	if(tJet->pT()>30. && fabs(tJet->eta())<2.8) 
	  { 
        //smearbtag75(tJet->btag(),tJet->pT(),engine);
        bool isb=tJet->btag();
        smearbtag75(&isb,tJet->pT(),engine);
        //tJet->set_btag(isb);
	    SignalJets.push_back(tJet); 
 
	     
	    //if((tJet->btag()) && (fabs(tJet->eta())< 2.5)) 
        if(isb && (fabs(tJet->eta())< 2.5)) 
	      { 
		bJetMomentum+=tJet->mom(); 
		bJets.push_back(tJet); 
			 
	      } 

	     
	     
	} 
	 
      } 
 double MET = pTmiss.pT(); 
     
    unsigned int NJets = SignalJets.size(); 
 
     
    //Preselection cuts 
 
 
     
    bool IsMET180=  (MET>180.); 
    bool IsNJets2 = (nj25>1); 
    bool IsMET = (MET>240.); 
    int nleptons=Electrons.size()+Muons.size(); 
     
     
    bool IsLeptons = (nleptons == 1); 
    bool IsNJets = (NJets < 4);	   
    bool isBJets = (bJets.size() == 2); 

    int NbaselineL = baseElectrons.size()+baseMuons.size() - nleptons;
    bool NoBaselineLeptons = (NbaselineL <1); // if it is negative that is fine ... but the idea is that we don't want extras. 
 /*
    unsigned int NbaselineL= BaselineLeptons.size(); 
 
    bool NoBaselineLeptons= (NbaselineL == 0); 
   */  
    bool isFromHiggs = false; 
    bool mbb50 = false; 
    if(isBJets) 
      { 
	double mbb = bJetMomentum.M(); 
	mbb50=(mbb>50.0); 
	isFromHiggs = (mbb > 100.) && (mbb < 140.); 
      } 
     
	 
ApplyCut(IsNJets2,"Njets25 >=2"); 
ApplyCut(IsLeptons,"1 signal lepton"); 
ApplyCut(NoBaselineLeptons,"Second baseline lepton veto"); 

if(!NoBaselineLeptons || !IsLeptons || ! IsNJets2) return;

    HEP::P4 leptonp; 
    if (Electrons.size() == 1) 
      { 
	// only one, so lepton pt is electron pt 
	leptonp=Electrons[0]->mom(); 
      } 
    else 
      { 
	if (Muons.size() > 0) 
	  { 
	    leptonp=Muons[0]->mom(); 
	  } 
	else 
	  { 
	    leptonp.clear(); 
	  } 
      }
      
      
    // mT defined in the paper like this, only true for massless leptons ... 
    //double mT = sqrt(2.0*leptonp.Pt()*MET*(1.0-cos(leptonp.DeltaPhi(pTmiss)))); 
 
    double mT = mTcalc_simple(leptonp,pTmiss); 
   
    bool IsMT50 = (mT > 50.); 

    if(IsMT50 && NoBaselineLeptons && IsLeptons &&  IsNJets2)
    {
        // cout << "mT >50: " << mT << endl;
         ApplyCut(true,"mT>50"); 
    }
    else
    {
        //cout << "mT <50: " << mT << endl;
        return;
    }
//ApplyCut(IsMT50,"mT>50"); 
ApplyCut(IsMET180,"ET>180"); 
ApplyCut(IsNJets,"Njets<=3"); 
ApplyCut(isBJets,"2 bjets"); 
ApplyCut(mbb50,"mbb>50"); 
ApplyCut(IsMET,"ET>240"); 
ApplyCut(isFromHiggs,"mbb"); 
     
    
    HEP::P4 bjetmom1,bjetmom2; 
 
    if(bJets.size() == 2) 
      { 
	bjetmom1=bJets[0]->mom(); 
	bjetmom2=bJets[1]->mom(); 
      } 
 
    // mCT defined in the paper like this, only true for massless jets. Pseudocode uses some pre-defined function. Who to believe? 
    //double mCT = sqrt(2.0*(bjetmom1.Pt())*(bjetmom2.Pt())*(1.0+cos(bjetmom1.DeltaPhi(bjetmom2)))); 
 
    double mCT = HEP::mCT(bjetmom1,bjetmom2); 
    double mlb = (bjetmom1+leptonp).M(); 
 
    // cuts that depend on the SR 
     
ApplyCut(mlb> 120.0,"mlb > 120"); 
 
ApplyCut((mT > 100.0) && (mT <= 160.0),"mT in [100,160]"); 
ApplyCut((mT > 160.0) && (mT <= 240.0),"mT in [160,240]"); 
ApplyCut(mT > 100.0,"mT > 100"); 
ApplyCut(mT > 160.0,"mT > 160"); 
ApplyCut(mT > 240.0,"mT > 240"); 
 
     
ApplyCut(mCT> 180.0,"minCT"); 
ApplyCut((mCT > 180.0) && (mCT <= 230.0),"mCT in [180,230]"); 
ApplyCut((mCT > 230.0) && (mCT <= 280.0),"mCT in [230,280]"); 
ApplyCut(mCT> 280.0,"mCT > 280"); 
     
    






}

