#include "ATLAS_SUSY_2019_09_offshell.h"


/*
#include "include/kinematics.h"
#include <algorithm>
#include "include/isolation.h"
#include "include/lester_mt2_bisect.h"
*/

/**
* @file
* Search for chargino--neutralino pair production in final states with three leptons and missing transverse momentum
* Targets WZ signature with W->lv and Z->l+l-
* Published in 
* 2106.01676
* HEPData-ins-1866951
* https://www.hepdata.net/record/ins1866951
* https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2019-09/
*
* This is the "offshell" version
*
* Recast be Mark Goodsell, arXiv:2404.XXXXX
**/


namespace HEP
{
    namespace ATLAS_SUSY_2019_09_offshell 
    {
/* Place here your user-defined functions */
        bool radiusCalcLepton(const HEP::Particle* lepton,const HEP::Jet* jet) { 
         
           double drspecial=0.04 + 10./lepton->pT();
           if (drspecial < 0.4)
           {
            if(lepton->mom().deltaR_eta(jet->mom()) > drspecial)
            {
                return false;
            }
           }
           
        return true;
	    };

         bool LessThan3Tracks(const HEP::Jet* jet,const HEP::Event* event) { 

           double dr=0.4;
           
            int ntracks=0;

           for (auto track: event->get_charged_hadrons()) // 
           {
            if((track->pT() > 0.5) && (jet->mom().deltaR_eta(track->mom()) < dr))
            {
               ntracks++;
            }
               if (ntracks>=3) {return false;}

           }
           
           
        return true;
	    };

      double singleElectrontriggerEff(double &x)
      {
         /// fitted function for single electron trigger efficiency
         double x2,x3,x4;
         x2=x*x;
         x3=x2*x;
         x4=x2*x;
         return 0.98 - 142647./x4 + 13543.1/x3 - 418.364/x2 + 3.36013/x;

      };

      double diElectrontriggerEff(double &x)
      {
          /// fitted function for *one leg* of double electron trigger efficiency
         double x2,x3;
         x2=x*x;
         x3=x2*x;
         
         return 0.98 - 945.54/x3 - 128.24/x2 + 1.653/x;

      };

      double singleMuontriggerEff(const HEP::Particle *muon)  // check for eta and pT
      {
         /// fitted function for trigger on single muon
         double pT = muon->pT();
         if(pT < 27.3) return 0.0;
         if(muon->abseta() < 1.05)
         {
            if(pT < 60)
            {
               return 0.62+0.08*(pT-26.0)/34.0;
            }
            else return 0.7;

         }
         else
         {
            if(pT < 60)
            {
               return 0.78+0.08*(pT-26.0)/34.0;
            }
            else return 0.86;
         }
      };

      double doubleMuontriggerEff(const HEP::Particle *muon)  // check for eta and pT
      {
         /// fitted function for trigger on double  muon
         double pT = muon->pT();
         if(pT < 14.6) return 0.0;
         if(muon->abseta() < 1.05)
         {
         
            return 0.75;

         }
         else
         {
            return 0.85;
         }
      };
      double tripleMuontriggerEff(const HEP::Particle *muon)  // check for eta and pT
      {
         /// fitted function for trigger on triple muon
         double pT = muon->pT();
         if(pT < 6.5) return 0.0;
         if(muon->abseta() < 1.05)
         {
         
            return 0.75;

         }
         else
         {
            return 0.9;
         }
      };
//    see figure 18 for mu4 which extend down to 4 GeV: efficiency about 0.75 for eta < 1.05, 0.9 for larger eta, but
// much less below about 8 GeV. 

//figure 19 for mu14: roughly constant efficiency 0.75 eta < 1.05, 0.9 for eta > 1.05, 



      /*
      double getMetSignificance(const HEP::Event* event)
      {
         // construct met resolution from "hard" electrons, muons, jets and soft term




      }
      */

    }
}

using namespace HEP::ATLAS_SUSY_2019_09_offshell;

void ATLAS_SUSY_2019_09_offshell::init() {
//

/*

2106.01676


*/
//this->DetectorFunction="filleventPYTHIA_ATLAS_GAMBIT";
this->DetectorFunction="ATLAS";

cout << "---------------------------------------------------" << endl;
cout << "-- Analysis ATLAS_SUSY_2019_09_offshell --" << endl;
cout << "---------------------------------------------------" << endl;

AddYodaHisto1D("pTe1",50,1.0,200.0);
AddYodaHisto1D("pTe2",50,1.0,200.0);
AddYodaHisto1D("pTe3",50,1.0,200.0);
AddYodaHisto1D("pTm1",50,1.0,200.0);
AddYodaHisto1D("pTm2",50,1.0,200.0);
AddYodaHisto1D("pTm3",50,1.0,200.0);





vector<double> METedges={10,30,50,70,90,110,130,150,200,250,300,350,400,1000.0};
    
    AddYodaHisto1D("MET",METedges);
    AddYodaHisto1D("METSig",METedges);
   AddYodaHisto1D("JetpT",50,1.0,200.0);  


// Read efficiencies for this analysis from the tables
ElectronBDTefficiency.readcsv("ATLAS_SUSY_2019_09/binned_ElectronBDTeff.csv");
MuonBDTefficiency.readcsv("ATLAS_SUSY_2019_09/binned_MuonBDTeff.csv");
ElectronCombinedEfficiency.readcsv("ATLAS_SUSY_2019_09/binned_CombinedElectroneff.csv");
MuonCombinedEfficiency.readcsv("ATLAS_SUSY_2019_09/binned_CombinedMuoneff.csv");

/*
addRegions({"3L","Presel","Presel_offshell"});
   addRegions({"Presellow_0j", "Presellow_nj", "Preselhigh_0j", "Preselhigh_nj"});
   addRegions({"SRlow_0j", "SRlow_nj", "SRhigh_0j", "SRhigh_nj"});
   addRegions({"CRWZ_0j", "CRWZ_nj", "VRWZ_0j", "VRWZ_nj", "VRWZ_nj_lowmll", "VRttbar"});
   addRegions({"SRlow_0jb", "SRlow_0jc", "SRlow_0jd", "SRlow_0je", "SRlow_0jf1", "SRlow_0jf2", "SRlow_0jg1", "SRlow_0jg2"});
   addRegions({"SRlow_njb", "SRlow_njc", "SRlow_njd", "SRlow_nje", "SRlow_njf1", "SRlow_njf2", "SRlow_njg1", "SRlow_njg2"});
   addRegions({"SRhigh_0jb", "SRhigh_0jc", "SRhigh_0jd", "SRhigh_0je", "SRhigh_0jf1", "SRhigh_0jf2", "SRhigh_0jg1", "SRhigh_0jg2"});
   addRegions({"SRhigh_nja", "SRhigh_njb", "SRhigh_njc", "SRhigh_njd", "SRhigh_nje", "SRhigh_njf", "SRhigh_njg"});
*/

const std::vector<std::string> allSRs={"SRlow_0j", "SRlow_nj", "SRhigh_0j", "SRhigh_nj","SRlow_0jb", "SRlow_0jc", "SRlow_0jd", "SRlow_0je", "SRlow_0jf1", "SRlow_0jf2", "SRlow_0jg1", "SRlow_0jg2","SRlow_njb", "SRlow_njc", "SRlow_njd", "SRlow_nje", "SRlow_njf1", "SRlow_njf2", "SRlow_njg1", "SRlow_njg2","SRhigh_0jb", "SRhigh_0jc", "SRhigh_0jd", "SRhigh_0je", "SRhigh_0jf1", "SRhigh_0jf2", "SRhigh_0jg1", "SRhigh_0jg2","SRhigh_nja", "SRhigh_njb", "SRhigh_njc", "SRhigh_njd", "SRhigh_nje", "SRhigh_njf", "SRhigh_njg"};



const std::vector<std::string> rlow0j={"SRlow_0j","SRlow_0jb", "SRlow_0jc", "SRlow_0jd", "SRlow_0je", "SRlow_0jf1", "SRlow_0jf2", "SRlow_0jg1", "SRlow_0jg2"};
const std::vector<std::string> rlownj={"SRlow_nj","SRlow_njb", "SRlow_njc", "SRlow_njd", "SRlow_nje", "SRlow_njf1", "SRlow_njf2", "SRlow_njg1", "SRlow_njg2"};
const std::vector<std::string> rhigh0j={"SRhigh_0j","SRhigh_0jb", "SRhigh_0jc", "SRhigh_0jd", "SRhigh_0je", "SRhigh_0jf1", "SRhigh_0jf2", "SRhigh_0jg1", "SRhigh_0jg2"};
const std::vector<std::string> rhighnj={"SRhigh_nj","SRhigh_nja", "SRhigh_njb", "SRhigh_njc", "SRhigh_njd", "SRhigh_nje", "SRhigh_njf", "SRhigh_njg"};    


const std::vector<std::string> lowSRs={"SRlow_0j","SRlow_0jb", "SRlow_0jc", "SRlow_0jd", "SRlow_0je", "SRlow_0jf1", "SRlow_0jf2", "SRlow_0jg1", "SRlow_0jg2","SRlow_nj","SRlow_njb", "SRlow_njc", "SRlow_njd", "SRlow_nje", "SRlow_njf1", "SRlow_njf2", "SRlow_njg1", "SRlow_njg2"};
const std::vector<std::string> highSRs={"SRhigh_0j","SRhigh_0jb", "SRhigh_0jc", "SRhigh_0jd", "SRhigh_0je", "SRhigh_0jf1", "SRhigh_0jf2", "SRhigh_0jg1", "SRhigh_0jg2","SRhigh_nj","SRhigh_nja", "SRhigh_njb", "SRhigh_njc", "SRhigh_njd", "SRhigh_nje", "SRhigh_njf", "SRhigh_njg"};

const std::vector<std::string> lowMETSRs={"SRlow_0j","SRlow_0jb", "SRlow_0jc", "SRlow_0jd", "SRlow_0je", "SRlow_0jf1", "SRlow_0jf2", "SRlow_0jg1", "SRlow_0jg2","SRlow_nj","SRlow_njb", "SRlow_njc", "SRlow_njd", "SRlow_nje", "SRlow_njf1", "SRlow_njf2", "SRlow_njg1", "SRlow_njg2","SRhigh_0j","SRhigh_0jb", "SRhigh_0jc", "SRhigh_0jd", "SRhigh_0je", "SRhigh_0jf1", "SRhigh_0jf2", "SRhigh_0jg1", "SRhigh_0jg2"};

const std::vector<std::string> zerojetSRs={"SRlow_0j","SRlow_0jb", "SRlow_0jc", "SRlow_0jd", "SRlow_0je", "SRlow_0jf1", "SRlow_0jf2", "SRlow_0jg1", "SRlow_0jg2","SRhigh_0j","SRhigh_0jb", "SRhigh_0jc", "SRhigh_0jd", "SRhigh_0je", "SRhigh_0jf1", "SRhigh_0jf2", "SRhigh_0jg1", "SRhigh_0jg2"};
const std::vector<std::string> njetSRs={"SRlow_nj","SRlow_njb", "SRlow_njc", "SRlow_njd", "SRlow_nje", "SRlow_njf1", "SRlow_njf2", "SRlow_njg1", "SRlow_njg2","SRhigh_nj","SRhigh_nja", "SRhigh_njb", "SRhigh_njc", "SRhigh_njd", "SRhigh_nje", "SRhigh_njf", "SRhigh_njg"};


for(std::string region : allSRs)
      {
	AddRegionSelection(region);
      }

 AddCut("3 Baseline Leptons",allSRs);

AddCut("3 Signal Leptons",allSRs);
AddCut("B Jet veto",allSRs);

AddCut("nOSSF > 0",allSRs);


//AddCut("Base Cut 1",allSRs);
//AddCut("Base Cut 2",allSRs);
AddCut("Low Trigger",lowSRs);
AddCut("High Trigger",highSRs);


AddCut("mll veto",lowMETSRs);
AddCut("Low lepton pT",lowMETSRs);



AddCut("Offshell Preselection",allSRs);
AddCut("Min dR >= 0.4",lowSRs);


AddCut("0 jets",zerojetSRs);
AddCut("n jets",njetSRs);

AddCut("Presellow_0j",rlow0j);
AddCut("Presellow_nj",rlownj);
AddCut("Preselhigh_0j",rhigh0j);
AddCut("Preselhigh_nj",rhighnj);

for(std::string region: allSRs)
{
    AddCut(region,region);
}


}

ATLAS_SUSY_2019_09_offshell::~ATLAS_SUSY_2019_09_offshell() {

};

void ATLAS_SUSY_2019_09_offshell::Execute(std::mt19937 &engine) {



static std::uniform_real_distribution<double> rd(0.0,1.0);
vector<const HEP::Jet*> baseJets,signalJets,bJets,jets,jets20; 
  vector<const HEP::Particle*>baseMuons,baseElectrons,baseLeptons,signalMuons,signalElectrons,signalLeptons; 



/*

 auto baselineElectrons  = event->getElectrons(4.5, 2.47, ELooseBLLH);
   auto baselineMuons      = event->getMuons(3.0, 2.5, MuMedium | MuNotCosmic | MuQoPSignificance);
   auto baselineJets       = event->getJets(20., 4.5);
   auto metVec             = event->getMET();
   double met              = metVec.Et();
   double metSig           = event->getMETSignificance();

   //====================================================================================================
   auto baselineleptons = baselineElectrons + baselineMuons;
   int nbaselineleptons = baselineleptons.size();
    */

   auto metVec             = Event->missingmom();

   // Smearing: 8.9 GeV uncertainty on MET from soft term

   static std::normal_distribution<> smearmet(0.0, 8.9);
   
   metVec.setPx(metVec.px()+smearmet(engine));
   metVec.setPy(metVec.py()+smearmet(engine));


   double met              = metVec.pT();

   FillYodaHisto1D("MET",met);

/*
https://cds.cern.ch/record/2630948/files/ATLAS-CONF-2018-038.pdf page 9:

The Emiss signiﬁcance is deﬁned to test the hypothesis that the total transverse momentum carried by
invisible particles (named pinvT in the following) is equal to zero against the hypothesis that pinvT 
is diﬀerent from zero.


Looking at figure 7 from that paper, even for Z -> ee nu nu only a modest proportion of events have a
significance < 3 (the largest value required for the signal in this search) since the distribution is quite
flat.

Comparing figures 7(a) and 7(c) it is reasonable to conclude that the significance is a function of Et 

BUT This does not work well at all!!

*/


   double metSig = 10.0;  // fake value since we are only injecting signal

   // Values from comparing figs 7a and 7c, we only care about the thresholds 1.5 and 3.0 for the significance. These will be replaced below.
   if(met < 35.0) metSig= 2.9;
   if(met < 15.0) metSig=1.4; 
   

// NB a lepton trigger is used for the SRlows; for SRhigh an ETmiss trigger is used too.
// But this means we shouldn't just use the MET trigger efficiency.

// This is for the MET significance
   double uncertV[2][2] ={{0.0,0.0},{0.0,0.0}};





  for(auto CurrentJet : Event->jets())
    {
      if ( CurrentJet->pT() > 20.0 && abs(CurrentJet->eta())<4.5) 
      {
         double pT = CurrentJet->pT();
         double cosphi = CurrentJet->mom().px()/pT;
         double sinphi=CurrentJet->mom().py()/pT;
         double sigmapT=pT*(0.02998661080448855 - 21.064427751806225/(pT*pT) + 4.770352126699648/pT);
         double sigmaperp=0.2*sigmapT;  // don't have good figures on this, just table 1 

         uncertV[0][0]+=sigmapT*sigmapT*cosphi*cosphi +sigmaperp*sigmaperp*sinphi*sinphi;
         uncertV[0][1]+=(sigmapT*sigmapT-sigmaperp*sigmaperp)*cosphi*sinphi;
         //uncertV[1][0]=uncertV[0][1];
         uncertV[1][1]+=sigmapT*sigmapT*sinphi*sinphi +sigmaperp*sigmaperp*cosphi*cosphi;
        if(CurrentJet->btag())
        {
          bJets.push_back(CurrentJet);
        }
        else
        {
        baseJets.push_back(CurrentJet); 
        }
      }
    }

    for(auto CurrentTrack : Event->electrons())
    {
      double pT = CurrentTrack->pT(); 
      double abseta = CurrentTrack->abseta(); 
      
      if(pT > 4.5 && abseta < 2.47)
	{
      double cosphi = CurrentTrack->mom().px()/pT;
         double sinphi=CurrentTrack->mom().py()/pT;
         double sigmapT=pT*(0.00382643 - 0.13687/(pT*pT) + 0.975747/pT);
         double sigmaperp=0.2*sigmapT; // no good figures, just table 1

         uncertV[0][0]+=sigmapT*sigmapT*cosphi*cosphi +sigmaperp*sigmaperp*sinphi*sinphi;
         uncertV[0][1]+=(sigmapT*sigmapT-sigmaperp*sigmaperp)*cosphi*sinphi;
         //uncertV[1][0]=uncertV[0][1];
         uncertV[1][1]+=sigmapT*sigmapT*sinphi*sinphi +sigmaperp*sigmaperp*cosphi*cosphi;

	  //double ptratio = sumpTisolation(CurrentTrack->mom(),Event,0.4);
	  //if (ptratio < 0.2*pt) 

     if(rd(engine) <  ElectronCombinedEfficiency.get_at(pT)/ElectronBDTefficiency.get_at(pT))  baseElectrons.push_back(CurrentTrack); 
         //if((CurrentTrack->pT() > 10.0) || rd(engine) < 0.6) baseElectrons.push_back(CurrentTrack); 
      } 

    }

  for(auto  CurrentTrack : Event->muons())
    {
      double pT = CurrentTrack->pT(); 
      double abseta = CurrentTrack->abseta(); 
      if(pT > 3. && abseta < 2.5){

         double cosphi = CurrentTrack->mom().px()/pT;
         double sinphi=CurrentTrack->mom().py()/pT;
         double sigmapT=pT*(0.0221905 + 0.000101559*pT + (1.64276e-8)*pT*pT);
         double sigmaperp=0.05*sigmapT; // no good figures, just table 1

         uncertV[0][0]+=sigmapT*sigmapT*cosphi*cosphi +sigmaperp*sigmaperp*sinphi*sinphi;
         uncertV[0][1]+=(sigmapT*sigmapT-sigmaperp*sigmaperp)*cosphi*sinphi;
         //uncertV[1][0]=uncertV[0][1];
         uncertV[1][1]+=sigmapT*sigmapT*sinphi*sinphi +sigmaperp*sigmaperp*cosphi*cosphi;
//	double ptratio = sumpTisolation(CurrentTrack->mom(),Event,0.4);
	//if (ptratio < 0.2*pt) 
  if(rd(engine) <  MuonCombinedEfficiency.get_at(pT)/MuonBDTefficiency.get_at(pT))  baseMuons.push_back(CurrentTrack);
      // if((CurrentTrack->pT() > 10.0) || rd(engine) < 0.6) baseMuons.push_back(CurrentTrack); 
	
      } 

    }

   // add soft term:

   uncertV[0][0]+=8.9*8.9;
   uncertV[1][1]+=8.9*8.9;
   uncertV[1][0]=uncertV[0][1];

   /**
   *   Construct MET significance from "hard" electrons, muons, jets and "soft term" -- which is just an 
   *   uncertainty of 8.9 GeV in each direction
   *
   *   baseline objects before removal are fine at this point
   *
   *   We have the resolution measurement of pT and pT perp from the reference.
   *
   **/

   double uncertVdet=uncertV[0][0]*uncertV[1][1]-uncertV[0][1]*uncertV[1][0];
   //if(uncertVdet ==0)
   //{
  //    metSig=0.0;
   //}
   if(uncertVdet != 0.0) // otherwise use default extrapolation
   {
      metSig=sqrt((uncertV[1][1]*metVec.px()*metVec.px()+uncertV[0][0]*metVec.py()*metVec.py()-2.0*uncertV[0][1]*metVec.px()*metVec.py())/uncertVdet);

   }

   //cout << "MET: " << met << ", metSig: " << metSig << endl;

   FillYodaHisto1D("METSig",metSig);




//cout << "MET: " << met << ", nleptons: " << (baseMuons.size()+baseElectrons.size()) << endl;

//cout << "Found " << bJets.size() << " baseline b jets " <<endl;
    //baseLeptons=baseElectrons+baseMuons;


   


 // cout << "Before first removal " << endl;

  baseElectrons  	= Removal(baseElectrons, baseMuons, 0.01);
  baseJets = Removal(baseJets,baseElectrons,0.2);
//baseJets = Removal(baseJets,baseMuons,0.2); // really need some sort of 'less than three tracks'

  vector<const HEP::Jet*> newbaseJets;
  for(auto bajet : baseJets)
  {
   bool keepit=true;
   for(auto mu : baseMuons)
   {
      // paper says 0.4, pseudocode says 0.2
      if( (mu->mom().deltaR_eta(bajet->mom()) < 0.2) &&  LessThan3Tracks(bajet,Event))
      {
         keepit=false;
         break;
      }

   }
   if(keepit) newbaseJets.push_back(bajet);
  }

  baseJets=newbaseJets;



baseElectrons= Removal(baseElectrons,baseJets,0.4,&radiusCalcLepton);
baseMuons= Removal(baseMuons,baseJets,0.4,&radiusCalcLepton);




/*

Baseline electrons are required to have
𝑝 T > 4.5 GeV and fall within the acceptance of the ID (|𝜂| < 2.47). They are further required to satisfy the
calorimeter- and tracking-based ‘Loose and B-layer likelihood’ identification [145]. Baseline muons must
have 𝑝 T > 3 GeV and |𝜂| < 2.5, and satisfy Medium identification criteria

*/


std::sort(baseElectrons.begin(),baseElectrons.end(),HEP::compare_pT);
std::sort(baseMuons.begin(),baseMuons.end(),HEP::compare_pT);

//signalElectrons=baseElectrons; // apply efficiencies for these later
//signalMuons=baseMuons; 

/*
 if(baseElectrons.size() > 0)  FillYodaHisto1D("pTe1",baseElectrons[0]->pT());
      if( (baseElectrons.size() > 1)) FillYodaHisto1D("pTe2",baseElectrons[1]->pT());
      if( (baseElectrons.size() > 2)) FillYodaHisto1D("pTe3",baseElectrons[2]->pT());
      if((baseMuons.size() > 0)) FillYodaHisto1D("pTm1",baseMuons[0]->pT());
      if((baseMuons.size() > 1)) FillYodaHisto1D("pTm2",baseMuons[1]->pT());
      if((baseMuons.size() > 2)) FillYodaHisto1D("pTm3",baseMuons[2]->pT());

if(baseJets.size() > 0) FillYodaHisto1D("JetpT",baseJets[0]->pT());
*/
std::string electronpoint="Loose";
//Gambit::ColliderBit::ATLAS::applyElectronIDEfficiency2019(baseElectrons,electronpoint,engine);
//applyElectronReconstructionEfficiency2020(baseElectrons, std::string operating_point,engine);

//Gambit::ColliderBit::ATLAS::applyElectronReconstructionEfficiency2020(baseElectrons,electronpoint,engine);


std::string muonpoint="Medium";
//Gambit::ColliderBit::ATLAS::applyMuonIDEfficiency2020(baseMuons,muonpoint,engine);
//Gambit::ColliderBit::ATLAS::applyMuonIDEfficiency2020(baseMuons,muonpoint,engine);






baseLeptons=baseElectrons;
baseLeptons.insert( baseLeptons.end(), baseMuons.begin(), baseMuons.end() );

std::sort(baseLeptons.begin(),baseLeptons.end(),HEP::compare_pT);



int nbaselineleptons = baseLeptons.size();


//cout << "MET: " << met << ", nbaselineleptons: " << nbaselineleptons << endl;
bool threeleptons = (nbaselineleptons == 3);

ApplyCut(threeleptons,"3 Baseline Leptons");
if (!threeleptons) return;

// Now apply the BDT efficiency

if(baseLeptons[2]->abspid() == 11) // electron
{
   if(rd(engine) > ElectronBDTefficiency.get_at(baseLeptons[2]->pT())) 
   //if(rd(engine) > ElectronCombinedEfficiency.get_at(baseLeptons[2]->pT())) 
   {
      baseLeptons.erase(baseLeptons.end() -1);
      baseElectrons.erase(baseElectrons.end()-1);
   }

}
else
{
   if(rd(engine) > MuonBDTefficiency.get_at(baseLeptons[2]->pT())) 
   //if(rd(engine) > MuonCombinedEfficiency.get_at(baseLeptons[2]->pT())) 
   {
      baseLeptons.erase(baseLeptons.end() -1);
      baseMuons.erase(baseMuons.end()-1);
   }
}


  /*
  auto muJetSpecial = [] (const HEP::Jet* jet, const HEP::Particle* muon) { 
	  if (muon->pT()/jet->pT()>0.5) return true;
	  else return false;
	};
  */
//cout << "Before first NEW removal " << endl;
 // signalJets = Removal(signalJets,baseMuons,0.4,&muJetSpecial);
  
   //signalElectrons= Removal(signalElectrons,bJets,0.4);
//signalMuons= Removal(baseMuons,bJets,0.4);
   

  // cout << "Before first filter " << endl;

/*

signalElectrons=filterObjects(signalElectrons,15.0,2.47,Event,engine,&IsTightElectron_ATLAS_Run2);
signalElectrons=filterObjects(signalElectrons,Event,engine,&ED0Sigma5_ATLAS_Run2);
signalMuons=filterObjects(signalMuons,15.0,2.4,Event,engine,&IsLooseTightMuon_ATLAS_Run2);
signalMuons=filterObjects(signalMuons,Event,engine,&MuD0Sigma3_ATLAS_Run2);
//signalLeptons=signalElectrons+signalMuons;

signalLeptons=signalElectrons;

//cout << "Before first insert " << endl;
signalLeptons.insert( signalLeptons.end(), signalMuons.begin(), signalMuons.end() );


//signalJets=signalJets+bJets;

signalJets.insert( signalJets.end(), bJets.begin(), bJets.end() );


	

std::sort(signalLeptons.begin(),signalLeptons.end(),HEP::compare_pT);
std::sort(signalJets.begin(),signalJets.end(),HEP::compare_pTJ);
*/


/*
   // signal objects
   auto electrons = filterObjects(baselineElectrons, 4.5, 2.47, EMediumLH | ED0Sigma5 | EZ05mm | EIsoGradient); // e: Gradient
   auto muons     = filterObjects(baselineMuons, 3.0, 2.5, MuD0Sigma3 | MuZ05mm | MuIsoFCLoose);                // m: FCLoose
   auto jets20    = filterObjects(baselineJets, 20, 2.8, JVT120Jet);
   auto jets      = filterObjects(baselineJets, 30, 2.8, JVT120Jet);
   auto bjets     = filterObjects(jets20, 20., 2.5, BTag85MV2c10);                                              // b: MV2c10, fc85
   auto leptons   = electrons + muons;

   //====================================================================================================
   int nleptons   = leptons.size();
   int nelectrons = electrons.size();
   int nmuons     = muons.size();
   int njets      = jets.size(); // use nJets_30, not 20
   int nbjets     = bjets.size(); // uses jets20

   // pT order the objects
   sortObjectsByPt(leptons);
   sortObjectsByPt(jets);

*/

//signalElectrons=filterObjects(baseElectrons,4.5,2.47,Event,engine,&IsTightElectron_ATLAS_Run2);
//signalElectrons=filterObjects(baseElectrons,4.5,2.47,Event,engine,&IsLooseElectron_ATLAS_Run2);





/* Loose muons but 'gradient' electrons: have to use efficiency formula*/


/*

the looser working point Gradient (Loose) is employed for electrons (muons)
in the off-shell 𝑊𝑍 selection

*/
// Let's just try GAMBIT efficiencies for now

/*
signalElectrons=filterObjects(baseElectrons,4.5,2.47,Event,engine,&IsGradientElectron_ATLAS_Run2);
signalElectrons=filterObjects(signalElectrons,Event,engine,&ED0Sigma5_ATLAS_Run2);
signalMuons=filterObjects(signalMuons,3.0, 2.5,Event,engine,&IsLooseTightMuon_ATLAS_Run2);
signalMuons=filterObjects(signalMuons,Event,engine,&MuD0Sigma3_ATLAS_Run2);
*/
//signalElectrons=filterObjects(baseElectrons,4.5,2.47);

electronpoint="Gradient";
signalElectrons=baseElectrons;
//Gambit::ColliderBit::ATLAS::applyElectronIDEfficiency2019(signalElectrons,electronpoint,engine);
//Gambit::ColliderBit::ATLAS::applyElectronIsolationEfficiency2020(signalElectrons,electronpoint,engine);


//signalMuons=filterObjects(signalMuons,3.0, 2.5);
muonpoint="Loose";
signalMuons=baseMuons;
//Gambit::ColliderBit::ATLAS::applyMuonIDEfficiency2020(signalMuons,muonpoint,engine);
//Gambit::ColliderBit::ATLAS::applyMuonIsolationEfficiency2020(signalMuons,muonpoint,engine);



//Gambit::ColliderBit::ATLAS::smearMuonMomentum(baseMuons,engine);
//Gambit::ColliderBit::ATLAS::smearElectronEnergy(baseElectrons,engine);

/*
signalElectrons=filterObjects(baseElectrons,4.5,2.47,Event,engine,&IsGradientElectron_ATLAS_Run2);
signalElectrons=filterObjects(signalElectrons,Event,engine,&ED0Sigma5_ATLAS_Run2);
signalMuons=filterObjects(signalMuons,3.0, 2.5,Event,engine,&IsLooseTightMuon_ATLAS_Run2);
signalMuons=filterObjects(signalMuons,Event,engine,&MuD0Sigma3_ATLAS_Run2);
*/



/* NB for jets we can use, from http://cds.cern.ch/record/1700870/files/ATLAS-CONF-2014-018.pdf:

A JVT > 0.4 (JVT > 0.7) requirement results in efficiencies of 86% to 96% (82% to 92%) for pref
from 20 to 60 GeV. The looser JVT > 0.2 requirement is about 90% efficient for pT = 20 GeV jets,
reaches an efficiency of at least 96% for jets with pT > 40 GeV, 
*/


filterEfficiency(baseJets,&ATLASMediumJetEff,engine);
jets20=baseJets;
jets=baseJets;
filterPhaseSpace(jets20, 20, 2.8);
filterPhaseSpace(jets, 30, 2.8);
 
 if((met > 200.0 )&& (jets.size() > 0))
 {
if(baseElectrons.size() > 0)  FillYodaHisto1D("pTe1",baseElectrons[0]->pT());
      if( (baseElectrons.size() > 1)) FillYodaHisto1D("pTe2",baseElectrons[1]->pT());
      if( (baseElectrons.size() > 2)) FillYodaHisto1D("pTe3",baseElectrons[2]->pT());
      if((baseMuons.size() > 0)) FillYodaHisto1D("pTm1",baseMuons[0]->pT());
      if((baseMuons.size() > 1)) FillYodaHisto1D("pTm2",baseMuons[1]->pT());
      if((baseMuons.size() > 2)) FillYodaHisto1D("pTm3",baseMuons[2]->pT());
 FillYodaHisto1D("JetpT",jets[0]->pT());
//if(baseJets.size() > 0) FillYodaHisto1D("JetpT",baseJets[0]->pT());
 }

// apply btag efficiency here for now ... improve on this later!

for(auto bjet : jets20)
{
    if(! bjet->btag()) continue;

    if(bjet->abseta() > 2.5) continue;
    if(rd(engine) > 0.85) continue;   // nowadays the b-tagging algorithms correct for inefficiencies
    
    bJets.push_back(bjet);

}

signalLeptons=signalElectrons;
signalLeptons.insert( signalLeptons.end(), signalMuons.begin(), signalMuons.end() );

int nleptons=signalLeptons.size();
threeleptons = (nleptons == 3);

ApplyCut(threeleptons,"3 Signal Leptons");
if (!threeleptons) return;

std::sort(signalLeptons.begin(),signalLeptons.end(),HEP::compare_pT);
std::sort(jets.begin(),jets.end(),HEP::compare_pTJ);

/*
cout << "SLpT: " << endl;
for(auto siglep: signalLeptons)
{
   cout << siglep->pT() << ", ";
}
cout << endl;
*/
int nbjets=bJets.size();
int njets      = jets.size(); // use nJets_30, not 20

 ApplyCut(nbjets==0,"B Jet veto");
 if(nbjets>0) return;


//====================================================================================================
   //do mll assignment & mindR_OSSF + mindR_3L
   double mZ  = 91.1876;
   double mll = -999.;
   double mdiff = 1e6;
   double minmll = 1e6;
   double maxmll = -999;
   double mindR_OSSF = 1e6;
   double mindR_3L = 1e6;
   int nOSSF = 0;
   int i0, i1, i2;
   int iZ1(-1), iZ2(-1), iW(-1); // closest to mZ order
   int jZ1(-1), jZ2(-1), jW(-1); // minmll order

   // lepton permutations Z1, Z2, W
   std::string index="012";
   do {
      i0 = index[0]-'0'; i1 = index[1]-'0'; i2 = index[2]-'0';  // char2int

      // only Z1,Z2 with Z1>Z2
      if (i0<i1) continue;

      double idR = signalLeptons[i0]->mom().deltaR_eta(signalLeptons[i1]->mom());
      // mindR_3L
      if (idR < mindR_3L) mindR_3L = idR;

      // OSSF pairs only
      if ( (signalLeptons[i0]->get_3Q() == signalLeptons[i1]->get_3Q()) || (signalLeptons[i0]->abspid() != signalLeptons[i1]->abspid()) ) continue;
      nOSSF++;

      double imll = (signalLeptons[i0]->mom()+signalLeptons[i1]->mom()).m();
      double imdiff = fabs(imll-mZ);

      // closest to mZ
      if (imdiff < mdiff) {
         mdiff = imdiff;
         mll = imll;
         iZ1 = i0; iZ2 = i1; iW = i2;
      }
      // minmll
      if (imll < minmll) {
         minmll = imll;
         jZ1 = i0; jZ2 = i1; jW = i2;
      }
      // maxmll
      if (imll > maxmll) maxmll = imll;
      // mindR_OSSF
      if (idR < mindR_OSSF) mindR_OSSF = idR;

   } while (std::next_permutation(index.begin(), index.end()));

  
    //  std::cout << "mll: " << mll << std::endl;
    //  std::cout << "minmll: " << minmll << std::endl;
    //  std::cout << "maxmll: " << maxmll << std::endl;
   

   // at least one OSSF pair


   ApplyCut((nOSSF>0),"nOSSF > 0");
   if (nOSSF==0) return;

// at this point everything should be assigned!
   if (iZ1==-1 || iZ2==-1 || iW==-1 || jZ1==-1 || jZ2==-1 || jW==-1) {
      std::cout << "Z(ll)W(lv) lepton assignment failed!" << std::endl;
      return;
   }

      


    //====================================================================================================
   // More baseline cuts


   bool trigger=false;

/**
*
*  Events are chosen for the 𝑊ℎ and on-shell 𝑊𝑍 selections using dilepton triggers and for the off-shell 𝑊𝑍
selection using single-lepton, dilepton and trilepton triggers
*
*  Single-electron triggers are not used
*
* 2004.13447:
*   The muon triggers must be HLT_mu26_ivarmedium: efficiency about 0.7, depends on eta and pT, figure 14
*   HLT_2mu14 -> ??
*   HLT_3mu6 (so one, two or three) ???
*
* p12:
* The multi-muon triggers are made either by requiring multiple muon candidates, each
* of which independently fires a single-muon trigger, or by finding multiple muons using the FS strategy
* after the leading muon candidate has been confirmed by a single-muon trigger. 
*
* The threshold on the tri-muon trigger was increased from 4 GeV to 6 GeV in 2018
*
* [It seems the FS ones are not relevant for this analysis. ]
*
* see figure 
*
* p24:
* While the primary single-muon triggers serve a wide variety of physics analyses, multi-object triggers
* are important in many cases where lower thresholds benefit the analysis. The efficiency of these triggers
* can be factorised as the product of the single-leg efficiencies
*
* see figure 18 for mu4 which extend down to 4 GeV: efficiency about 0.75 for eta < 1.05, 0.9 for larger eta, but
* much less below about 8 GeV. 
* 
* figure 19 for mu14: roughly constant efficiency 0.75 eta < 1.05, 0.9 for eta > 1.05, 
*
* ----------------------------
* 1909.00761 
*   The electron triggers must be L1_2EM15VH or  L1_2EM20VH or  L1_2EM15VHI. I can't find tri-electron triggers
*   
*   single E trigger: figure 17 page 27
*   diEtrigger: fig 20(a) Evolution of efficiencies for dielectron trigger legs
*   
*   no reference to tri-electron triggers -> maybe doesn't exist?
*
*
**/


// FIRST do lepton triggers: di-electron triggers (efficiencies are products of the leg efficiencies) and 

//diElectrontriggerEff

      if( (baseElectrons.size() > 1) && (baseElectrons[1]->pT() > 18.0))  // dielectron trigger
      {
         double teff=0.0;
         //double teff1,teff2;
         double pT1= baseElectrons[0]->pT();
         double pT2= baseElectrons[1]->pT();
         teff=diElectrontriggerEff(pT1)*diElectrontriggerEff(pT2);

         if(rd(engine) < teff) trigger=true;

      }

      if(!trigger && (baseElectrons.size() >2) && (baseElectrons[2]->pT() > 18.0))  // dielectron trigger, another bite at the cherry
      {
          double teff=0.0;
         //double teff1,teff2;
         double pT1= baseElectrons[0]->pT();
         double pT2= baseElectrons[2]->pT();
         teff=diElectrontriggerEff(pT1)*diElectrontriggerEff(pT2);

         if(rd(engine) < teff) trigger=true;

      }

      // now do muon triggers
       if(!trigger && (baseMuons.size() > 0) && (baseMuons[0]->pT() > 27.3)) // single muon trigger
      {
         double teff=singleMuontriggerEff(baseMuons[0]);

         if(rd(engine) < teff) trigger = true;
      }

      if(!trigger && (baseMuons.size() > 1) && (baseMuons[1]->pT() > 14.7)) // double muon trigger
      {
         // second bite at the cherry for the single muon trigger
         double teffsingle=singleMuontriggerEff(baseMuons[1]);

         if(rd(engine) < teffsingle)
         { 
            trigger = true;
         }
         else  // actually check dimuon trigger
         {
            double teffdouble=doubleMuontriggerEff(baseMuons[0])*doubleMuontriggerEff(baseMuons[1]);
            if(rd(engine) < teffdouble)  trigger = true;
         }

      }
      if(!trigger && (baseMuons.size() > 2) && (baseMuons[2]->pT() > 6.5)) // triple muon trigger
      {
         double teffsingle=singleMuontriggerEff(baseMuons[2]);

         if(rd(engine) < teffsingle)
         { 
            trigger = true;
         }
         
         double teffdouble=doubleMuontriggerEff(baseMuons[0])*doubleMuontriggerEff(baseMuons[2]);
         if(rd(engine) < teffdouble)  trigger = true;



         if(!trigger)
         {
            double tefftriple=tripleMuontriggerEff(baseMuons[0])*tripleMuontriggerEff(baseMuons[1])*tripleMuontriggerEff(baseMuons[2]);
            if(rd(engine) < tefftriple)  trigger = true;
         }
      }

      /*
      if( (baseElectrons.size() > 1) && (baseElectrons[1]->pT() > 18.0))  // dielectron trigger
      {
         double teff=0.0;
         double pT= baseElectrons[1]->pT();
         if(pT < 30.0)
         {
            teff = 0.55+(pT-20.0)/10.0*0.35;
         }
         else if(pT < 60.0)
         {  
            teff=0.9+0.07*(pT-30.0)/30.0;
         }
         else
         {
            teff=0.97;
         }
         if(rd(engine) < teff) trigger=true;
      }
      if(!trigger && (baseElectrons.size() >2) && (baseElectrons[2]->pT() > 18.0))  // dielectron trigger, another bite at the cherry
      {
         double teff=0.0;
         double pT= baseElectrons[2]->pT();
         if(pT < 30.0)
         {
            teff = 0.55+(pT-20.0)/10.0*0.35;
         }
         else if(pT < 60.0)
         {  
            teff=0.9+0.07*(pT-30.0)/30.0;
         }
         else
         {
            teff=0.97;
         }
         if(rd(engine) < teff) trigger=true;
      }
      if(!trigger && (baseMuons.size() > 0) && (baseMuons[0]->pT() > 27.3)) // single muon trigger
      {
         if(rd(engine) < 0.7) trigger = true;
      }

      if(!trigger && (baseMuons.size() > 1) && (baseMuons[0]->pT() > 14.7)) // double muon trigger
      {
         if(rd(engine) < 0.75) trigger = true;
      }

      if(!trigger && (baseMuons.size() > 2) && (baseMuons[0]->pT() > 6.5)) // triple muon trigger
      {
         if(rd(engine) < 0.8) trigger = true;
      }

      */
      /*
      if( (baseElectrons.size() > 0) && (baseElectrons[0]->pT() > 18.0)) trigger=true;
      else if( (baseElectrons.size() > 1) && (baseElectrons[1]->pT() > 18.0)) trigger=true;
      else if( (baseElectrons.size() > 2) && (baseElectrons[2]->pT() > 18.0)) trigger=true;
      else if((baseMuons.size() > 0) && (baseMuons[0]->pT() > 27.3)) trigger=true;
      else if((baseMuons.size() > 1) && (baseMuons[1]->pT() > 14.7)) trigger=true;
      else if((baseMuons.size() > 2) && (baseMuons[2]->pT() > 6.5)) trigger=true;

      */
     // else if((signalLeptons[2]->pT()<10.)) trigger=false;
   //}

   ApplyCut(trigger,"Low Trigger");

   if(met > 200) trigger = true;
   ApplyCut(trigger,"High Trigger");

   if(!trigger) return;


   // resonance vetoes in low met regions

   
    //bool BaseCut1 = ( met<200. && ((minmll>3.0 && minmll<3.2) || (minmll>9.&& minmll<12.) ));

   //|| (signalLeptons[2]->pT() < 10.0 )

    bool BaseCut1 = ( ((minmll>3.0 && minmll<3.2) || (minmll>9.&& minmll<12.) ));
    ApplyCut(!BaseCut1,"mll veto");
    //if(BaseCut1) return;
   ApplyCut(signalLeptons[2]->pT()>10.,"Low lepton pT");



   // MET (high met regions) or pT (other)

   // pseudocode claims this, which corresponds just to table 9:
    //bool BaseCut2 = ( met>200. || signalLeptons[2]->pT()>10. ) ;
    // but paper says the triggers are 18 GeV for each electron, or 27.3, 14.7 and 6.5 GeV for muons
    // and in the paper this is refined to 25,20,10 for leptons
    //ApplyCut(BaseCut2,"Base Cut 2");
    //if(!BaseCut2) return;

  

    

   // PLV on 3rd lepton
   //if (leptons[2]->abspid()==11 && !leptons[2].pass(EIsoPLVTight)) return;
   //if (leptons[2].type()==MUON && !leptons[2].pass(MuIsoPLVTight)) return;

   bool presel_offshell=(mll < 75. && maxmll < 75. && minmll > 1.0);

   ApplyCut(presel_offshell,"Offshell Preselection");
   if(!presel_offshell) return;
   ApplyCut(mindR_3L >= 0.4,"Min dR >= 0.4");
   if (mindR_3L < 0.4) return;

   ApplyCut(njets==0,"0 jets");
   ApplyCut(njets>0,"n jets");

   //fill("cutflow", cutflowcounter++);

   double m3l(-1), mT_mZ(-1), mT_minmll(-1), mT2_100_minmll(-1), dRWMET(-1), pT3L(-1), pT3LoverMET(-1), mW_pZB(-1);
   int lfaketype(-1);
   bool anticonv(false), efake(0), mfake(0);

   //====================================================================================================
   // define additional kinematic variables
   m3l = (signalLeptons[0]->mom()+signalLeptons[1]->mom()+signalLeptons[2]->mom()).m();
   mT_mZ = mTcalc_simple(signalLeptons[iW]->mom(), metVec);
   mT_minmll = mTcalc_simple(signalLeptons[jW]->mom(), metVec);

   HEP::P4 sumlepton12=signalLeptons[jZ1]->mom()+signalLeptons[jZ2]->mom();
   //mT2_100_minmll = calcAMT2(leptons[jZ1]+leptons[jZ2], leptons[jW], metVec, 100, 100);
    mT2_100_minmll = asymm_mt2_lester_bisect::get_mT2(sumlepton12.m(),sumlepton12.px(), sumlepton12.py(),  // Visible 1: mass, px, py
                                                  signalLeptons[jW]->mom().m(), signalLeptons[jW]->mom().px(), signalLeptons[jW]->mom().py(),  // Visible 2: mass, px, py
                                                  metVec.px(), metVec.py(), // Missing transverse momentum: x, y
                                                  100.0, 100.0);

   dRWMET = signalLeptons[jW]->mom().deltaR_eta(metVec);
   pT3L = (signalLeptons[0]->mom()+signalLeptons[1]->mom()+signalLeptons[2]->mom()).pT();
   pT3LoverMET = pT3L / met;
   mW_pZB = 0.;
   HEP::P4 wnu_pzb;
   wnu_pzb.setXYZM(metVec.px(), metVec.py(), (signalLeptons[jZ1]->mom()+signalLeptons[jZ2]->mom()).pz() - signalLeptons[jW]->mom().pz(), 0.); // nu_pz assuming W & Z are balanced in pZ.
   mW_pZB = (signalLeptons[jW]->mom() + wnu_pzb).m();
   efake = (signalLeptons[2]->abspid()==11)?1:0;
   mfake = (signalLeptons[2]->abspid()==13)?1:0;
   lfaketype = (efake)?1:2;
   if ( (efake==true && fabs(m3l-mZ)>20. && mindR_OSSF<2.4 && mindR_OSSF>0.6) || (mfake==true) ) anticonv = true;
   
   //====================================================================================================
   // Baseline regions (b-veto added only after CRs/VRs)
   // Preselected: 3L, >=1 OSSF, SUSY2/16 pT/MET sel, dR cleaning (0.4), PLV iso 3rd lepton
  
   //accept("Presel");
   // Preselected: 3L, >=1 OSSF, SUSY2/16 pT/MET sel, dR cleaning (0.4), PLV iso 3rd lepton && offshell

   

  //  ApplyCut(nbjets==0,"B Jet veto");
 //   if(nbjets>0) return;


    // Baseline regions
   bool presellow_0j(false), presellow_nj(false), preselhigh_0j(false), preselhigh_nj(false);

   bool minmllgt12 = (minmll>12.?1:0);

   if ( njets==0 && maxmll < 75.) { // 0j
      if ( met<50. && metSig>1.5 && anticonv && minmllgt12 && minmll<75. ) { presellow_0j = true; }
      else if ( met>50. && metSig>3.0 && minmllgt12 && minmll<75. )        { preselhigh_0j = true;  }
   }
   else if (maxmll < 75.) {         // nj
      bool softlep(true);
      for (int ilep=0; ilep<nleptons; ilep++) { if ( (signalLeptons[ilep]->abspid()==11 && signalLeptons[ilep]->pT()<4.5) || (signalLeptons[ilep]->abspid()==13 && signalLeptons[ilep]->pT()<3.0) ) { softlep = false; break; } }
      //
      if ( met<200. && metSig>3.0 && anticonv && minmllgt12 && minmll<75. ) { presellow_nj = true;  }
      else if ( met>200. && metSig>3.0 && softlep && minmll<75. )           { preselhigh_nj = true;  }
   }

    ApplyCut(presellow_0j,"Presellow_0j");
    ApplyCut(preselhigh_0j,"Preselhigh_0j");
    ApplyCut(presellow_nj,"Presellow_nj");
    ApplyCut(preselhigh_nj,"Preselhigh_nj");


   

   if (!(presellow_0j||presellow_nj||preselhigh_0j||preselhigh_nj)) return;

    //====================================================================================================
   // Signal regions
   //
bool SRlow_0jb(false), SRlow_0jc(false), SRlow_0jd(false), SRlow_0je(false), SRlow_0jf1(false), SRlow_0jf2(false), SRlow_0jg1(false), SRlow_0jg2(false);
   bool SRlow_njb(false), SRlow_njc(false), SRlow_njd(false), SRlow_nje(false), SRlow_njf1(false), SRlow_njf2(false), SRlow_njg1(false), SRlow_njg2(false);
   bool                    SRhigh_0jb(false), SRhigh_0jc(false), SRhigh_0jd(false), SRhigh_0je(false), SRhigh_0jf1(false), SRhigh_0jf2(false), SRhigh_0jg1(false), SRhigh_0jg2(false);
   bool SRhigh_nja(false), SRhigh_njb(false), SRhigh_njc(false), SRhigh_njd(false), SRhigh_nje(false), SRhigh_njf(false),                      SRhigh_njg(false);
   
   // SRlow_0j
   if (presellow_0j) {
      if (maxmll<60. && minmll<40. && signalLeptons[2]->pT()>10. && mT_minmll<60. && pT3LoverMET<1.3) {
         if      (minmll>12. && minmll<15. && mT_minmll<50. && pT3LoverMET<1.1 && mT2_100_minmll<115. && mindR_3L<1.6) { SRlow_0jb = true; ApplyCut(true,"SRlow_0jb"); }
         else if (minmll>15. && minmll<20. && mT_minmll<50. && pT3LoverMET<1.1 && mT2_100_minmll<120. && mindR_3L<1.6) { SRlow_0jc = true; ApplyCut(true,"SRlow_0jc"); }
         else if (minmll>20. && minmll<30. && mT_minmll<50. && pT3LoverMET<1.1 && mT2_100_minmll<130. && mindR_3L<1.6) { SRlow_0jd = true; ApplyCut(true,"SRlow_0jd"); }
         else if (minmll>30.)                                                                                          { SRlow_0je = true; ApplyCut(true,"SRlow_0je"); }
      }
      else if (minmll>40. && minmll<75. && signalLeptons[2]->pT()>15. && (mT_minmll<60.||mT_minmll>90.) && pT3LoverMET<1.4 && m3l>100) {
         if (minmll<60.) {
            if (mT_minmll<60.)      { SRlow_0jf1 = true; ApplyCut(true,"SRlow_0jf1"); }
            else if (mT_minmll>90.) { SRlow_0jf2 = true; ApplyCut(true,"SRlow_0jf2"); }
         }
         else if (minmll>60.) {
            if (mT_minmll<60.)      { SRlow_0jg1 = true; ApplyCut(true,"SRlow_0jg1"); }
            else if (mT_minmll>90.) { SRlow_0jg2 = true; ApplyCut(true,"SRlow_0jg2"); }
         }
      }
   }
   // SRlow_nj
   else if (presellow_nj) {
      if (maxmll<60. && minmll<40. && signalLeptons[2]->pT()>10. && mT_minmll<60. && pT3LoverMET<1.0) {
         if      (minmll>12. && minmll<15. && mT_minmll<50. && mT2_100_minmll<115. && mindR_3L<1.6) { SRlow_njb = true; ApplyCut(true,"SRlow_njb"); }
         else if (minmll>15. && minmll<20. && mT_minmll<50. && mT2_100_minmll<120. && mindR_3L<1.6) { SRlow_njc = true; ApplyCut(true,"SRlow_njc"); }
         else if (minmll>20. && minmll<30. && mT_minmll<50. && mT2_100_minmll<130. && mindR_3L<1.6) { SRlow_njd = true; ApplyCut(true,"SRlow_njd"); }
         else if (minmll>30.)                                                                       { SRlow_nje = true; ApplyCut(true,"SRlow_nje"); }
      }
      else if (minmll>40. && signalLeptons[2]->pT()>15. && (mT_minmll<60.||mT_minmll>90.) && pT3LoverMET<1.2) {
         if (minmll<60.) {
            if (mT_minmll<60.)      { SRlow_njf1 = true; ApplyCut(true,"SRlow_njf1"); }
            else if (mT_minmll>90.) { SRlow_njf2 = true; ApplyCut(true,"SRlow_njf2"); }
         }
         else if (minmll>60. && minmll<75.) {
            if (mT_minmll<60.)      { SRlow_njg1 = true; ApplyCut(true,"SRlow_njg1"); }
            else if (mT_minmll>90.) { SRlow_njg2 = true; ApplyCut(true,"SRlow_njg2"); }
         }
      }
   }
   // SRhigh_0j
   else if (preselhigh_0j && signalLeptons[0]->pT()>25. && signalLeptons[1]->pT()>15. && signalLeptons[2]->pT()>10.) {
      if      (minmll>12. && minmll<15. && mT2_100_minmll<115. && mT_minmll<50.) { SRhigh_0jb  = true; ApplyCut(true,"SRhigh_0jb"); }
      else if (minmll>15. && minmll<20. && mT2_100_minmll<120. && mT_minmll<50.) { SRhigh_0jc  = true; ApplyCut(true,"SRhigh_0jc"); }
      else if (minmll>20. && minmll<30. && mT2_100_minmll<130. && mT_minmll<60.) { SRhigh_0jd  = true; ApplyCut(true,"SRhigh_0jd"); }
      else if (minmll>30. && minmll<40. && mT2_100_minmll<140. && mT_minmll<60.) { SRhigh_0je  = true; ApplyCut(true,"SRhigh_0je"); }
      else if (minmll>40. && minmll<60. && mT2_100_minmll<160. && mT_minmll<70.) { SRhigh_0jf1 = true; ApplyCut(true,"SRhigh_0jf1"); }
      else if (minmll>40. && minmll<60. && mT2_100_minmll<160. && mT_minmll>90.) { SRhigh_0jf2 = true; ApplyCut(true,"SRhigh_0jf2"); }
      else if (minmll>60. && minmll<75. && mT2_100_minmll<175. && mT_minmll<70.) { SRhigh_0jg1 = true; ApplyCut(true,"SRhigh_0jg1"); }
      else if (minmll>60. && minmll<75. && mT2_100_minmll<175. && mT_minmll>90.) { SRhigh_0jg2 = true; ApplyCut(true,"SRhigh_0jg2"); }
   }
   // SRhigh_nj
   else if (preselhigh_nj && pT3LoverMET<1.0) {
      if      (minmll>1.  && minmll<12. && mT2_100_minmll<112. && pT3LoverMET<0.2) { SRhigh_nja = true; ApplyCut(true,"SRhigh_nja"); }
      else if (minmll>12. && minmll<15. && mT2_100_minmll<115. && pT3LoverMET<0.2) { SRhigh_njb = true; ApplyCut(true,"SRhigh_njb"); }
      else if (minmll>15. && minmll<20. && mT2_100_minmll<120. && pT3LoverMET<0.3) { SRhigh_njc = true; ApplyCut(true,"SRhigh_njc"); }
      else if (minmll>20. && minmll<30. && mT2_100_minmll<130. && pT3LoverMET<0.3) { SRhigh_njd = true; ApplyCut(true,"SRhigh_njd"); }
      else if (minmll>30. && minmll<40. && mT2_100_minmll<140. && pT3LoverMET<0.3) { SRhigh_nje = true; ApplyCut(true,"SRhigh_nje"); }
      else if (minmll>40. && minmll<60. && mT2_100_minmll<160. && pT3LoverMET<1.0) { SRhigh_njf = true; ApplyCut(true,"SRhigh_njf"); }
      else if (minmll>60. && minmll<75. && mT2_100_minmll<175. && pT3LoverMET<1.0) { SRhigh_njg = true; ApplyCut(true,"SRhigh_njg"); }
   }

   // grouped
   if (            SRlow_0jb ||SRlow_0jc ||SRlow_0jd ||SRlow_0je ||SRlow_0jf1 ||SRlow_0jf2 ||SRlow_0jg1 ||SRlow_0jg2)  ApplyCut(true,"SRlow_0j");
   if (            SRlow_njb ||SRlow_njc ||SRlow_njd ||SRlow_nje ||SRlow_njf1 ||SRlow_njf2 ||SRlow_njg1 ||SRlow_njg2)  ApplyCut(true,"SRlow_nj");
   if (            SRhigh_0jb||SRhigh_0jc||SRhigh_0jd||SRhigh_0je||SRhigh_0jf1||SRhigh_0jf2||SRhigh_0jg1||SRhigh_0jg2) ApplyCut(true,"SRhigh_0j");
   if (SRhigh_nja||SRhigh_njb||SRhigh_njc||SRhigh_njd||SRhigh_nje||SRhigh_njf              ||SRhigh_njg              ) ApplyCut(true,"SRhigh_nj");



    // Done!
    return;
}

