

#ifndef DV_h
#define DV_h
#include "include/BaseAnalysis.h"
#include "include/isolation.h"
#include "include/HEPData.h"

using namespace std;

#ifndef MAdouble64
#define MAdouble64 double
#endif

//bool found_ee;
//bool found_emu;
//bool found_mumu;
const static bool USE_REWEIGHT=false;
static std::uniform_real_distribution<double> rd(0.0,1.0);

inline bool mycomp(const HEP::Particle* p1, const HEP::Particle* p2)
	{
		return (p1->pT() > p2->pT());  // to sort with highest pt first
	};

class dv_candidate
    {
    private:
      HEP::P4 position;
	
    public:
      std::vector <const HEP::Particle *> particles;
	
	bool single_photon_trigger = false;
	bool di_photon_trigger = false;
	bool muon_trigger = false;
	double weight_multiplier;
	

	void sort_reverse_pt()
	{	
		std::sort(particles.begin(),particles.end(),mycomp);
	}

      HEP::P4 get_momentum () {
	HEP::P4 p_tot (0.,0.,0.,0.);
	    for (int i=0; i<particles.size(); i++)
		p_tot += particles[i]->mom();
	    return p_tot;
	}
	
	double get_invariant_mass () {
	    return get_momentum().m();
	};

	double get_pt () {
	    return get_momentum().pT();
	};	

      HEP::P4 get_position () const { return position; };

      void set_particle (const HEP::Particle * part) {
	    if (particles.size() > 0)
		particles.clear();
	    add_particle(part);
	}
      void add_particle (const HEP::Particle * part) {
	    particles.push_back(part);
	}

	double d_xy () {
	  //double dxy = sqrt(get_position().px2() + get_position().py2());
	  double dxy=position.pT();
	    return dxy;
	}

	int Nelectrons () {
	    int Nelectrons = 0;
	    for (int i=0; i<particles.size(); i++) {
	      //if(dynamic_cast<const RecLeptonFormat *>(particles[i]) != 0)
	      if(abs(particles[i]->pid()) == 11) Nelectrons++;
	    }
	    return Nelectrons;
	}

	int N_leptons () {
	    int Nleptons = 0;
	    for (int i=0; i<particles.size(); i++) {
	      //if(dynamic_cast<const RecLeptonFormat *>(particles[i]) != 0)
	      if(PDG::isLepton(particles[i]->pid())) Nleptons++;
	    }
	    return Nleptons;
	}
	
      dv_candidate (const HEP::Particle * part) {
	    particles.push_back(part);
	    //int tpid=part->pid();
		position = part->prod_vertex();
	    //if(PDG::isLepton(tpid) || (tpid==22)) position = part->prod_vertex();
	   
	    
	    // if (dynamic_cast<const RecLeptonFormat *>(part) != 0) {
	    // 	position = dynamic_cast<const RecLeptonFormat *>(part)->ProductionVertex();
	    // }
	    // else if (dynamic_cast<const RecPhotonFormat *>(part) != 0) {
	    // 	position = dynamic_cast<const RecPhotonFormat *>(part)->ProductionVertex();
	    // }
	}

	dv_candidate()
	{
		weight_multiplier=1.0;
		position=HEP::P4();
		particles.clear();
	}
	
	
};



void update_vertex_types(vector<dv_candidate> & dv_candidates, bool & found_ee, bool & found_emu, bool & found_mumu);

MAdouble64 DVRecoEfficiencyZPrime100GeVToEE (MAdouble64 r_xy, MAdouble64 pT) ;

MAdouble64 DVRecoEfficiencyZPrime100GeVToEMU (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyZPrime100GeVToEMU (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyZPrime100GeVToMUMU (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyZPrime250GeVToEE (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyZPrime250GeVToEMU (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyZPrime250GeVToMUMU (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyZPrime500GeVToEE (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyZPrime500GeVToEMU (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyZPrime500GeVToMUMU (MAdouble64 r_xy, MAdouble64 pT) ;

MAdouble64 DVRecoEfficiencyZPrime750GeVToEE (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyZPrime750GeVToEMU (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyZPrime750GeVToMUMU (MAdouble64 r_xy, MAdouble64 pT);
MAdouble64 DVRecoEfficiencyZPrime1000GeVToEE (MAdouble64 r_xy, MAdouble64 pT);
MAdouble64 DVRecoEfficiencyZPrime1000GeVToEMU (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyZPrime1000GeVToMUMU (MAdouble64 r_xy, MAdouble64 pT);

MAdouble64 DVRecoEfficiencyRPVEE (MAdouble64 r_xy, MAdouble64 mDV, MAdouble64 pT) ;

MAdouble64 DVRecoEfficiencyRPVEMU (MAdouble64 r_xy, MAdouble64 mDV, MAdouble64 pT);
MAdouble64 DVRecoEfficiencyRPVMUMU (MAdouble64 r_xy, MAdouble64 mDV, MAdouble64 pT);

bool vertexMatch (const HEP::P4& v1, const HEP::P4& v2);

bool fullMatch(const HEP::Particle &p1, const HEP::Particle &p2);

double drCos (HEP::P4 v1,HEP::P4 v2);

template<typename T1> bool crVeto1(vector<T1> &v1, const double &drmin)
{
    // Determining with objects should be removed
    for (int j=0;j<v1.size();j++)
	for (int i=j+1;i<v1.size();i++)
	    // if (v1[i]->dr(v1[j]) < drmin)
	    if (drCos(v1[i]->mom(),v1[j]->mom()) < drmin)
		return true;
    return false;
}

template<typename T1, typename T2> bool crVeto2(vector<T1> &v1, vector<T2> &v2, const double &drmin)
{
    // Determining with objects should be removed
    for (int j=0;j<v1.size();j++)
	for (int i=0;i<v2.size();i++)
	    // if (v2[i]->dr(v1[j]) < drmin)
	    if (drCos(v2[i]->mom(),v1[j]->mom()) < drmin)
		return true;
    return false;
}

bool cosmicRayVeto(vector<HEP::Particle*> &finalStateElectrons, vector<HEP::Particle*> &finalStateMuons);


double materialVetoWeight(HEP::P4 &vertex);

double disabledPixelModulesVetoWeight(HEP::P4 &vertex);

void  apply_d_xy_criterion (vector<dv_candidate> &DV_candidates, double d_xy_min);

void apply_fiducial_volume_criterion (vector<dv_candidate> &DV_candidates, double radius, double half_length);

double apply_dis_pixel_mod_veto (vector<dv_candidate> &DV_candidates, std::mt19937 &engine);

double apply_material_veto (vector<dv_candidate> &DV_candidates, std::mt19937 &engine);

void require_leptons (vector<dv_candidate> &DV_candidates, int N_min) ;

void require_invariant_mass (vector<dv_candidate> &DV_candidates, int m_min);

void require_opposite_charge (vector<dv_candidate> &DV_candidates);

void apply_trigger_matching (vector<dv_candidate> &DV_candidates);

void apply_preselection_matching (vector<dv_candidate> &DV_candidates) ;
bool is_loose_photon (const HEP::Particle *photon, const HEP::Event* event);
#endif
