








import argparse

try:
    parser = argparse.ArgumentParser(description='Please give the name of the json file.')
    parser.add_argument("--significance", help="Include significance", action="store_true",default=False)
    parser.add_argument('scanjson',
                        metavar='File', type=str, help='Input file name')
    parser.add_argument('--Output', type=str, help='Output file name',action="store",default=None)
    args = parser.parse_args()
except Exception as e:
    print("Please give an input file, "+str(e))
    raise SystemExit


import json
from collections import OrderedDict
try:
    with open(args.scanjson,'r') as scan_json_file:
        scan_dict=json.load(scan_json_file, object_pairs_hook=OrderedDict)
except Exception as e:
    print("Failed to load json: "+e)
    raise SystemExit

"""
Now we want the variables, csv file, YAML file (for the list of analyses)

"""

import os
import pandas as pd ## for storing outputs and reading csv files for the variables
scan_name=scan_dict["Setup"]["RunName"]

if not os.path.isdir(scan_name):
    print(f"Could not find scan <{scan_name}> outputs" )
    raise SystemExit

if args.Output is not None:
    output_file=args.Output
else:
    output_file=os.path.join(scan_name,"HAstats.csv")


use_csv=True
if "csv" not in scan_dict["Setup"] or not eval(scan_dict["Setup"]["csv"]):
    print("This code currently only supports scans storing csv inputs/outputs")
    use_csv=False

if use_csv:
    if 'csv name' in scan_dict['Setup']:
        merge_csv=os.path.join(scan_dict['Setup']['RunName'], 'Spectrum_Files',scan_dict['Setup']['csv name'])
    elif 'Output File' in scan_dict['Setup']:
        merge_csv = os.path.join(scan_dict['Setup']['RunName'], 'Spectrum_Files',scan_dict['Setup']['Output File']+'.csv')
    else:
	    merge_csv = os.path.join(scan_dict['Setup']['RunName'], 'Spectrum_Files','SLHA_Output.csv')
    csvdata=pd.read_csv(merge_csv)

BSMArt_output='SLHA_Output'
if 'Output File' in scan_dict['Setup']:
    BSMArt_output=scan_dict['Setup']['Output File']

store_everything=False
if 'StoreEverything' in scan_dict['Setup']:
    store_everything=eval(scan_dict['Setup']['StoreEverything'])

store_separate=False
if 'StoreSeparateFiles' in scan_dict['Setup']:
    store_separate=eval(scan_dict['Setup']['StoreSeparateFiles'])

if not store_everything and not store_separate:
    print("This script does not support results stored in only one file")
    raise SystemExit

if not store_everything and store_separate and not use_csv:
    print("Need to have a csv file if we're reading the spectrum files")
    raise SystemExit

"""
input_variables=[]
for var in scan_dict["Variables"]:
    input_variables.append(var)
"""


"""

Now import the stats modules and set all that up 
"""

analyses=[]

for codedata in scan_dict['Codes'].values():
    if eval(codedata['Run']) and 'YAML file' in codedata:
        yamlfile=codedata['YAML file']
        import yaml

        if os.path.exists(yamlfile):
            with open(yamlfile, 'r') as yfile:
                yaml_data = yaml.safe_load(yfile)
        elif store_separate:  
            outdirs=os.listdir(os.path.join(scan_name,'All_Outputs'))
            yampath=os.path.join(scan_name,'All_Outputs',outdirs[0],yamlfile)
            if os.path.exists(yampath):
                with open(yampath, 'r') as yfile:
                    yaml_data = yaml.safe_load(yfile)
            else:
                print('Could not locate yaml file')
                raise SystemExit
        else:
            print('Could not locate yaml file')
            raise SystemExit
        for analysis in yaml_data['analyses']:
            analyses.append(analysis.upper())

        if 'Efficiency Filename' in yaml_data['settings']:
            efficiency_file=yaml_data['settings']['Efficiency Filename']
        else:
            print('Could not locate Efficiency file info')
            raise SystemExit
        break

## Now initialise the analyses
#import pkgutil

try:
    from importlib import resources as impresources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as impresources

#from . import Analyses
#from . import Data

import Analyses as HAAnalyses
import HAstats


handlerdict={}

for analysis in analyses:
    infojsonname=analysis+'_info.json'
    #thisinfo=pkgutil.get_data(__name__, "Analyses/"+infojsonname)
    #thisinfo=impresources.files(HAAnalyses).joinpath(infojsonname).open_text()

    try:
        with impresources.open_text(HAAnalyses,infojsonname) as thisinfo:
            analysis_infodict=json.load(thisinfo)
    except Exception as e:
        print(f"Failed to initialised analysis {analysis}:"+str(e))
        continue
    handlerdict[analysis]=HAstats.AnalysisStats(analysis_infodict)



print("Processing points")

""" 
Now start processing points
"""

points=[]

if store_separate:
    outdirs=os.listdir(os.path.join(scan_name,'All_Outputs'))

    for point_dir in outdirs:
        id=int(point_dir.split('_')[-1])
        
        if use_csv:
            try:
                data=pd.DataFrame(csvdata.loc[csvdata['ID']==id]).values.tolist()[0]
            #dataline=','.join(list(map(str,data)))
                #print(data) 
            except Exception as e:
                print("Failed to find data "+str(e))
                data=[]
        points.append({'efficiencyfile':os.path.join(scan_name,'All_Outputs',point_dir,efficiency_file),'id':id,'data':data})
        
else: ## we're using csv
    for row in csvdata:
        try:
            id=row['ID']
        except:
            print('Need an ID number!')
            raise SystemExit
        
        efffile=os.path.join(scan_name,'Spectrum Files',BSMArt_output+'_'+str(id))
        data=pd.DataFrame(csvdata.loc(csvdata['ID']==id)).to_list()[0]
        points.append({'efficiencyfile':efffile,'id':id,'data':data})

#print(points)
import zslha

#OF=open(output_file,'w')
if os.path.exists(output_file):
    os.remove(output_file)



if use_csv:
    columns=list(csvdata)
    
    #os.system('head -n1 '+merge_csv+' > '+output_file)
else:
    columns=['ID']

for analysis in handlerdict:
    if handlerdict[analysis].type=='pyhf':
        columns.append(analysis+"_CLsobs")
        columns.append(analysis+"_CLsexp")
        if args.significance:
            columns.append(analysis+"_pval")
    elif handlerdict[analysis].type=='MA5spey':
        columns.append(analysis+"_muobs")
        columns.append(analysis+"_muexp")
        if args.significance:
            columns.append(analysis+"_pval")
            columns.append(analysis+"_mumax")
outline=','.join(list(map(str,columns)))
with open(output_file,'w') as OF:
    OF.write(outline+'\n')


for point in points:
    
    data=list(point['data'])
    print(data)
    try:
        for analysis in handlerdict:
            print("Processing "+analysis+ " in file "+point["efficiencyfile"])
            results = handlerdict[analysis].getstats(point['efficiencyfile'],inputxs=None,with_significance=args.significance)
            print('Obtained '+str(results))
            if handlerdict[analysis].type=='pyhf':
                data.append("%.5f"%results['CLs obs'])
                data.append("%.5f"%results['CLs exp'][2])  ## expected values are -2, -1, 0, 1, 2 sigma list
                if args.significance:
                    data.append("%.5e"%results['pval'])
            elif handlerdict[analysis].type=='MA5spey':
                data.append("%.5e"%results['uppermuobs'])
                data.append("%.5e"%results['uppermuexp'])
                if args.significance:
                    data.append("%.5e"%results['pval'])
                    data.append("%.5e"%results['mumax'])
    except Exception as e:
        print("Failed to process: "+str(e))
        continue

    outline=','.join(list(map(str,data)))
    with open(output_file,'a') as OF:
        OF.write(outline+'\n')
    
