import os
import sys

import json
import math
#here=os.getcwd()

from collections import OrderedDict



"""
Script to combine json files produced by HackAnalysis into one
Very useful when we run HackAnalysis more than once on a point
"""

def addresults(in1,in2):
    res=OrderedDict()
    if len(in1) == 0: ## i.e. it's the first one
        res=in2
        return res
    
    for analysis in in1:
        if analysis not in in2:
            print('analysis %s not in second input!' %s)
            continue
        #print("combining analysis %s" %analysis)
        res[analysis]=OrderedDict()

        N1=in1[analysis]['nevents']
        N2=in2[analysis]['nevents']
        res[analysis]['nevents']=N1+N2
        xs1=float(in1[analysis]['xs'])
        xs2=float(in1[analysis]['xs'])
        meanxs=(float(N1)*xs1+float(N2)*xs2)/(float(N1+N2))
        res[analysis]['xs']=meanxs
        res[analysis]["Regions"] = OrderedDict()
        for region in in1[analysis]["Regions"]:
            regionout={}

            regionout["N0"]=in1[analysis]["Regions"][region]["N0"]+in2[analysis]["Regions"][region]["N0"]

            initevents=in1[analysis]["Regions"][region]["Cuts"][0]["events"]+in2[analysis]["Regions"][region]["Cuts"][0]["events"]
            initweights=in1[analysis]["Regions"][region]["Cuts"][0]["weightsum"]+in2[analysis]["Regions"][region]["Cuts"][0]["weightsum"]
            initsqweights=in1[analysis]["Regions"][region]["Cuts"][0]["sqweightsum"]+in2[analysis]["Regions"][region]["Cuts"][0]["sqweightsum"]

            outcuts=[]
            if len(in1[analysis]["Regions"][region]["Cuts"]) != len(in2[analysis]["Regions"][region]["Cuts"]):
                print("Something wrong with cuflow %s" % region)
                continue


            eff=0.0
            weightsum=0.0
            sqweightsum=0.0
            staterr=0.0
            for cut1,cut2 in zip(in1[analysis]["Regions"][region]["Cuts"],in2[analysis]["Regions"][region]["Cuts"]):
                thiscut={}
                if cut1["name"] != cut2["name"]:
                    print("Mismatched names in cutflow %s! %s, %s" %(region,cut1["name"],cut2["name"]))
                    raise

                thiscut["name"]=cut1["name"]
                thiscut["events"]=cut1["events"]+cut2["events"]

                weightsum=cut1["weightsum"]+cut2["weightsum"]
                sqweightsum=cut1["sqweightsum"]+cut2["sqweightsum"]
                thiscut["weightsum"]=weightsum
                thiscut["sqweightsum"]=sqweightsum

                if abs(initweights) > 0:
                    eff=weightsum/initweights
                else:
                    eff=0.0
                if sqweightsum > 0:
                    neff=weightsum**2/sqweightsum
                else:
                    neff=0
                if neff > 0:
                    staterr=math.sqrt((1.0-eff)/neff)*100.0
                else:
                    staterr=100 # no clue!
                thiscut["efficiency"]=eff
                thiscut["stat"]=staterr
                thiscut["weightsum"]=weightsum
                thiscut["sqweightsum"]=sqweightsum
                outcuts.append(thiscut)

            regionout["Cuts"]=outcuts
            #regionout["weightsum"]=float(in1[analysis]["Regions"][region]["weightsum"])+float(in2[analysis]["Regions"][region]["weightsum"])
            #regionout["sqweightsum"]=float(in1[analysis]["Regions"][region]["sqweightsum"])+float(in2[analysis]["Regions"][region]["sqweightsum"])
            regionout["efficiency"]=eff
            regionout["weightsum"]=weightsum
            regionout["sqweightsum"]=sqweightsum
            regionout["stat"]=staterr
            
            res[analysis]["Regions"][region]=regionout
    return res


