

import numpy as np


import random
import math

import os

import math
import sys

import re
from collections import OrderedDict

#from scipy.interpolate import UnivariateSpline
#from scipy.interpolate import interp1d
#from scipy import interpolate

#from math import log10

try:
    import scipy.stats
except ImportError:
    print("Cannot import scipy")
    raise


from statistics import mean







def Proportions(NumObserved, ExpectedBG, BGError, SigHypothesis, NumToyExperiments):
       

    ### NumToyExperiments is a maximum. We'll keep going until we have an accurate enough answer.
    
    # generate a set of expected-number-of-background-events, one for each toy
    # experiment, distributed according to a Gaussian with the specified mean
    # and uncertainty
    min_proportional_error= 0.025
    min_absolute_error= 0.005
    properrB=1.0
    properrSplusB=1.0
    pts=0
    batch_size=5000
    pbs = []
    
    pSplusBs = []
    totalB=0
    totalBplusS=0
    ErrorTooLarge=True
    ## proportional error on background, absolute error on signal
    while ErrorTooLarge and pts < NumToyExperiments:
        
        pts = pts + batch_size
        ExpectedBGs = scipy.stats.norm.rvs(loc=ExpectedBG, scale=BGError, size=batch_size)

        # Ignore values in the tail of the Gaussian extending to negative numbers
        ExpectedBGs = [value for value in ExpectedBGs if value > 0]
    
        # For each toy experiment, get the actual number of background events by
        # taking one value from a Poisson distribution created using the expected
        # number of events.
        ToyBGs = scipy.stats.poisson.rvs(ExpectedBGs)
        ToyBGs = list(map(float, ToyBGs))

        # The probability for the background alone to fluctutate as LOW as
        # observed = the fraction of the toy experiments with backgrounds as low as
        # observed = p_b.
        # NB (1 - this p_b) corresponds to what is usually called p_b for CLs.
        #print(ToyBGs)
        #print(NumObserved)
        #https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.percentileofscore.html
        # 'weak': This kind corresponds to the definition of a cumulative distribution function. A percentileofscore of 80% means that 80% of values are less than or equal to the provided score.
        #p_b = scipy.stats.percentileofscore(ToyBGs, NumObserved, kind='weak')*.01
        batch_p_b=scipy.stats.percentileofscore(ToyBGs, NumObserved, kind='weak')*.01
        totalB=totalB+len(ToyBGs)
        #print("pb: %.3f" %p_b)
        # Toy MC for background+signal
        ExpectedBGandS = [expectedbg + SigHypothesis for expectedbg in ExpectedBGs]
        ExpectedBGandS = [x for x in ExpectedBGandS if x > 0]
        
        if len(ExpectedBGandS)==0:
            continue
            #return 0., p_b
        ToyBplusS = scipy.stats.poisson.rvs(ExpectedBGandS)
        ToyBplusS = list(map(float, ToyBplusS))
        totalBplusS=totalBplusS+len(ToyBplusS)
        
        # Calculate the fraction of these that are >= the number observed,
        # giving p_(S+B). Divide by (1 - p_b) a la the CLs prescription.
        batch_p_SplusB = scipy.stats.percentileofscore(ToyBplusS, NumObserved, kind='weak')*.01
        pSplusBs.append(batch_p_SplusB*len(ToyBplusS)/batch_size)
        pbs.append(batch_p_b*len(ToyBGs)/batch_size)
        p_b=np.mean(pbs)
        p_SplusB=np.mean(pSplusBs)

        ErrorTooLarge=False
        ### Proportional error on background, because if p_b is small need to be quite sure of it ... 
        if p_b > 0.1:
            ## Accept absolute error
            properrB=math.sqrt(p_b*(1-p_b)/(totalB))
            if properrB > min_absolute_error:
                ErrorTooLarge=True
        elif p_b > 0.0:
            ## Require proportional error
            properrB=math.sqrt((1-p_b)/(p_b*totalB))
            if properrB > min_proportional_error:
                ErrorTooLarge=True
            #properrB=math.sqrt(p_b*(1-p_b)/(totalB))
        else:
            properrB=0.0
            ## Not sure what to do about the error here ... keep going?
            ErrorTooLarge=True
            
        ## Not proportional error anymore, just absolute error on the probability. Actually that is all we care about
        # if p_SplusB > 0.0:
        #     #properrSplusB=math.sqrt((1-p_SplusB)/(p_SplusB*totalBplusS))
        #     properrSplusB=math.sqrt(p_SplusB*(1-p_SplusB)/(totalBplusS))
        # else:
        #     properrSplusB=0.0

        # Check for uncertainty on signal, 
        # but also check we are not wasting our time on very low or very high probability points
        
        if p_SplusB > 0.1:
            ## Accept absolute error
            properrSplusB=math.sqrt(p_SplusB*(1-p_SplusB)/(totalBplusS))
            if properrSplusB > min_absolute_error:
                ErrorTooLarge=True
            ## Don't waste time on points with small signal
            if p_b > 0.1:
                if (p_SplusB-3*properrSplusB)/p_b > 0.5:
                    ErrorTooLarge=False 
        elif p_SplusB > 0.0:
            ## Require proportional error
            properrSplusB=math.sqrt((1-p_SplusB)/(p_SplusB*totalBplusS))
            if properrSplusB > min_proportional_error:
                ErrorTooLarge=True
            #properrB=math.sqrt(p_b*(1-p_b)/(totalB))
            ## Don't waste time on points with very large signal!
            if p_b > 0.1:
                if p_SplusB*(1+3*properrSplusB)/p_b < 0.01:
                    ErrorTooLarge=False 
        else:
            properrSplusB=0.0
            ## Presumably this is ok that we have a very small probability of signal
            #ErrorTooLarge=True
        
        
        
        
        #err=max(properrB,properrSplusB)
    #print("%d %.4f (%.4f) %.4f (%.4f)" %(pts,p_b,properrB,p_SplusB,properrSplusB))
    return p_SplusB, p_b


def CLs(Observed, ExpectedBGs, BGErrors, SigHypotheses, NumToyExperiments):
    p_sb=1.0
    p_b=1.0

    for x,y,z,u in zip(Observed,ExpectedBGs,BGErrors,SigHypotheses):
        #print(x,y,z,u)
        try:
            tp_sb,tp_b = Proportions(x,y,z,u,NumToyExperiments)
            if tp_sb > tp_b:
                tCLs = 0.0
            else:
                tCLs = 1.0 - tp_sb/tp_b
            #print("s+b: %.3f b: %.3f, region CLs: %.3f" %(tp_sb,tp_b,tCLs))
        except:
            print("failed")
            return False

        p_sb=p_sb*tp_sb
        p_b=p_b*tp_b

    if p_sb>p_b:
        #print(" CLs: %.3f" %(0.0))
        return 0.
    else:
        #print(" CLs: %.3f" %(1.-(p_sb / p_b)))
        return 1.-(p_sb / p_b) # 1 - CLs





def GetLimits(Observed,backgrounds,BGerrors,signals):
    ## First get the raw CLs
    temptoys=500000
    mycls=CLs(Observed,backgrounds,BGerrors,signals,temptoys)

    return mycls

def GetSigLimit(Observed,backgrounds,BGerrors,refSigs):

    #refxs=XSdata_chch.getxs(pointmass)+XSdata_chnu.getxs(pointmass)
    temptoys=100000
    if max(refSigs) ==0.0:
        raise
    def GetSig95(x):  
        nsigs=[x * y for y in refSigs]
        if max(nsigs) < 0.001:
            return -0.95
        return CLs(Observed,backgrounds,BGerrors,nsigs,temptoys)-0.95

    mlow = 1
    mhig = 1

    while GetSig95(mlow)>0.0:
        mlow=mlow*0.1
 
    while GetSig95(mhig)<0.0:
        mhig=mhig*10.0

    #print("Testing between %.3f and %.3f " %(mlow,mhig))
    try:
        m95 = scipy.optimize.brentq(GetSig95,mlow,mhig,xtol=mlow/100.)
    except Exception as e:
        print(("%f, %f Brent failure: " %(mlow,mhig))+str(e))
        return 10.0

    return m95


#def CLs(Observed, ExpectedBGs, BGErrors, SigHypotheses, NumToyExperiments)


class HAnative:
    """
    Load the data about backgrounds, uncertainties and observations
    """
    def __init__ (self, inputdict):
        #self.inputs=inputdict
        self.bgs=[]
        self.bgerrors=[]
        self.observations=[]
        self.combineregions=[]
        self.lumis={}
        if "CombineRegions" in inputdict:
            for tocombine in inputdict["CombineRegions"]:
                combinations=[]
                backs=[]
                deltabacks=[]
                obs=[]

                for region in inputdict["AllRegions"][tocombine]:
                    if "lumi" in inputdict["RegionData"][region]:
                        self.lumis[region]=inputdict["RegionData"][region]["lumi"]
                    else:
                        self.lumis[region]=inputdict["Lumi"]
                    combinations.append[region]
                    backs.append(inputdict["RegionData"][region]["nb"])
                    deltabacks.append(inputdict["RegionData"][region]["deltanb"])
                    obs.append(inputdict["RegionData"][region]["observed"])
                self.combineregions.append(combinations)
                self.bgs.append(backs)
                self.bgerrors.append(deltabacks)
                self.observations.append(obs)
        else:
            for region in inputdict["RegionData"]:
                if "lumi" in inputdict["RegionData"][region]:
                    self.lumis[region]=inputdict["RegionData"][region]["lumi"]
                else:
                    self.lumis[region]=inputdict["Lumi"]
                self.combineregions.append([region])
                self.bgs.append([inputdict["RegionData"][region]["nb"]])
                self.bgerrors.append([inputdict["RegionData"][region]["deltanb"]])
                self.observations.append([inputdict["RegionData"][region]["observed"]])
            
        
    def getlimitsfromsignals(self,signal_data,with_upper_limits=True):
        results={}
        allresults=[]
        bestexp=0.0
        bestlimit=0.0
        bestregion=None
        allsignals=[]
        for n,tocombine in enumerate(self.combineregions):

            theseresults={}

            signals=[]
            for region in tocombine:
                sig=signal_data[region]
                signals.append(sig)
            allsignals.append(signals)
            expected=CLs(self.bgs[n],self.bgs[n],self.bgerrors[n],signals,500000)
            observed=CLs(self.observations[n],self.bgs[n],self.bgerrors[n],signals,500000)


            

            theseresults['CLs exp']=expected
            theseresults['CLs obs']=observed
            theseresults['Regions']=tocombine
            """
            if with_upper_limits:
                try:
                    sig95exp=GetSigLimit(self.bgs[n],self.bgs[n],self.bgerrors[n],signals)
                    sig95obs=GetSigLimit(self.observations[n],self.bgs[n],self.bgerrors[n],signals)
                    theseresults['mu95exp']=sig95exp
                    theseresults['mu95obs']=sig95obs    
                except Exception as e:
                    print("Signal limit finding failed, "+str(e))
                    theseresults['mu95exp']=10
                    theseresults['mu95obs']=10
            """    
                


            if expected > bestexp:
                bestregion=n
                bestexp=expected
                bestlimit=observed
            #print(tocombine)
            #print(theseresults)
            allresults.append(theseresults)

        
        results['All Regions'] = allresults
        if bestregion is not None:
            #print("Best region "+str(bestregion)+" : "+str(self.combineregions[bestregion]))
            results['CLs exp']=allresults[bestregion]["CLs exp"]
            results['CLs obs']=allresults[bestregion]["CLs obs"]
            
            results['Best']=bestregion
            results['Best Regions']=self.combineregions[bestregion]
            if with_upper_limits:
                #results['mu95exp']=allresults[bestregion]['mu95exp']
                #results['mu95obs']=allresults[bestregion]['mu95obs']
                try:
                    sig95exp=GetSigLimit(self.bgs[bestregion],self.bgs[bestregion],self.bgerrors[bestregion],allsignals[bestregion])
                    sig95obs=GetSigLimit(self.observations[bestregion],self.bgs[bestregion],self.bgerrors[bestregion],allsignals[bestregion])
                    results['mu95exp']=sig95exp
                    results['mu95obs']=sig95obs    
                except Exception as e:
                    print("Signal limit finding failed, "+str(e))
                    theseresults['mu95exp']=10
                    theseresults['mu95obs']=10

        else:
            results['CLs exp']=0.0
            results['CLs obs']=0.0
            if with_upper_limits:
                results['s95exp']=100000
                results['s95obs']=100000
                results['mu95exp']=10
                results['mu95obs']=10

        return results

    def getlimits(self,efficiency_data,xs,with_upper_limits=True):
        results={}
        allresults=[]
        bestexp=0.0
        bestlimit=0.0
        bestregion=None
        for n,tocombine in enumerate(self.combineregions):

            theseresults={}

            signals=[]
            for region in tocombine:
                sig=efficiency_data[region]*self.lumis[region]*xs
                signals.append(sig)

            expected=CLs(self.bgs[n],self.bgs[n],self.bgerrors[n],signals,500000)
            observed=CLs(self.observations[n],self.bgs[n],self.bgerrors[n],signals,500000)


            

            theseresults['CLs exp']=expected
            theseresults['CLs obs']=observed
            theseresults['Regions']=tocombine
            
            if with_upper_limits:
                sig95exp=GetSigLimit(self.bgs[n],self.bgs[n],self.bgerrors[n],signals)
                sig95obs=GetSigLimit(self.observations[n],self.bgs[n],self.bgerrors[n],signals)
                theseresults['mu95exp']=sig95exp
                theseresults['mu95obs']=sig95obs
                theseresults['s95exp']=sig95exp*xs
                theseresults['s95obs']=sig95obs*xs
            

            if expected > bestexp:
                bestregion=n
                bestexp=expected
                bestlimit=observed

            allresults.append(theseresults)

        results['All Regions'] = allresults
        if bestregion is not None:
            results['CLs exp']=allresults[bestregion]["CLs exp"]
            results['CLs obs']=allresults[bestregion]["CLs obs"]
            results['Best']=bestregion
            results['Best Regions']=self.combineregions[bestregion]
            if with_upper_limits:
               
                results['s95exp']=allresults[bestregion]['s95exp']
                results['s95obs']=allresults[bestregion]['s95obs']
                results['mu95exp']=allresults[bestregion]['mu95exp']
                results['mu95obs']=allresults[bestregion]['mu95obs']


        else:
            results['CLs exp']=0.0
            results['CLs obs']=0.0
            if with_upper_limits:
                results['s95exp']=100000
                results['s95obs']=100000
                results['mu95exp']=10
                results['mu95obs']=10

        return results

        
