import os,sys,math

import numpy as np
import json
import spey

try:
    from lxml import ET
except Exception as err:

    try:
        import xml.etree.ElementTree as ET
    except Exception as err:
        print('lxml or xml not available... the ma5 module cannot be used')
        print(str(err))

class HandleMA5:
    def __init__(self):
        self.analysis=""
        self.global_likelihoods_switch=True
        self.error_extrapolation='linear'
        self.cov_config={}

    def __init__(self, etree, analysis, extrapolated_lumi='default'):
        self.analysis=""
        self.global_likelihoods_switch=True
        self.error_extrapolation='linear'
        self.cov_config={}
        self.header_info_file(etree, analysis, extrapolated_lumi)


    def header_info_file(self, etree, analysis, extrapolated_lumi='default'):
        print('Reading info from the file related to '+analysis + '...')
        self.analysis=analysis
        ## checking the header of the file
        info_root = etree.getroot()
        if info_root.tag != "analysis":
            print('Invalid info file (' + analysis+ '): <analysis> tag.')
            return -1,-1,-1
        if info_root.attrib["id"].lower() != analysis.lower():
            print('Invalid info file (' + analysis+ '): <analysis id> tag.')
            return -1,-1,-1
        ## extracting the information
        lumi         = 0
        lumi_scaling = 1.
        regions      = []
        self.cov_config  = {}
        self.pyhf_config = {}
        regiondata   = {}
        # Getting the description of the subset of SRs having covariances
        # Now the cov_switch is activated here
        if "cov_subset" in info_root.attrib and self.global_likelihoods_switch:
            self.cov_config[info_root.attrib["cov_subset"]] = dict(cov_regions = [],
                                                                   covariance = [])
        # activate pyhf
        # no, don't
        """
        if self.main.recasting.global_likelihoods_switch and self.main.session_info.has_pyhf and self.cov_config == {}:
            try:
                self.pyhf_config = self.pyhf_info_file(info_root)
                print(str(self.pyhf_config))
            except Exception as err:
                print('Check pyhf_info_file function!\n' + str(err))
                self.pyhf_config = {}
        """

        ## first we need to get the number of regions
        for child in info_root:
            # Luminosity
            if child.tag == "lumi":
                try:
                    lumi = float(child.text)
                    if extrapolated_lumi!='default':
                        print('Setting lumi scaling')
                        lumi_scaling = round(extrapolated_lumi/lumi,8)
                        lumi=lumi*lumi_scaling
                except Exception as err:
                    print('Invalid info file (' + analysis+ '): ill-defined lumi')
                    print(str(err))
                    return -1, -1, -1
                print('The luminosity of ' + analysis + ' is ' + str(lumi) + ' fb-1.')
            # regions
            if child.tag == "region" and ("type" not in child.attrib or child.attrib["type"] == "signal"):
                if "id" not in child.attrib:
                    print('Invalid info file (' + analysis+ '): <region id> tag.')
                    return -1, -1, -1
                if child.attrib["id"] in regions:
                    print('Invalid info file (' + analysis+ '): doubly-defined region.')
                    return -1, -1, -1
                regions.append(child.attrib["id"])
                # If one covariance entry is found, the covariance switch is turned on
                if self.global_likelihoods_switch:
                    for grand_child in child.findall("covariance"):
                        if "cov_subset" in info_root.attrib:
                            if grand_child.attrib.get("cov_subset", "default") in [info_root.attrib["cov_subset"], "default"]:
                                if child.attrib["id"] not in self.cov_config[info_root.attrib["cov_subset"]]["cov_regions"]:
                                    self.cov_config[info_root.attrib["cov_subset"]]["cov_regions"].append(child.attrib["id"])
                        else:
                            if grand_child.attrib.get("cov_subset", False):
                                subsetID = grand_child.attrib["cov_subset"]
                                if subsetID not in list(self.cov_config.keys()):
                                    self.cov_config[subsetID] = dict(cov_regions = [],
                                                                     covariance  = [] )
                                if child.attrib["id"] not in self.cov_config[subsetID]["cov_regions"]:
                                    self.cov_config[subsetID]["cov_regions"].append(child.attrib["id"])

        if self.cov_config != {}:
            for cov_subset in self.cov_config.keys():
                length = len(self.cov_config[cov_subset]["cov_regions"])
                self.cov_config[cov_subset]["covariance"] = [
                    [0. for i in range(length)] for j in range(length)
                ]

        ## getting the region information
        for child in info_root:
            if child.tag == "region" and ("type" not in child.attrib or child.attrib["type"] == "signal"):
                nobs, nb, deltanb, syst, stat = [-1]*5
                for rchild in child:
                    # self.logger.debug(rchild.tag)
                    # self.logger.debug(str(lumi)+' '+str(regions)+ ' '+str(regiondata))
                    try:
                        myval=float(rchild.text)
                    except Exception as err:
                        print('Invalid info file (' + analysis+ '): region data ill-defined.')
                        print(str(err))
                        return -1,-1,-1
                    if rchild.tag=="nobs":
                        nobs = myval
                    elif rchild.tag=="nb":
                        nb = myval
                    elif rchild.tag=="deltanb":
                        deltanb = myval
                    elif rchild.tag=="deltanb_syst":
                        syst = myval
                    elif rchild.tag=="deltanb_stat":
                        stat = myval
                    elif rchild.tag=="covariance":
                        if self.cov_config != {}:
                            for cov_subset, item in self.cov_config.items():
                                if child.attrib["id"] not in item["cov_regions"] or \
                                        rchild.attrib["region"] not in item["cov_regions"]:
                                    continue
                                i = item["cov_regions"].index(child.attrib["id"])
                                j = item["cov_regions"].index(rchild.attrib["region"])
                                if self.error_extrapolation=='sqrt':  ### fails if some value negative!!
                                    print(f'Sqrt error, doing scaling {myval}, {lumi_scaling}')
                                    myval = round(math.sqrt(myval)*lumi_scaling,8); 
                                elif self.error_extrapolation=='linear':
                                    myval *= lumi_scaling**2
                                else:
                                    myval = round(myval*lumi_scaling**2*self.error_extrapolation[0]**2 + \
                                                  math.sqrt(myval)*lumi_scaling*self.error_extrapolation[1]**2,8);

                                self.cov_config[cov_subset]["covariance"][i][j] = myval
                    else:
                        print('Invalid info file (' + analysis+ '): unknown region subtag.')
                        return -1,-1,-1
                if syst == -1 and stat == -1:
                    if self.error_extrapolation=='sqrt':
                        err_scale=math.sqrt(lumi_scaling)
                        deltanb = round(deltanb*err_scale,8)
                    elif self.error_extrapolation=='linear':
                        err_scale = lumi_scaling
                        deltanb = round(deltanb*err_scale,8)
                    else:
                        nb_new = nb*lumi_scaling
                        deltanb = round(math.sqrt(self.error_extrapolation[0]**2*nb_new**2 
                                                  + self.error_extrapolation[1]**2*nb_new), 8);
                else:
                    if syst==-1:
                        syst=0.
                    if stat==-1:
                        stat=0.
                    deltanb = round(math.sqrt( (syst/nb)**2 + (stat/(nb*math.sqrt(lumi_scaling)))**2 )*nb*lumi_scaling,8)
                regiondata[child.attrib["id"]] = { "nobs":nobs*lumi_scaling, "nb":nb*lumi_scaling, "deltanb":deltanb}
        #print(self.cov_config)
        tmp = {}
        for cov_subset, item in self.cov_config.items():
            if item["covariance"] != []:
                tmp[cov_subset] = item
        self.cov_config = tmp
        #print(self.cov_config['Combined']['covariance'])

        self.lumi=lumi
        self.regions=regions
        self.regiondata=regiondata

        #return lumi, regions, regiondata

    def runspeycombined(self,efficiency_data,xs,with_significance=True):
        signal_yields=[]
        background_yields=[]
        observed_yields=[]
        for region in self.regions:
            ## lame for now
            eff=float(efficiency_data[region])
            signal_yields.append(eff*self.lumi*xs)
            Nb=float(self.regiondata[region]['nb'])
            background_yields.append(Nb)

            observed_yields.append(float(self.regiondata[region]['nobs']))

        signal_yields=np.array(signal_yields)
        pdf_wrapper = spey.get_backend("default_pdf.correlated_background" )
        statistical_model = pdf_wrapper(signal_yields,np.array(background_yields),np.array(observed_yields),np.array(self.cov_config['Combined']['covariance']))
        results={}
        if with_significance:
            mumax,_=statistical_model.maximize_likelihood()
            sigmamu=statistical_model.sigma_mu(mumax)
            _,_,pvals,_ = statistical_model.significance()
            pval=pvals[0]
            #pval=0
            #print(statistical_model.exclusion_confidence_level())
            #   print(f'mumax: {mumax}, sigmamu: {sigmamu}, xs*mumax: {xs*mumax}')
            print(f'mumax: {mumax}, sigmamu: {sigmamu}, pval: {pval}')
            results={'mumax':mumax, 'sigmamu':sigmamu, 'pval':pval}
        
        uppermuobs=statistical_model.poi_upper_limit()
        uppermuexp=statistical_model.poi_upper_limit(expected=spey.ExpectationType.apriori) 
        upperlim=xs*uppermuobs

        results['uppermuexp']=uppermuexp
        results['uppermuobs']=uppermuobs
        results['s95obs']=xs*uppermuobs
        results['s95exp']=xs*uppermuexp
        print(f'Upper limit: {upperlim}')

        
        return results
    

