#include "include/running.h"




//void run(int nevents, const string & infile, const string & outfile, int numthreads)
void run(SettingsContainer &runSettings)
{
int nevents = runSettings.nevents;
int numthreads =runSettings.ncores;
string infile = runSettings.lhefile;
string outfile = runSettings.outfile;
string histofile = runSettings.histofile;
string cutflowfile = runSettings.cutflowfile;
double Rjet = runSettings.Rjet;

  int effseeds[numthreads];
  int teffseed;

unsigned seed1 = std::chrono::system_clock::now().time_since_epoch().count();
std::mt19937 seeder(seed1);

  std::uniform_int_distribution<int> EfficiencyDistribution(0,RAND_MAX);
 std::srand(EfficiencyDistribution(seeder));



std::vector<GZ::igzstream*> eventreaders;
for(int ithread=0;ithread<numthreads; ithread++)
  {
    teffseed = EfficiencyDistribution(seeder);
	  effseeds[ithread] =teffseed;

    GZ::igzstream* tstream= new GZ::igzstream();
    std:string tinfile=infile;
    if(numthreads > 0)
    {
      
      tinfile=tinfile+std::to_string(ithread);
      tinfile=tinfile+".ha.gz";
    }
   // char toutfile[100]="hackanalaysis_events_";
    //strcat(toutfile,)

    tstream->open(tinfile.c_str());
    eventreaders.push_back(tstream);
  }   


cout << "Finding " << nevents << " across " << numthreads << " threads"<< endl;
 
/*
PDG::user_invisibles.push_back(12345678);


cout << "is my new particle invisible? " <<endl;
if(PDG::isInvisible(12345678))
{
  cout << "Yes! " << endl;

}
else
{
  cout << "no! " << endl;
}

return;

*/
struct winsize w;
ioctl(STDOUT_FILENO, TIOCGWINSZ, &w);
shellWidth=w.ws_col;

//std::srand(500); 









vector<BaseAnalysis*> Master_analyses;

vector<std::string> fillroutines;

for(auto analysis_name : runSettings.Analyses_To_Run)
{
  BaseAnalysis* new_analysis = mkAnalysis(analysis_name);
  new_analysis->_totalevents=0;
  new_analysis->_totalweight=0.0;
  new_analysis->_isMaster=true;
  new_analysis->init();
  Master_analyses.push_back(new_analysis);
}

int iEvent=0;
auto starttime = std::chrono::high_resolution_clock::now();
 
#pragma omp parallel shared(iEvent,Master_analyses,fillroutines) num_threads(numthreads) //spawn the threads
{
  int threadnum = omp_get_thread_num();


double totalweight=0.0;
double survivingweight=0.0;
double evtweight;



std::mt19937 engine(effseeds[threadnum]);
cout << "initialising engine with " << effseeds[threadnum] << endl;

 
progressbar(0.0,"");
std::string messg = "";


vector<BaseAnalysis*> main_analyses;

for(auto analysis_name : runSettings.Analyses_To_Run)
{
  BaseAnalysis* new_analysis = mkAnalysis(analysis_name);
  new_analysis->init();
  new_analysis->_totalevents=0;
  main_analyses.push_back(new_analysis);

}


double last_inc = 0.0;

int npileup=0;
 bool this_thread_end=false;
 while((iEvent < nevents) && (!this_thread_end)) {  
//cout << evt->weights().size() << " ... " << evt->weights()[0] <<  " ... " << evt->weights()[1] <<  " ... " << evt->weights()[2] << endl;
  
  auto nowtime = std::chrono::high_resolution_clock::now();

  std::chrono::duration<double> timediff = std::chrono::duration_cast<std::chrono::duration<double>>(nowtime-starttime);
  double time_inc = timediff.count();
  
  std::stringstream sts;
  sts.precision(6);

  //sts << timediff.count();
  sts << (time_inc-last_inc);
  last_inc=time_inc;

  messg = sts.str();

  if(threadnum == 0)
  {
    progressbar(((float)iEvent)/((float) nevents),messg);
  }

  HEP::Event HUevent;
  if(!ReadHAEvent(HUevent,*eventreaders[threadnum])) 
  {
    this_thread_end=true;
    continue;
  }
   ++iEvent;


  
  



  for(auto main_analysis : main_analyses)
  {
    if(! main_analysis->ProcessEvent(&HUevent))
    {
      cout << "Event analysis failed" << endl;
      continue;
    }
    main_analysis->Execute(engine);

  }
  
  HUevent.clear();
  

};

for(int acount = 0; acount < Master_analyses.size(); acount++)
{
  if(runSettings.xs > 0.0)
  {
    Master_analyses[acount]->set_xsection(runSettings.xs);
  }

  /*
  else
  {
    Master_analyses[acount]->set_xsection(pythias[threadnum]->info.sigmaGen()*1.e12);
  }
  */
  Master_analyses[acount]->add(*main_analyses[acount]);
  main_analyses[acount]->cleanup();
  delete main_analyses[acount];
}



}; // end of mpi pragma


for( auto Master_analysis : Master_analyses)
{
  Master_analysis->Finalise();
}



progressbar(1.0, "");
cout << endl;

auto endtime = std::chrono::high_resolution_clock::now();

 std::chrono::duration<double> total_diff = std::chrono::duration_cast<std::chrono::duration<double>>(endtime - starttime);
cout << "Total run took " << total_diff.count() << std::endl;
  
std::ofstream ResultFile,CutFlowFile,HistoFile;

ResultFile.open(outfile,std::ofstream::out);
CutFlowFile.open(cutflowfile,std::ofstream::out);


for(auto master_analysis : Master_analyses)
{
  master_analysis->write_results(ResultFile);

  master_analysis->print_cutflows(CutFlowFile);

}

ResultFile.close();
CutFlowFile.close();

if(runSettings.writehistos) 
{
  HistoFile.open(histofile,std::ofstream::out);

  for(auto master_analysis : Master_analyses)
  {
    master_analysis->WriteHistos(HistoFile);
    // and delete the histograms
  }
  HistoFile.close();
}

for(auto master_analysis : Master_analyses)
  {
    // delete the histograms and anything else that needs cleaning up.
    master_analysis->cleanup();
    delete master_analysis;
  }

//printcutflow();
cout << "Done " << endl;
//main_analysis.~ThisAnalysis();
//cout << "Returning" << endl;
  // Done.
 // return 0;
}



void help( const char * name )
{
	  cout << "syntax: " << name << " [-h] <yaml input filename>" << endl;
    exit( 0 );
};

int main( int argc, const char * argv[] ) {

  SettingsContainer runSettings;
  
  runSettings.ncores = omp_get_max_threads()/2;

  if(argc < 2)
  {
    help(argv[0]);
  }
  string fname=(string) argv[1];
  runSettings.ReadYAML(fname);

  if(runSettings.Analyses_To_Run.size() ==0)
  {
    // If the user hasn't selected any analyses, add them all ...
    CreateAnalysisList(runSettings.Analyses_To_Run);
  }

  run(runSettings);
  cout << "Finished" << endl;
  return 0;


}
