#pragma once


void SortEvent_DefaultHEPMC(HepMC::GenEvent *event, HEP::Event &OutEvt, int npileups, std::vector<HEP::PileupEvent*> &pileups,double &Rjet, std::mt19937 &engine){

fastjet::JetDefinition jetDef(fastjet::antikt_algorithm, Rjet);
std::vector <HEP::Jet*> outjets;
//std::vector <HEPUtils::Jet> outjets;
std::vector <fastjet::PseudoJet> inclusiveJets, jets;
std::vector <fastjet::PseudoJet> pileupJets, pJInputs; // for calculating rho

std::vector <fastjet::PseudoJet> fjInputs, bquark_momenta; //particles for applying jet clustering
    // Loop over particles in the event: kinematic distribution
fjInputs.resize(0);
pJInputs.resize(0);
bquark_momenta.resize(0);
//cout << "Starting search for jets" << endl;
HepMC::GenEvent::particle_iterator p = event->particles_begin();

//HEP::P4 p4miss(0.0,0.0,0.0,0.0);

while( p != event->particles_end())
  {
   
    int pid=(*p)->pdg_id();
    int apid=abs(pid);
    
    //cout << pid << endl;
    if( isfinal(*p))
    {
      HEP::Particle* newpart = new HEP::Particle((*p)->momentum().px(),(*p)->momentum().py(),(*p)->momentum().pz(),(*p)->momentum().e(),pid);
      // All particles other than invisibles and muons are jet constituents. Bollecks, really we only want photons
      //if( pid == 12 || pid == 14 || pid == 16 || pid == 18 || pid == 13 || pid == 11)
      if ( (*p)->production_vertex() ) 
        {
        
              HepMC::FourVector pos =  (*p)->production_vertex()->position();
                  if (pos.rho() > 0.0001)
                  {
                    // check if it's actually produced inside the detector
                    if((pos.perp() > 12e3) || (abs(pos.z()) > 23e3 )) { p++; delete newpart; continue;}
                    newpart->set_prod(HEP::P4(pos.x(),pos.y(),pos.z(),pos.t()));
                  }
        }

      if(PDG::isHadron(pid))
         {
           //p4miss-=newpart->mom();
           int pcharge=PDG::get3Q(pid);
           //if(PDG::isCharged(pid))
           if(pcharge != 0)
           {
             //OutEvt.add_charged_hadron((*p)->momentum().px(),(*p)->momentum().py(),(*p)->momentum().pz(),(*p)->momentum().e());

             
              newpart->set_3Q(pcharge);
              OutEvt.add_charged_hadron(newpart);
           }
           else
           {
             OutEvt.add_neutral_hadron((*p)->momentum().px(),(*p)->momentum().py(),(*p)->momentum().pz(),(*p)->momentum().e());
             delete newpart;
           }
         }
         else
          {
            newpart->set_prompt(); // sets it to be prompt, otherwise the routine deletes it!!!

            if(apid == 22)
              {
                newpart->set_3Q(0);
               
              }

            if((apid == 11) || (apid == 13))
            { 
              // Some debug code when I was looking at displaced vertex decays 
              // Turned out you can get what looks like a four-electron vertex when an emitted photon
              // produced an electron-positron pair in the same place as the main decay
              /*
               if((newpart->prod_vertex().pT()) > 1530 && (newpart->prod_vertex().pT() < 1532))
               {
                 cout << "Dodgy electron " << newpart->mom() << endl;
                 DebugMyElectron(event,*p);
               }
               */
                if(pid > 0)
                {
                  newpart->set_3Q(-3);

                }
                else
                {
                  newpart->set_3Q(3);
                }
            }
            else
            {
              if(apid > 13) // care about heavy stable charged particles 
              {
                newpart->set_3Q(PDG::get3Q(pid));
              }
            }      

            OutEvt.add_particle(newpart);
          }
       

      if( pid == 12 || pid == 14 || pid == 16 || pid == 18  ) // include electrons and muons in jets
      {
        
        
        p++;
        continue;
      }
      /*
      if(PDG::get3Q(pid) != 0)
        {
          p4miss-=newpart->mom();
        }
        */
       if((pid > 1000000) && (pid < 9000000)) 
      {
        
        p++;
        continue;
      }
      HepMC::FourVector v4((*p)->momentum());
      //cout << "v4x: " << v4.px() << endl;


      fastjet::PseudoJet particleTemp(v4.px(),v4.py(),v4.pz(),v4.e());
      //particleTemp.set_user_index(njetparts);
      //njetparts++;
      fjInputs.push_back(particleTemp);
      
        p++;
      //constituent_pids.push_back(apid);
      //if(njetparts != constituent_pids.size())
      //{
      //  cout << "Error in size of pids in jets!" << endl;
      //}
    }
    else
    {
      //cout << "check here" << endl;
      if(!((*p)->end_vertex())) {  p++; continue;}
      
      if(isDT(*p,event)) 
      {
  //      cout << "DT: " << pid  << std::endl;
        HEP::Particle* newpart = new HEP::Particle((*p)->momentum().px(),(*p)->momentum().py(),(*p)->momentum().pz(),(*p)->momentum().e(),pid);
        HepMC::FourVector pos =  (*p)->production_vertex()->position();           
        newpart->set_prod(HEP::P4(pos.x(),pos.y(),pos.z(),pos.t()));
        HepMC::FourVector dec =  (*p)->end_vertex()->position();
        newpart->set_decay(HEP::P4(dec.x(),dec.y(),dec.z(),dec.t()));
        newpart->set_3Q(PDG::get3Q(pid));
        newpart->set_prompt();
        OutEvt.add_particle(newpart);  
        p++; continue;        
      }

      if(isMetaStableHadron(*p,event))
      {
        HEP::Particle* newpart = new HEP::Particle((*p)->momentum().px(),(*p)->momentum().py(),(*p)->momentum().pz(),(*p)->momentum().e(),pid);
        HepMC::FourVector pos =  (*p)->production_vertex()->position();           
        newpart->set_prod(HEP::P4(pos.x(),pos.y(),pos.z(),pos.t()));
        HepMC::FourVector dec =  (*p)->end_vertex()->position();
       
        newpart->set_decay(HEP::P4(dec.x(),dec.y(),dec.z(),dec.t()));
        int ch = PDG::get3Q(pid);
        newpart->set_3Q(ch);
        newpart->set_prompt();  
        if(ch !=0)
        {
          
          OutEvt.add_charged_hadron(newpart);
          //OutEvt.add_HSCP(newpart);

        }
        else
        {
          delete newpart;
        }
        
      }

      // don't care about b-tagging
      /*
      if(((*p)->end_vertex()) && (apid == 5))// use quarks as proxy, , (ought to use b hadrons, but whatever)
       {
    // ought to check to make sure they are part of the final process and not the incoming one, but whatever, I doubt it matters!
      // 
        //Make sure it's the last b quark
        bool islast = true;
        for ( HepMC::GenVertex::particle_iterator des =(*p)->end_vertex()->particles_begin(HepMC::descendants); des != (*p)->end_vertex()->particles_end(HepMC::descendants);
			  ++des ) {
          if (abs((*des)->pdg_id()) == 5)
          {
            islast=false;
            break;
          }
		    }
        if(islast)
        {
          HepMC::FourVector v4((*p)->momentum());
        
          fastjet::PseudoJet tpjet(v4.px(),v4.py(),v4.pz(),v4.e());

          bquark_momenta.push_back(tpjet);
        }
        
        //bquark_momenta.push_back(fastjet::PseudoJet(v4.px(),v4.py(),v4.pz(),v4.e()));
       }  
       */

       
      p++;

       //cout << "check done" << endl;
    }
  
  };

/*std::cout << "Found " << bquark_momenta.size() << " b quarks" << endl;
for( auto bm : bquark_momenta)
{
  cout << bm.perp() << ", ";

}
cout << endl;
*/

/*
for (int i = 0; i < event.size(); ++i) {
        // Require visible particles inside detector.
if (! isfinal(event[i]) || event[i].isLepton() || abs(event[i].id()) == 1000024 || abs(event[i].id()) == 1000022
      || event[i].id() == 23
      || abs(event[i].id()) == 24){continue;}
        fastjet::PseudoJet particleTemp(event[i].px(),event[i].py(),event[i].pz(),event[i].e());
        particleTemp.set_user_index(i);
        fjInputs.push_back(particleTemp);
    }

*/    
fastjet::ClusterSequence clustSeq(fjInputs, jetDef);
inclusiveJets = clustSeq.inclusive_jets(1.0); // argument is pTjet
    jets    = sorted_by_pt(inclusiveJets);

std::uniform_int_distribution<> rd_el(0,pileups.size()-1);

static std::normal_distribution<double> zdist(0,53.0);
static std::normal_distribution<double> tdist(0,160e-12);

//auto starttime = std::chrono::high_resolution_clock::now();
//
//time_t starttime = time(NULL);

for(int jp=0; jp< npileups; jp++)
{
  int pileup_number=rd_el(engine);
  //cout << "selecting pileup number " << pileup_number << std::endl;
  //for(auto nhad : pileups[pileup_number]->get_neutral_hadrons())
 // {
  //  OutEvt.add_neutral_hadron(nhad)
 // }
  double dz=zdist(engine);
  double dt=tdist(engine);
  OutEvt.add_particles(pileups[pileup_number]->translate_particles(dz,dt));
  OutEvt.add_neutral_hadrons(pileups[pileup_number]->get_neutral_hadrons());
  //OutEvt.add_charged_hadrons(pileups[pileup_number]->get_charged_hadrons());
  OutEvt.add_charged_hadrons(pileups[pileup_number]->translate_charged_hadrons(dz,dt));
 // OutEvt.add_hadron_vertices(pileups[pileup_number]->translate_hadron_vertices(dz,dt));
}

// now to compute Jets including pileup
pJInputs.clear();
for(auto const chad : OutEvt.get_charged_hadrons())
{
  fastjet::PseudoJet particleTemp(chad->mom().px(),chad->mom().py(),chad->mom().pz(),chad->E());
      //particleTemp.set_user_index(njetparts);
      //njetparts++;
      pJInputs.push_back(particleTemp);

}

for(auto const neut : OutEvt.get_neutral_hadrons())
{
  fastjet::PseudoJet particleTemp(neut->px(),neut->py(),neut->pz(),neut->E());
      //particleTemp.set_user_index(njetparts);
      //njetparts++;
      pJInputs.push_back(particleTemp);

}

for(auto const vis : OutEvt.visible_particles())
{
  fastjet::PseudoJet particleTemp(vis->mom().px(),vis->mom().py(),vis->mom().pz(),vis->mom().E());
      //particleTemp.set_user_index(njetparts);
      //njetparts++;
      pJInputs.push_back(particleTemp);

}

//fastjet::ClusterSequence PJSeq(pJInputs, jetDef);
//pileupJets = PJSeq.inclusive_jets(1.0); // argument is pTjet
//GridMedianBackgroundEstimator bge(double max_rapidity,double requested_grid_spacing);
////bge.set_particles(event_particles);
//// the median of (pt/area) for grid cells, or for jets that pass the selection cut,
//// making use also of information on empty area in the event (in the jets case)
//rho = bge.rho();
//// an estimate of the fluctuations in theptdensity per unit√A,
//sigma = bge.sigma();

// 0.6 according to https://arxiv.org/pdf/1607.03663.pdf
// I'll use 0.5 since it's the cone size 
fastjet::GridMedianBackgroundEstimator bge(4.7,0.5);
//bge.set_particles(pileupJets);
bge.set_particles(pJInputs);
OutEvt.Average_rho=bge.rho();

pileupJets.clear();

//auto endtime = std::chrono::high_resolution_clock::now();
//std::chrono::duration<double> timediff = std::chrono::duration_cast<std::chrono::duration<double>>(endtime-starttime);
//double timediff = difftime(endtime, starttime) * 1000.0;

/*
cout.precision(10);
cout << "Filling pileup took " << timediff.count() << std::endl;
cout << "Event now has " << OutEvt.get_charged_hadrons().size() << " charged hadrons, "  << OutEvt.get_neutral_hadrons().size() << " neutral hadrons, "  << OutEvt.visible_particles().size() << " particles " << std::endl;
cout << "Now filling event of n particles: " << event.size() << std::endl ;
*/




// now do b tagging via method of looking if the delta_R of each jet to each initial b_quark is large enough
//int btagged=0;
//std::uniform_real_distribution<double> rd(0.0,1.0);
/*
for (auto n : electron_numbers)
{
  cout << n << ", " ;
}
cout << endl;
*/

//cout << "Found " << bquark_momenta.size() << "bquarks" << endl;
//cout << "Found " << candidate_electrons.size() << " electron candidates" << endl;
//bool btag;




//cout << "smearing jets" << endl;
// now smear the jets
//smearJets(outjets,engine);
 
for(auto jet : jets)
{
  HEP::Jet* tjet = new HEP::Jet(jet.px(),jet.py(),jet.pz(),jet.E());
  outjets.push_back(tjet);

}

OutEvt.set_jets(outjets);
// now return them

//return outjets;


}


void filleventHEPMC(HEP::Event &OutEvt, HepMC::GenEvent* evt, int npileups, std::vector<HEP::PileupEvent*> &pileups, std::mt19937 &engine)
//void filleventHEPMC(HEP::Event &OutEvt, HepMC::GenEvent* evt, SettingsContainer &runsettings, std::vector<HEP::PileupEvent*> &pileups, std::mt19937 &engine)
{
// Function for filling a heputils event from the HepMC one, doing the smearing etc.

// First let's set the weight
// this needs correcting
//OutEvt.set_weights(evt->weights());
if(evt->weights().size() > 0)
    {
      
      OutEvt.set_weight(evt->weights()[0]);
    }
  else
    {
       OutEvt.set_weight(1.0);
    }

//HEPUtils::Event OutEvt;

//int npileups = runsettings.npileup;
//double dR = runsettings.Rjet;
double dR=0.4;

//std::vector<HEP::Jet*> tjets;

//cout << "Getting jets " << endl;

//cout << "sorting event" << endl;
SortEvent_DefaultHEPMC(evt,OutEvt,npileups,pileups,dR,engine);
// fill up the jets
//OutEvt.set_jets(tjets);

// fill up final state particles, compute MET

//cout << "filling leptons" << endl;

//betterCalcMissingMomentum(evt, OutEvt);
anotherCalcMissingMomentum(evt, OutEvt);
//calcMissingMomentum(OutEvt, engine);

//cout << "Before returning we have " << OutEvt.electrons().size() << " electrons and " << OutEvt.muons().size() << " muons" << endl;
//cout << "Before smearing we have " << OutEvt.electrons().size() << " electrons and " << OutEvt.muons().size() << " muons" << endl;


/*
filterEfficiency(OutEvt.jets(),JetEff,engine);


filterEfficiency(OutEvt.electrons(),ElectronEff,engine);
filterEfficiency(OutEvt.muons(),MuonEff,engine);

smearElectronEnergy(OutEvt.electrons(),engine);
smearMuonMomentum(OutEvt.muons(),engine);
*/


//cout << "After smearing we have " << OutEvt.electrons().size() << " electrons and " << OutEvt.muons().size() << " muons" << endl;
//return OutEvt;




}





void filleventHEPMC_FatJet(HEP::Event &OutEvt, HepMC::GenEvent* evt, int npileups, std::vector<HEP::PileupEvent*> &pileups, std::mt19937 &engine)
//void filleventHEPMC(HEP::Event &OutEvt, HepMC::GenEvent* evt, SettingsContainer &runsettings, std::vector<HEP::PileupEvent*> &pileups, std::mt19937 &engine)
{

if(evt->weights().size() > 0)
    {
      
      OutEvt.set_weight(evt->weights()[0]);
    }
  else
    {
       OutEvt.set_weight(1.0);
    }

double dR=1.0;
SortEvent_DefaultHEPMC(evt,OutEvt,npileups,pileups,dR,engine);
anotherCalcMissingMomentum(evt, OutEvt);


}