#pragma once
bool isfinal(const HepMC::GenParticle* p )
{
//if ( (!(p->end_vertex())) && (p->status()==1) ) return true;
//if(p->status() ==3) return false;

if ( (!(p->end_vertex()))) 
{
  if(p->status() != 1) 
  {
    cout << "Particle with no end vertex and status" << p->status() << endl;
    return false;
  }
  return true;
}

return false;

};


/*

HepMC: perp() gives pT, rho() gives 3-vector length



*/


bool isDT(const HepMC::GenParticle* p, const HepMC::GenEvent *event)
//Return True (False) if the particle is (is not) a HSCP
{
int pid = p->pdg_id();
 int pch = PDG::get3Q(pid);

	//Skip neutral particles
	if (pch == 0) return false;
	
  
  // Care if it's a hadron! The metastable charged hadrons will be included elsewhere ...
  if(PDG::isHadron(pid)) return false;
	
  
  //Skip light particles
	//if (fabs(particle->m()) < 20.) return false;
  if(p->momentum().m() < 20.) return false;

  

	//Check if it is not an intermediate state (has itself as daughter)
  for ( HepMC::GenVertex::particle_iterator des = p->end_vertex()->particles_begin(HepMC::descendants); des != p->end_vertex()->particles_end(HepMC::descendants);
			  ++des ) {
          if ((*des)->pdg_id() == pid)
          {
            return false;
 
          }
		    }

  
  // check produced and decays inside
  if(!p->end_vertex()) return false;
  if(!p->production_vertex()) return false;

  HepMC::FourVector pos =  p->production_vertex()->position();
  HepMC::FourVector dec = p->end_vertex()->position();

  double pperp = pos.perp();
  double dperp = dec.perp();
  double pz = pos.pz();
  double dz = dec.pz();
  
  // check if track is long enough
  if(((dperp - pperp) < 3.0 ) && (fabs(dz-pz) < 3.0)) return false;
  
  // check whether it shows up as a track
  //if((pperp > 1100.0) || (dperp < 160.0) || (fabs(pz) > 2700.0) || (fabs(dz) > 2700.0)) return false;
  if((pperp > 1100.0) || (dperp < 160.0) || (fabs(pz) > 2700.0)) return false;
  //cout << "so far so good " << pid <<  " " << pz << ", " << dz << "," << dperp << std::endl;
	//std::vector<int> daughters = particle->daughterListRecursive();
	//for (int i = 0; i < daughters.size(); ++i){
	//	int idaughter = daughters[i];
	//	Pythia8::Particle daughterParticle = event[idaughter];
	//	if (daughterParticle.id() == particle->id()) return false;
	//}

	return true;
}

bool isMetaStableHadron(const HepMC::GenParticle* p, const HepMC::GenEvent *event)
//Return True (False) if the particle is (is not) a HSCP
{
int pid = p->pdg_id();
// int pch = PDG::get3Q(pid);
 //Skip neutral particles
//	if (pch == 0) return false;
 // skip hadrons
if(!(PDG::isHadron(pid))) return false;
	
	

  
	//Skip light particles
	//if (fabs(particle->m()) < 20.) return false;
  if(p->momentum().m() < 20.) return false;
  
  // We're interested in the tracker
  //if( (p->production_vertex()->position().perp() > 1100.) || (fabs(p->production_vertex()->position().pz()) > 3100.)) {return false;}
  // Let's say must be produced in the pixel detector, decays somewhere in the tracker or outside. That way, we include it in
  // the isolation calculation only once

  double prodT=p->production_vertex()->position().perp();
  double prodZ=p->production_vertex()->position().pz();

  if( (prodT > 200.) || (fabs(prodZ) > 270.)) {return false;}

  


	//Check if it is not an intermediate state (has itself as daughter)
  for ( HepMC::GenVertex::particle_iterator des = p->end_vertex()->particles_begin(HepMC::descendants); des != p->end_vertex()->particles_end(HepMC::descendants);
			  ++des ) {
          if ((*des)->pdg_id() == pid)
          {
            return false;
 
          }
		    }
  double decT=p->end_vertex()->position().perp();
  double decZ=p->end_vertex()->position().pz();
  
  //if( (p->end_vertex()->position().perp() < 33.0)){return false;}
  if( (decT < 300.0) ){return false;}

  

	//std::vector<int> daughters = particle->daughterListRecursive();
	//for (int i = 0; i < daughters.size(); ++i){
	//	int idaughter = daughters[i];
	//	Pythia8::Particle daughterParticle = event[idaughter];
	//	if (daughterParticle.id() == particle->id()) return false;
	//}

	return true;
}





bool isHSCP(const HepMC::GenParticle* p, const HepMC::GenEvent *event)
//Return True (False) if the particle is (is not) a HSCP
{
int pid = p->pdg_id();
 int pch = PDG::get3Q(pid);
 //Skip neutral particles
	if (pch == 0) return false;
 // skip hadrons
if(PDG::isHadron(pid)) return false;
	
	

  
	//Skip light particles
	//if (fabs(particle->m()) < 20.) return false;
  if(p->momentum().m() < 20.) return false;

  //if( (prodVertex.pT() > 1100.) || (abs(prodVertex.pz()) > 3100.)) {return false;}
 
	//Check if it is not an intermediate state (has itself as daughter)
  for ( HepMC::GenVertex::particle_iterator des = p->end_vertex()->particles_begin(HepMC::descendants); des != p->end_vertex()->particles_end(HepMC::descendants);
			  ++des ) {
          if ((*des)->pdg_id() == pid)
          {
            return false;
 
          }
		    }

	//std::vector<int> daughters = particle->daughterListRecursive();
	//for (int i = 0; i < daughters.size(); ++i){
	//	int idaughter = daughters[i];
	//	Pythia8::Particle daughterParticle = event[idaughter];
	//	if (daughterParticle.id() == particle->id()) return false;
	//}

	return true;
}

/*
inline bool fromHadron(int n, const Pythia8::Event& evt) {
      // Root particle is invalid
      if (n == 0) return false;
      const Pythia8::Particle& p = evt[n];
      if (p.isHadron()) return true;
      if (p.isParton()) return false; // stop the walking at the end of the hadron level
      for (int m : p.motherList()) {
        if (fromHadron(m, evt)) return true;
      }
      return false;
    }

bool isfinal(const HepMC::GenParticle* p )
{
if ( !p->end_vertex() && p->status()==1 ) return true;
return false;

};
*/

/*
double d0Calc(const HepMC::FourVector xv, const HepMC::FourVector p)
//Compute the transverse impact parameter for a particle with momentum p created at vertex xv
//assuming no magnetic field
// Look at x-y plane and find clsost distance: this is |xv| sin varphi, where the angle is between the vertex
// and the momentum
// this is just given by xv cross phat, where each is represented as a 2d vector
{
    double calpha = p.px()/p.perp();
    double salpha = p.py()/p.perp();

    double a0 = xv.px()*salpha - xv.py()*calpha;

    return fabs(a0);
}

double dzCalc(const HepMC::FourVector xv, const HepMC::FourVector p)
//Compute the longitudinal impact parameter for a particle with momentum p created at vertex xv
//assuming no magnetic field
// dz is the z value at the point of closest d0.
// v_min = xv - lambda \hat{p}
// lambda = |xv| cos varphi = xv dot \hat{p} (in 2d)
{
    //double calpha = p.px()/p.pT();
    //double salpha = p.py()/p.pT();

    //double a0 = xv.px()*salpha - xv.py()*calpha;
    double lambda= (p.px()*xv.px()+p.py()*xv.py())/p.perp();
    double zmin = xv.pz()-lambda*p.pz()/p.perp();
    return fabs(zmin);
}

double dz_sintheta(const HepMC::FourVector xv, const HepMC::FourVector p)
//Compute the longitudinal impact parameter for a particle with momentum p created at vertex xv
{
    //double calpha = p.px()/p.pT();
    //double salpha = p.py()/p.pT();

    //double a0 = xv.px()*salpha - xv.py()*calpha;
    if(p.rho() == 0.0) return 0.0;
    double dz = dzCalc(xv,p);

    return fabs(dz*p.perp()/p.rho());
}
*/

void DebugMyElectron(HepMC::GenEvent *event,const HepMC::GenParticle* p )
{

const HepMC::GenParticle* q = new HepMC::GenParticle;
q=p;
cout << "Diagnosing" << endl;
int pid = p->pdg_id();
bool foundGenuineMother=false;
while((!foundGenuineMother) && (q->production_vertex()))
{
  for ( HepMC::GenVertex::particle_iterator mother= q->production_vertex()->particles_begin(HepMC::parents);
			  mother != q->production_vertex()->particles_end(HepMC::parents); ++mother)
  {
    cout << "mother id: " << (*mother)->pdg_id() << ", ";
    if((*mother)->pdg_id() == pid)
    {
      q=*mother;
      break;
    }
    else
    {
      foundGenuineMother=true;
      break;
    }
  }
}
cout << endl;
}





bool SAFEisFromHadron(HepMC::GenEvent *event,const HepMC::GenParticle* p )
{
  // follow the chain up until we get to a vertex where there are no mothers equal to p
  //if ( !(*p)->production_vertex() ) return false; // weird case here, the particle has no production vertex???
  
  bool foundGenuineMother=false;
  int pid = p->pdg_id();
  const HepMC::GenParticle* q = new HepMC::GenParticle;

  int mpid;
  bool ismotherhadron;
  q = p;
  while((!foundGenuineMother) && (q->production_vertex()))
  { 
  foundGenuineMother= true;
  for ( HepMC::GenVertex::particle_iterator mother= q->production_vertex()->particles_begin(HepMC::parents);
			  mother != q->production_vertex()->particles_end(HepMC::parents); 
			  ++mother )
        {
          if((*mother)->pdg_id() == pid)
          {
            q = *mother; // the particle iterator points to the pointer to the particle
            foundGenuineMother=false;
            break;
          }
          else
          {
            mpid=(*mother)->pdg_id();
            ismotherhadron=PDG::isHadron(mpid);
            if(ismotherhadron)
            {
              foundGenuineMother=true;
              q = *mother;
              break;
            }
          }
          
        }
  };

  // here we've found the genuine mother
  if(ismotherhadron)
  {
   // cout << " Hadronic mother " << mpid << endl;
    return true;
  }

  /*
  else
  {
    cout << " non-hadronic mother(s): " ;
    for ( HepMC::GenVertex::particle_iterator mother= q->production_vertex()->particles_begin(HepMC::parents);
			  mother != q->production_vertex()->particles_end(HepMC::parents); 
			  ++mother )
        {
          cout << (*mother)->pdg_id() << ", ";
        }
    
    cout << endl;
  }
  */

  return false; //ismotherhadron;

} 

bool isFromHadron(HepMC::GenEvent *event,const HepMC::GenParticle* p )
{
  // follow the chain up until we get to a vertex where there are no mothers equal to p
  //if ( !(*p)->production_vertex() ) return false; // weird case here, the particle has no production vertex???
  
  bool foundGenuineMother=false;
  int pid = p->pdg_id();
  int apid = abs(pid);

  if (apid < 10) return false;  // quarks
  if (apid == 21) return false; // gluon 
  if (PDG::isHadron(pid)) return true;

  // really we're only interested in electrons or photons that come from hadrons
  // So ignore BSM particles
  // This basically means anything with pdg code > 24 after we have already checked if it's a hadron

  if(apid > 24) return false; 

  if(!(p->production_vertex())) return false;

  for ( HepMC::GenVertex::particle_iterator mother= p->production_vertex()->particles_begin(HepMC::parents);
			  mother != p->production_vertex()->particles_end(HepMC::parents); 
			  ++mother )
        {
          if (isFromHadron(event, *mother)) return true;
          
        };


  return false; //ismotherhadron;

} 


void calcMissingMomentum(HEP::Event &OutEvt, std::mt19937 &engine)
{
  // look at the invisibles. Bit stupid I know but whatever. 
  HEP::P4 pmiss;
  pmiss.clear();

  /*
  if (abs(ptc.vProd().pz()) > 6100. || ptc.vProd().pT() > 2300.)

  */
    
  for(HEP::Particle* invisible : OutEvt.invisible_particles())
  {
   // std::cout << fabs(invisible->prod_vertex().pz()) << ", " << invisible->prod_vertex().pT() << std::endl;
    //if((fabs(invisible->prod_vertex().pz()) > 6100.) || (invisible->prod_vertex().pT() > 2300.)) continue;
    //if((fabs(invisible->prod_vertex().pz()) > 3150.) || (invisible->prod_vertex().pT() > 1200.)) continue;
    pmiss+=invisible->mom();
  }
  
  /*
  for(HEP::Particle* invisible : OutEvt.HSCPs())
  {
    //if((fabs(invisible->prod_vertex().pz()) > 6100.) || (invisible->prod_vertex().pT() > 2300.)) continue;
   // if((fabs(invisible->decay_vertex().pz()) < 6100.) || (invisible->decay_vertex().pT() < 2300.)) continue;
    pmiss+=invisible->mom();
  }
  */
  OutEvt.set_missingmom(pmiss);

  // smear here ...
}



void anotherCalcMissingMomentum(HepMC::GenEvent *event,HEP::Event &OutEvt)
{

  HEP::P4 pmiss;
  pmiss.clear();

  HepMC::GenEvent::particle_iterator p = event->particles_begin();
  //HepMC::FourVector HMCpmiss;
//HEP::P4 p4miss(0.0,0.0,0.0,0.0);

while( p != event->particles_end())
  {
   
    int pid=(*p)->pdg_id();
    int apid=abs(pid);
    //HEP::P4 vprod(0.0,0.0,0.0,0.0);
    double prodZ=0.0;
    double prodT=0.0;
    if ( (*p)->production_vertex() ) 
        {
          HepMC::FourVector pos =  (*p)->production_vertex()->position();
          if (pos.rho() > 0.0001)
                  {
                    prodT=pos.perp();
                    prodZ=fabs(pos.z());
                   
                    // If produced outside the detector then it is no use to us
                     if((prodT > 7e3) || (prodZ > 11e3 )) { p++;  continue;}
                    
                    

                  }

        }

    if( isfinal(*p))
    {
      // Need it to be produced before end of HCAL, unless it is produced
      // with something like a muon track (ignore that case)
      //if((prodT > 2.95e3) || (prodZ > 5.5e3 )) { p++;  continue;}
      if(PDG::isInvisible(pid))
      {
        pmiss+=HEP::P4((*p)->momentum().px(),(*p)->momentum().py(),(*p)->momentum().pz(),(*p)->momentum().e());

      }




    }
    else
    {
      // want the last *visible* particle that decays to an invisible outside the HCAL (but still inside the detector)
      // NO! Actually we want the last *invisible* metastable particle that decays outside
      if(!PDG::isInvisible(pid)) {p++; continue;} 
      // see if it decays to an invisible particle beyond the calo
       
       double decT = (*p)->end_vertex()->position().perp();
       double decz = fabs((*p)->end_vertex()->position().pz());
        // Decay must be outside the detector, we already know it's produced inside

        // If it decays outside the muon system then it doesn't contribute to pmiss!!
       //if((decT > 7e3 ) || (decz > 11e3)) { p++; continue;}

        // If it decays in the HCAL then it doesn't contribute, but we've already checked it's in the detector
        //if((decT < 2.95e3 ) && (decz < 5.5e3)) { p++; continue;}
	// change to the muon system; if it decays inside then it is no good, it has to match above
	// Here we are only interested in invisibles produced in the detector that escape
	if((decT < 7.0e3 ) && (decz < 11.0e3)) { p++; continue;}



       // 
       bool islast = true;
       //bool invisibleproduct = false;
	
      for ( HepMC::GenVertex::particle_iterator des =(*p)->end_vertex()->particles_begin(HepMC::descendants); des != (*p)->end_vertex()->particles_end(HepMC::descendants);
			  ++des ) {
          if ((*des)->pdg_id() == pid)
          {
            islast=false;
            break;
          }
	 /*	
          else
          {
            if(PDG::isInvisible((*des)->pdg_id())) 
            {
              invisibleproduct=true;
              
            }
          }
	  */
        }

        if(!islast) {p++; continue;}
        //if(invisibleproduct)
        //{
          pmiss+=HEP::P4((*p)->momentum().px(),(*p)->momentum().py(),(*p)->momentum().pz(),(*p)->momentum().e());
        //}

    }

    p++;



  }
  OutEvt.set_missingmom(pmiss);
}


void betterCalcMissingMomentum(HepMC::GenEvent *event,HEP::Event &OutEvt)
{
  HEP::P4 pmiss;
  pmiss.clear();

  HepMC::GenEvent::particle_iterator p = event->particles_begin();
  //HepMC::FourVector HMCpmiss;
//HEP::P4 p4miss(0.0,0.0,0.0,0.0);

while( p != event->particles_end())
  {
   
    int pid=(*p)->pdg_id();
    int apid=abs(pid);
    //HEP::P4 vprod(0.0,0.0,0.0,0.0);
    double prodZ=0.0;
    double prodT=0.0;
    if ( (*p)->production_vertex() ) 
        {
          HepMC::FourVector pos =  (*p)->production_vertex()->position();
          if (pos.rho() > 0.0001)
                  {
                    prodT=pos.perp();
                    prodZ=fabs(pos.z());
                    if((prodT > 12e3) || (prodZ > 23e3 )) { p++;  continue;}

                    

                  }

        }


    
    //cout << pid << endl;
    if( isfinal(*p))
    {

      if(PDG::isHadron(pid))
         {
           // make sure produced before end of HCAL
           if((prodT > 3000.0)|| (prodZ > 5000.0)) {p++; continue;}
          pmiss-=HEP::P4((*p)->momentum().px(),(*p)->momentum().py(),(*p)->momentum().pz(),(*p)->momentum().e());
         }
         else
         {
            if( (apid == 22) || (PDG::get3Q(pid) !=0 ) )
            {
              if((prodT > 1700.0)|| (prodZ > 4000.0)) {p++; continue;}
              pmiss-=HEP::P4((*p)->momentum().px(),(*p)->momentum().py(),(*p)->momentum().pz(),(*p)->momentum().e());
            }
         }

    }
    else
    {
      // check last of its kind
      if((!(*p)->end_vertex()) || (!(*p)->production_vertex())) {p++; continue;}

      bool islast = true;
      for ( HepMC::GenVertex::particle_iterator des =(*p)->end_vertex()->particles_begin(HepMC::descendants); des != (*p)->end_vertex()->particles_end(HepMC::descendants);
			  ++des ) {
          if ((*des)->pdg_id() == pid)
          {
            islast=false;
            break;
          }
        }

        if(!islast) { p++; continue;}

      double decZ= fabs((*p)->end_vertex()->position().pz());
      double decT=(*p)->end_vertex()->position().perp();

      if(PDG::isHadron(pid))
      {
        if((prodT > 3000.0)|| (prodZ > 5000.0) || (decZ < 4e3 ) ||(decT < 1.77e3)) {p++; continue;}
        pmiss-=HEP::P4((*p)->momentum().px(),(*p)->momentum().py(),(*p)->momentum().pz(),(*p)->momentum().e());
      }
      else
      {
        //if( (apid == 22) || (apid == 11) )
        if(PDG::isCharged(pid))
            {
              // treat as a muon
              //if((prodT > 1700.0)|| (prodZ > 4000.0) || (decZ < 3.15e3 ) ||(decT < 1.2e3)) {p++; continue;}
              if( (prodT > 1.77e3) || (prodZ > 4000.0) || (decZ < 1.2e4) || (decT < 7.0e3)){ p++; continue;}
              pmiss-=HEP::P4((*p)->momentum().px(),(*p)->momentum().py(),(*p)->momentum().pz(),(*p)->momentum().e());
            }
      }

    }

    
  p++;
  }
  
  //HEP::P4 pmiss(HMCpmiss.px(),HMCpmiss.py(),HMCpmiss.pz(),HMCpmiss.E());
  OutEvt.set_missingmom(pmiss);


}



