#!/usr/bin/env python3
import os
import sys

import regenerate

if len(sys.argv) < 2:
    print("Need to give an analysis name!")
    raise SystemExit

ANAME=sys.argv[1]

if len(sys.argv) > 2 and sys.argv[2] =='--ONNX':
    ONNXAnalysis=True
else:
    ONNXAnalysis=False

ADIR='UserAnalyses/'

HNAME=ADIR+ANAME+'.h'
CCNAME=ADIR+ANAME+'.cc'

HHNAME=ANAME+'_h'

if not os.path.exists(ADIR[:-1]):
    os.mkdir(ADIR[:-1])

if os.path.exists(HNAME) or os.path.exists(CCNAME):
    print('Analysis already exists! If you want to create a new blank version, just delete the .h and .cc files')
    raise SystemExit

############### Write HEADER!

try:
    OF=open(HNAME,'w')
except:
    print('Could not create header file')
    raise SystemExit

#OF.write('#pragma once\n\n')

if ONNXAnalysis:
    OF.write('#ifdef ONNXPATH\n')
OF.write('#ifndef '+HHNAME+'\n')
OF.write('#define '+HHNAME+'\n')

OF.write('#include "include/BaseAnalysis.h"\n')
OF.write('#include "include/HEPData.h"\n\n')

OF.write('using namespace std;\n\n')

OF.write('class '+ANAME+' : public BaseAnalysis {\n')
OF.write('    public:\n\n')
OF.write('       ~'+ANAME+'();\n')
OF.write('       void init();\n\n')
OF.write('       void Execute(std::mt19937 &engine);\n\n')
OF.write('       void Finalise() { return; }\n\n')
#OF.write('       '+ANAME+'() { this->setup(); this->analysisname="'+ANAME+'"; this->init(); };\n\n')
OF.write('       '+ANAME+'() { this->setup(); this->analysisname="'+ANAME+'"; };\n\n')
OF.write('};\n\n')

OF.write('#endif\n')
if ONNXAnalysis:
    OF.write('#endif\n')

OF.close()




############### Write CPP file!

try:
    OF=open(CCNAME,'w')
except:
    print('Could not create source file')
    raise SystemExit


if ONNXAnalysis:
    OF.write('#ifdef ONNXPATH\n')

#OF.write('#pragma once\n\n')



OF.write('#include "'+ANAME+'.h"\n\n')

OF.write('namespace HEP\n{\n')
OF.write('    namespace '+ANAME+' \n')
OF.write('    {\n/* Place here your user-defined functions */\n\n')
OF.write('    }\n')
OF.write('}\n\n')

OF.write('using namespace HEP::'+ANAME+';\n\n')


OF.write('void '+ANAME+'::init() {\n\n')
OF.write('cout << "---------------------------------------------------" << endl;\n')
OF.write('cout << "-- Analysis '+ANAME+' --" << endl;\n')
OF.write('cout << "---------------------------------------------------" << endl;\n')
OF.write('}\n\n')

OF.write(ANAME+'::~'+ANAME+'() {\n\n')
OF.write('};\n\n')

OF.write('void '+ANAME+'::Execute(std::mt19937 &engine) {\n\n')
OF.write('}\n\n')

if ONNXAnalysis:
    OF.write('#endif\n')
    
OF.close()


###### Now to regenerate the analysis list

regenerate.regenerate_analysislist()






