def regenerate_analysislist(library=False):
    import os
    import sys
    import fnmatch
    try:
        OF=open('analysislist.h','w')
    except:
        print('Could not recreate the analysis list!')
        raise SystemExit


    ANALYSES=[]
    ONNX_ANALYSES=[]
    AHEADS=[]

    if library:
        alist=['Analyses']
    else:
        alist=['Analyses','UserAnalyses']

    for adir in alist:
        if not os.path.exists(adir):
            continue
        AHEADS=[]
        for f in os.listdir(adir):
            if not fnmatch.fnmatch(f,'*.h'):
                continue

            try:
                ANAME=os.path.splitext(f)[0]
            except:
                continue
            firstline=""
            with open(os.path.join(adir,f),'r') as HF:
                firstline=HF.readline()
                
            if firstline.strip().startswith('#ifdef ONNXPATH'):
                ONNX_ANALYSES.append(ANAME)
            else:
                ANALYSES.append(ANAME)
            AHEADS.append(f)

        for f in AHEADS:
            OF.write('#include "'+adir+'/'+f+'"\n')

    OF.write('''\n\n// Here we use macros to set up the list of all analyses
// and functions to check whether to run them. Sadly we can't
// also generate the include lines above. 
// However, deleting an entry from here will completely disable the analysis;
// we also have settings at runtime to do that on a per-run basis.\n\n''')

    OF.write('#define MAP_ONNX_ANALYSES(F) ')
    for f in ONNX_ANALYSES:
        OF.write('\\'+'\nF('+f+')')
    OF.write('\n\n')
    
    OF.write('#define MAP_ANALYSES(F) ')
    for f in ANALYSES:
        OF.write('\\'+'\nF('+f+')')

    OF.write('\n')
    OF.close()

if __name__ == "__main__":
    import sys
    if len(sys.argv) > 1 and sys.argv[1] == '--library':
        regenerate_analysislist(library=True)
    else:
        regenerate_analysislist()
