#include "include/ReadWriteEvent.h"


void vecout(HEP::P4 &vec, GZ::ogzstream &ss)
{
  ss << vec.px() << " " << vec.py() << " " <<  vec.pz() << " " << vec.E() << " ";
}


void vecout(const HEP::P4 &vec, GZ::ogzstream &ss)
{
  ss << vec.px() << " " << vec.py() << " " <<  vec.pz() << " " << vec.E() << " ";
}




void vecout(HEP::P4 *vec, GZ::ogzstream &ss)
{
  ss << vec->px() << " " << vec->py() << " " <<  vec->pz() << " " << vec->E() << " ";
}

/*
void write_stable(const std::vector<Particle*>& particles, ifstream &ss,int &particle_counter)
{
    for(auto particle: particles)
    {
        particle_counter++;
        ss << particle_counter << " " << particle->pid() << " ";
        ss << particle->get_3Q() << " ";

        vecout(particle->mom(),ss);
        vecout(particle->prod_vertex(),ss);

        
        if(particle->does_decay())
        {
            ss << 1 << " ";
            vecout(particle->decay_vertex(),ss);
        }
        else
        {
            ss << 0 << " ";
        }
        
    }


}
*/

void write_particles(const std::vector<Particle*>& particles, GZ::ogzstream &ss,int &particle_counter)
{
    for(auto particle: particles)
    {
        particle_counter++;
        ss << particle_counter << " " << particle->pid() << " ";
        ss << particle->get_3Q() << " ";

        vecout(particle->mom(),ss);
        vecout(particle->prod_vertex(),ss);

        
        if(particle->does_decay())
        {
            ss << 1 << " ";
            vecout(particle->decay_vertex(),ss);
        }
        else
        {
            ss << 0 << " ";
        }
        
        ss << std::endl;
    }


}



/*

need to store info for:
  std::vector<Particle*> _photons, _electrons, _muons, _taus, _invisibles, _HSCPs, _chargedhadrons;
    //@}

    /// Jets collection (mutable to allow sorting)
    mutable std::vector<Jet*> _jets;

    /// Missing momentum vector
    P4 _pmiss;

    std::vector<P4*> _neutralhadrons;


NB we allow only stable particles for photons, electrons, muons
They need vprod, PID, and number in event.
charged hadrons are the same but stored in a separate container

HSCP and invisibles need vprod and may decay: add a flag for "does decay"

Jets need a b tag (for now) and momentum

we want PIDs for 



*/



void WriteEvent(HEP::Event &outevent,GZ::ogzstream &outstream, bool writehadrons)
{
outstream << "<Event>" << std::endl;
// start with weights

outstream << "<CentralWeight>" <<std::endl;
outstream << outevent.weight() << std::endl;
outstream << "</CentralWeight>" <<std::endl;

outstream << "<weights>" << endl;
for (auto weight: outevent.weights())
{
    outstream << weight << " ";

}

int particlecount=0;

outstream << endl << "</weights>" << endl;
outstream << "<ptmiss>" << endl;
vecout(outevent.missingmom(),outstream);
outstream << endl << "</ptmiss>" << endl;
outstream << "<stable particles>" << endl;
write_particles(outevent.electrons(),outstream,particlecount);
write_particles(outevent.muons(),outstream,particlecount);
write_particles(outevent.photons(),outstream,particlecount);
outstream << "</stable particles>" << endl;

outstream << "<invisible particles>" << endl;
write_particles(outevent.invisible_particles(),outstream,particlecount);
outstream << "</invisible particles>" << endl;
outstream << "<HSCPs>" << endl;
write_particles(outevent.HSCPs(),outstream,particlecount);
outstream << "</HSCPs>" << endl;

// now for Jets

outstream << "<Jets>" << endl;
for(auto jet : outevent.jets())
{
    //vecout(nh,outstream);

    vecout(jet->mom(),outstream);
    
    if(jet->btag())
        {
            outstream << 1 << " ";
            
        }
        else
        {
            outstream << 0 << " ";
        }
    outstream << endl;
}
outstream << "</Jets>" << endl;


// now for pileup
if(writehadrons)
{
    outstream << "<charged hadrons>" << endl;
    write_particles(outevent.get_charged_hadrons(),outstream,particlecount);
    outstream << "</charged hadrons>" << endl;

    outstream << "<neutral hadrons>" << endl;

    for(auto nh : outevent.get_neutral_hadrons())
    {
        vecout(nh,outstream);
        outstream << endl;
    }
    outstream << "</neutral hadrons>" << endl;
}


outstream << "</Event>" << endl;

}

/*
void vecin(HEP::P4 &vec, std::istringstream ss)
{
double px,py,pz,e;
  ss >> px >> py >> pz >>e;
}
*/
HEP::P4 vecin(std::istringstream &ss)
{
    double x,y,z,t;
    if(! (ss >> x >> y >> z >> t))
    {
        return HEP::P4(0.0,0.0,0.0,0.0);
    }
    else
    {
        return HEP::P4(x,y,z,t);
    }


}
HEP::Particle* read_particle(std::istringstream &ss)
{
    int particle_counter,pid,Q3,does_decay;
    double px,py,pz,E;


        ss >> particle_counter >> pid >> Q3;
            ss >> px >> py >> pz >> E;

        //vecout(particle->mom(),ss);
        //vecout(particle->prod_vertex(),ss);
        HEP::Particle* particle = new HEP::Particle(px,py,pz,E,pid);
        particle->set_3Q(Q3);

        HEP::P4 prod_vertex = vecin(ss);
        particle->set_prod(prod_vertex);

        ss >> does_decay;
        
        if(does_decay > 0)
        {
            HEP::P4 decay_vertex = vecin(ss);
            particle->set_decay(decay_vertex);
           
        }
        particle->set_prompt();
        return particle;


}





bool ReadHAEvent(HEP::Event &inevent,GZ::igzstream &instream)
{
// reads the next event and pushes it into the new event inevent. 

std::string line;
bool foundend=false;
bool foundstart=false;
bool readinggroup=false;
int tag_index=-1;
std::vector<std::string> tags={"<weights>","<ptmiss>","<stable particles>","<invisible particles>","<HSCPs>","<Jets>","<charged hadrons>","<neutral hadrons>","<CentralWeight>"};
    while (std::getline(instream, line) && (!foundend)) {
        line=rtrim(line);
        if(!foundstart)
        {
            if(! line.compare(0,7,"<Event>")) 
            {
                foundstart=true;
                continue;
            }
        }
    
        if(! line.compare(0,8,"</Event>")) 
            {
                foundend=true;
                break;
            }

        if(!readinggroup) // look for which tag
        {   
            auto it=std::find(tags.begin(), tags.end(), line);
            if(it!= tags.end())
            {
                readinggroup=true;
                tag_index=it-tags.begin();
            }
            continue;
        }

        // must be reading group to get here: check for end!
        if(! line.compare(0,2,"</")) 
            {
                readinggroup=false;
                continue;
            }

        // Now actually read!! assume there is some data here

        std::istringstream data(line);
        if(tag_index == 0) // weights
        {
            std::vector<double> weights;
            double current_weight;
            //while(data.get(current_weight))
            //cout << "Reading weights: ";
            while(data >> current_weight)
            {
                weights.push_back(current_weight);
               // cout << current_weight << " ";
            }
            //cout << endl;
            inevent.set_weights(weights);

        }
        else if(tag_index==1) // ptmiss
        {
            HEP::P4 ptmiss=vecin(data);
            inevent.set_missingmom(ptmiss);
            //cout << "Event ptmiss: " << ptmiss << endl;
        }
        else if(tag_index<5)
        {
            inevent.add_particle(read_particle(data));
            //cout << "added particle " <<endl;
        }
        else if(tag_index==5)
        {
            HEP::P4 jetmom=vecin(data);
            int btag;
            data >> btag;
            bool isbtag = (btag>0);
            HEP::Jet* newjet = new HEP::Jet(jetmom,isbtag);
            inevent.add_jet(newjet);
           // cout << "added jet " << newjet->mom() << endl;
        }
        else if(tag_index==6)
        {
            inevent.add_charged_hadron(read_particle(data));
        }
        else if(tag_index==7)
        {
           
            inevent.add_neutral_hadron(vecin(data));
        }
        else if(tag_index==8)
        {
            double central_weight;
             while(data >> central_weight)
            {
                inevent.set_weight(central_weight);
            }
        }
        


        }
if(!foundstart) return false; //means found no events ...
if(!foundend) return false; // something went wrong
return true; // otherwise have a valid event

}
