#ifdef ONNXPATH
#include "include/handle_onnx.h"
using namespace std;

HEPONNX::HEPONNX(std::string filename)
{
  this->addonnx(filename);
};

HEPONNX::~HEPONNX()
{
  delete _env;
  delete _session;
};


void HEPONNX::addonnx(std::string filename)
{
    std::string fullfilename=std::string(HEPDATAINCLUDEDIR)+filename;

    this->_env = new Ort::Env();
    this->_session = new Ort::Session(*(this->_env),fullfilename.c_str(),Ort::SessionOptions{nullptr});
    //Ort::AllocatorWithDefaultOptions allocator;
  //Ort::Session session{env, "SUSY-2019-04_4jets.onnx", Ort::SessionOptions{nullptr}};

    const size_t num_input_nodes = _session->GetInputCount();

    if(num_input_nodes > 1)
    {
        cout << "For now, more than one input node is not supported! " << endl;

    }
 // std::vector<Ort::AllocatedStringPtr> input_names_ptr;
 // std::vector<const char*> input_node_names;
  input_names_ptr.reserve(num_input_nodes);
  input_node_names.reserve(num_input_nodes);
 

    input_tensor_size =1;


    for (size_t i = 0; i < num_input_nodes; i++) {
    // print input node names
    auto input_name = _session->GetInputNameAllocated(i, allocator);
    std::cout << "Input " << i << " : name =" << input_name.get() << std::endl;
    input_node_names.push_back(input_name.get());
    input_names_ptr.push_back(std::move(input_name));

    // print input node types
    auto type_info = _session->GetInputTypeInfo(i);
    auto tensor_info = type_info.GetTensorTypeAndShapeInfo();

    ONNXTensorElementDataType type = tensor_info.GetElementType();
    std::cout << "Input " << i << " : type = " << type << std::endl;

    // print input shapes/dims
    input_node_dims = tensor_info.GetShape();
    
    std::cout << "Input " << i << " : num_dims = " << input_node_dims.size() << '\n';
    for (size_t j = 0; j < input_node_dims.size(); j++) {
        input_tensor_size *=input_node_dims[j];
      std::cout << "Input " << i << " : dim[" << j << "] =" << input_node_dims[j] << '\n';
    }
    std::cout << std::flush;
  }

 //const size_t num_output_nodes = _session->GetOutputCount();
 num_output_nodes = _session->GetOutputCount();
    output_tensor_size = 1;

    if(num_output_nodes > 1)
    {
        cout << "More than one output node not yet supported! " << endl;
    }

//  std::vector<Ort::AllocatedStringPtr> output_names_ptr;
  //std::vector<const char*> output_node_names;
  output_names_ptr.reserve(num_output_nodes);
  output_node_names.reserve(num_output_nodes);
  std::vector<int64_t> output_node_dims;  // simplify... this model has only 1 input node {1, 3, 224, 224}.
                                         // Otherwise need vector<vector<>>

  std::cout << "Number of outputs = " << num_output_nodes << std::endl;

    
  
  // iterate over all output nodes
  for (size_t i = 0; i < num_output_nodes; i++) {
    // print input node names
    auto output_name = _session->GetOutputNameAllocated(i, allocator);
    std::cout << "Output " << i << " : name =" << output_name.get() << std::endl;
    output_node_names.push_back(output_name.get());
    output_names_ptr.push_back(std::move(output_name));

    // print input node types
    auto type_info = _session->GetOutputTypeInfo(i);
    auto tensor_info = type_info.GetTensorTypeAndShapeInfo();

    ONNXTensorElementDataType type = tensor_info.GetElementType();
    std::cout << "Output " << i << " : type = " << type << std::endl;

    // print input shapes/dims
    output_node_dims = tensor_info.GetShape();
    
    std::cout << "Output " << i << " : num_dims = " << output_node_dims.size() << '\n';
    for (size_t j = 0; j < output_node_dims.size(); j++) {
        output_tensor_size *=output_node_dims[j];
      std::cout << "output " << i << " : dim[" << j << "] =" << output_node_dims[j] << '\n';
    }
    std::cout << std::flush;
  }


    cout << "Found internal tensor size: " << input_tensor_size << endl;
/*

  // print model input layer (node names, types, shape etc.)
  Ort::AllocatorWithDefaultOptions allocator;

  // print number of model input nodes
  const size_t num_input_nodes = session.GetInputCount();
  std::vector<Ort::AllocatedStringPtr> input_names_ptr;
  std::vector<const char*> input_node_names;
  input_names_ptr.reserve(num_input_nodes);
  input_node_names.reserve(num_input_nodes);
  std::vector<int64_t> input_node_dims;  // simplify... this model has only 1 input node {1, 3, 224, 224}.
                                         // Otherwise need vector<vector<>>

  std::cout << "Number of inputs = " << num_input_nodes << std::endl;

  // iterate over all input nodes
  for (size_t i = 0; i < num_input_nodes; i++) {
    // print input node names
    auto input_name = session.GetInputNameAllocated(i, allocator);
    std::cout << "Input " << i << " : name =" << input_name.get() << std::endl;
    input_node_names.push_back(input_name.get());
    input_names_ptr.push_back(std::move(input_name));

    // print input node types
    auto type_info = session.GetInputTypeInfo(i);
    auto tensor_info = type_info.GetTensorTypeAndShapeInfo();

    ONNXTensorElementDataType type = tensor_info.GetElementType();
    std::cout << "Input " << i << " : type = " << type << std::endl;

    // print input shapes/dims
    input_node_dims = tensor_info.GetShape();
    std::cout << "Input " << i << " : num_dims = " << input_node_dims.size() << '\n';
    for (size_t j = 0; j < input_node_dims.size(); j++) {
      std::cout << "Input " << i << " : dim[" << j << "] =" << input_node_dims[j] << '\n';
    }
    std::cout << std::flush;
  }

  // print number of model input nodes
  const size_t num_output_nodes = session.GetOutputCount();
  std::vector<Ort::AllocatedStringPtr> output_names_ptr;
  std::vector<const char*> output_node_names;
  output_names_ptr.reserve(num_output_nodes);
  output_node_names.reserve(num_output_nodes);
  std::vector<int64_t> output_node_dims;  // simplify... this model has only 1 input node {1, 3, 224, 224}.
                                         // Otherwise need vector<vector<>>

  std::cout << "Number of outputs = " << num_output_nodes << std::endl;


  
  // iterate over all output nodes
  for (size_t i = 0; i < num_output_nodes; i++) {
    // print input node names
    auto output_name = session.GetOutputNameAllocated(i, allocator);
    std::cout << "Output " << i << " : name =" << output_name.get() << std::endl;
    output_node_names.push_back(output_name.get());
    output_names_ptr.push_back(std::move(output_name));

    // print input node types
    auto type_info = session.GetOutputTypeInfo(i);
    auto tensor_info = type_info.GetTensorTypeAndShapeInfo();

    ONNXTensorElementDataType type = tensor_info.GetElementType();
    std::cout << "Output " << i << " : type = " << type << std::endl;

    // print input shapes/dims
    output_node_dims = tensor_info.GetShape();
    std::cout << "Output " << i << " : num_dims = " << output_node_dims.size() << '\n';
    for (size_t j = 0; j < output_node_dims.size(); j++) {
      std::cout << "output " << i << " : dim[" << j << "] =" << output_node_dims[j] << '\n';
    }
    std::cout << std::flush;
  }

*/
}



 float HEPONNX::get_result(std::vector<double> &nninputs)
 {  
    // Take a flat vector of inputs and get the result from the network ...
 //std::vector<float> input_tensor_values(input_tensor_size);
  //  std::vector<const char*> output_node_names = {"39"};

  // initialize input data with values in [0.0, 1.0]
  //for (unsigned int i = 0; i < input_tensor_size; i++) input_tensor_values[i] = nn_inputs[i];

  // create input tensor object from data values
 // cout << "Starting get_result " << endl;
  auto memory_info = Ort::MemoryInfo::CreateCpu(OrtArenaAllocator, OrtMemTypeDefault);


  //   const size_t num_outpOrt::Value::CreateTensor(info, values, sizeof(values),dims.data(), dims.size(), ONNX_TENSOR_ELEMENT_DATA_TYPE_FLOAT16);

  //cout << "internal input tensor size: " << input_tensor_size << endl;
vector<float> nn_inputs;
for (auto el : nninputs)
{
    //cout << el << ",";
    nn_inputs.push_back(float(el));
    //nninputs.push_back(float(el));
}




//cout << endl << input_node_dims[0] << ", "<< input_node_dims[1] << ", " << ONNX_TENSOR_ELEMENT_DATA_TYPE_FLOAT << ", " << input_tensor_size << endl;
 //   const size_t num_outpOrt::Value::CreateTensor(info, values, sizeof(values),dims.data(), dims.size(), ONNX_TENSOR_ELEMENT_DATA_TYPE_FLOAT16);
  //auto input_tensor = Ort::Value::CreateTensor<double>(memory_info, nn_inputs.data(), input_tensor_size, input_node_dims.data(), 2);
 // auto input_tensor = Ort::Value::CreateTensor<double>(memory_info, nn_inputs.data(), input_tensor_size, input_node_dims.data(), ONNX_TENSOR_ELEMENT_DATA_TYPE_DOUBLE);

 //static Value CreateTensor(const OrtMemoryInfo* info, T* p_data, size_t p_data_element_count, const int64_t* shape, size_t shape_len);
 //static Value CreateTensor(const OrtMemoryInfo* info, void* p_data, size_t p_data_byte_count, const int64_t* shape, size_t shape_len,
  //                          ONNXTensorElementDataType type);
 auto input_tensor = Ort::Value::CreateTensor<float>(memory_info, nn_inputs.data(), input_tensor_size, input_node_dims.data(), input_node_dims.size());

  //cout << "Made input_tensor " << endl;
    auto output_tensors = _session->Run(Ort::RunOptions{nullptr}, input_node_names.data(), &input_tensor, 1, output_node_names.data(), 1);

  //cout << "made output tensors"<<endl;
  // assert(output_tensors.size() == 1 && output_tensors.front().IsTensor());

  // Get pointer to output tensor float values
  //double* floatarr = output_tensors.front().GetTensorMutableData<double>();
 float* floatarr = output_tensors.front().GetTensorMutableData<float>();
  
  //assert(abs(floatarr[0] - 0.000045) < 1e-6);

  // score the model, and print scores for first 5 classes

  /*
  for (int i = 0; i < 1; i++) {
    std::cout << "Score for class [" << i << "] =  " << floatarr[i] << '\n';
  }
  std::cout << std::flush;
  */

    return floatarr[0];


 }



#endif
