/*

Routines for computing various kinematic variables of use

*/

#include "include/kinematics.h"


/* 
double mct_330   (const LorentzTransverseVector& a, 
		    const LorentzTransverseVector& b) {
    const double mctsq = mct_330_Sq(a, b);
    // if less than zero, this is only evidence of inexact maths:
    if (mctsq<=0) {
      return 0;
    } else {
      return sqrt(mctsq);
    }
  }
  
  double mct_330_Sq(const LorentzTransverseVector& a,  
		    const LorentzTransverseVector& b) {
    return a.masssq() + b.masssq() + 2.0*a.contralinearDot(b);
  }

*/


namespace HEP {
  

/*
double mTcalc(MALorentzVector &v1, MALorentzVector &v2)
{
  //  m_T^2(p_1, p_2) =& m_1^2 + m_2^2 + 2 m_T^{(1)} m_T^{(2)} - 2 |p_T^{(1)}| |p_T^{(2)}| \cos \phi_{12} 

  
  
  return sqrt(v1.M2()+v2.M2() + 2.0*mT(v1)*mT(v2) - 2.0*v1.Pt()*v2.Pt()*cos(v1.DeltaPhi(v2)));

};
*/

/*
// we only use mTcalc for leptonp and pTmiss so fudge it
double mTcalc(MALorentzVector &v1, MALorentzVector &v2)
{
  //  m_T^2(p_1, p_2) =& m_1^2 + m_2^2 + 2 m_T^{(1)} m_T^{(2)} - 2 |p_T^{(1)}| >
  return sqrt(v1.M2() + 2.0*mT(v1)*v2.Pt() - 2.0*v1.Pt()*v2.Pt()*cos(v1.DeltaPhi(v2)));


};
*/

/*   mT is defined in Vectors.h
double mT(MALorentzVector &v1)
{
  return sqrt(v1.M2() + v1.Perp2());
}
*/


double mCT2(const HEP::P4 &v1, const HEP::P4 &v2)
{
  //  m_{CT}^2(p_1, p_2) =& m_1^2 + m_2^2 + 2 m_T^{(1)} m_T^{(2)} + 2 |p_T^{(1)}| |p_T^{(2)}| \cos \phi_{12} 
    // 0910.0174
    // mCT^2 = m1^2 + m2^2 + 2ET1 ET2 + 2 pT1 cdot pT2
    // ET^2 = m^2 + pT^2 -> our mT2() in Vectors.h
  return v1.m2()+v2.m2() + 2.0*v1.mT()*v2.mT() + 2.0*v1.pT()*v2.pT()*cos(v1.deltaPhi(v2));

};

double mCT(const HEP::P4 &v1, const HEP::P4 &v2)
{
  double res = mCT2(v1,v2);
  if (res < 0)
  {
    return 0.0;
  }
  else
  {
    return sqrt(res);
  }


}





};
